
# 13 "parsing/g_constr.mlg"
 

open Names
open Constr
open Libnames
open Glob_term
open Constrexpr
open Constrexpr_ops
open Util

open Procq
open Procq.Prim
open Procq.Constr

(* TODO: avoid this redefinition without an extra dep to Notation_ops *)
let ldots_var = Id.of_string ".."

let binder_of_name expl { CAst.loc = loc; v = na } =
  CLocalAssum ([CAst.make ?loc na], None, Default expl,
    CAst.make ?loc @@ CHole (Some (GBinderType na)))

let binders_of_names l =
  List.map (binder_of_name Explicit) l

let pat_of_name CAst.{loc;v} = match v with
| Anonymous -> CAst.make ?loc @@ CPatAtom None
| Name id -> CAst.make ?loc @@ CPatAtom (Some (qualid_of_ident id))

(* Hack to parse "(x:=t)" as an explicit argument without conflicts with the *)
(* admissible notation "(x t)" *)
let test_lpar_id_coloneq =
  let open Procq.Lookahead in
  to_entry "test_lpar_id_coloneq" begin
    lk_kw "(" >> lk_ident >> lk_kw ":="
  end

(* Hack to parse "(n:=t)" as an explicit argument without conflicts with the *)
(* syntax "(n t)" *)
let test_lpar_nat_coloneq =
  let open Procq.Lookahead in
  to_entry "test_lpar_id_coloneq" begin
    lk_kw "(" >> lk_nat >> lk_kw ":="
  end

let ensure_fixannot =
  let open Procq.Lookahead in
  to_entry "check_fixannot" begin
    lk_kw "{" >> lk_kws ["wf"; "struct"; "measure"]
  end

let test_name_colon =
  let open Procq.Lookahead in
  to_entry "test_name_colon" begin
    lk_name >> lk_kw ":"
  end

let aliasvar = function { CAst.v = CPatAlias (_, na) } -> Some na | _ -> None

let test_array_opening =
  let open Procq.Lookahead in
  to_entry "test_array_opening" begin
    lk_kw "[" >> lk_kw "|" >> check_no_space
  end

let test_array_closing =
  let open Procq.Lookahead in
  to_entry "test_array_closing" begin
    lk_kw "|" >> lk_kw "]" >> check_no_space
  end

let test_sort_qvar =
  let open Procq.Lookahead in
  to_entry "test_sort_qvar" begin
    lk_ident >> lk_list lk_field >> lk_kw "|"
  end

type univ_level_or_quality =
  | SProp | Prop | Set | Type | Anon of Loc.t | Global of Libnames.qualid

let force_univ_level ?loc = function
  | SProp -> UNamed CSProp
  | Prop -> UNamed CProp
  | Set -> UNamed CSet
  | Type -> UAnonymous {rigid=UnivRigid}
  | Anon _ -> UAnonymous {rigid=UnivFlexible false}
  | Global qid -> UNamed (CType qid)

let force_quality ?loc = function
  | SProp -> CQConstant QSProp
  | Prop -> CQConstant QProp
  | Set -> CErrors.user_err ?loc Pp.(str "Universe levels cannot be Set.")
  | Type -> CQConstant QType
  | Anon loc -> CQualVar (CQAnon (Some loc))
  | Global qid -> CQualVar (CQVar qid)

(* XXX use registered ref? but currently constrexpr doesn't have a node for registered refs
   and we can't do [Rocqlib.lib_ref] at parsing time, it's only available in the interp phase. *)
let sigref loc = mkRefC (Libnames.qualid_of_string ~loc "Corelib.Init.Specif.sig")



let _ =
  let universe_increment = Procq.Entry.make "universe_increment"
  and universe_expr = Procq.Entry.make "universe_expr"
  and universe = Procq.Entry.make "universe"
  and array_elems = Procq.Entry.make "array_elems"
  and fields_def = Procq.Entry.make "fields_def"
  and field_def = Procq.Entry.make "field_def"
  and atomic_constr = Procq.Entry.make "atomic_constr"
  and inst = Procq.Entry.make "inst"
  and evar_instance = Procq.Entry.make "evar_instance"
  and univ_annot = Procq.Entry.make "univ_annot"
  and univ_level_or_quality = Procq.Entry.make "univ_level_or_quality"
  and fix_decls = Procq.Entry.make "fix_decls"
  and cofix_decls = Procq.Entry.make "cofix_decls"
  and fix_decl = Procq.Entry.make "fix_decl"
  and cofix_body = Procq.Entry.make "cofix_body"
  and term_match = Procq.Entry.make "term_match"
  and case_item = Procq.Entry.make "case_item"
  and case_type = Procq.Entry.make "case_type"
  and as_return_type = Procq.Entry.make "as_return_type"
  and branches = Procq.Entry.make "branches"
  and mult_pattern = Procq.Entry.make "mult_pattern"
  and eqn = Procq.Entry.make "eqn"
  and record_pattern = Procq.Entry.make "record_pattern"
  and record_patterns = Procq.Entry.make "record_patterns"
  and fixannot = Procq.Entry.make "fixannot"
  and let_type_cstr = Procq.Entry.make "let_type_cstr"
  in
  let () = assert (Procq.Entry.is_empty Constr.ident) in
  let () =
  Procq.grammar_extend Constr.ident
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm Prim.ident)))
                            (fun id loc -> 
# 122 "parsing/g_constr.mlg"
                             id 
                                           )])]))
  in let () =
  Procq.grammar_extend Prim.name
  (Procq.Reuse (None, [Procq.Production.make
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PKEYWORD ("_")))))
                       (fun _ loc -> 
# 125 "parsing/g_constr.mlg"
                 CAst.make ~loc Anonymous 
                                     )]))
  in let () = assert (Procq.Entry.is_empty global) in
  let () =
  Procq.grammar_extend global
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm Prim.reference)))
                            (fun r loc -> 
# 128 "parsing/g_constr.mlg"
                                r 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty constr_pattern) in
  let () =
  Procq.grammar_extend constr_pattern
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm constr)))
                            (fun c loc -> 
# 131 "parsing/g_constr.mlg"
                        c 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty cpattern) in
  let () =
  Procq.grammar_extend cpattern
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm lconstr)))
                            (fun c loc -> 
# 134 "parsing/g_constr.mlg"
                         c 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty sort) in
  let () =
  Procq.grammar_extend sort
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PKEYWORD ("Type")))))
                               ((Procq.Symbol.token (Tok.PKEYWORD ("@{")))))
                              ((Procq.Symbol.nterm universe)))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                            (fun _ u _ _ loc -> 
# 144 "parsing/g_constr.mlg"
                                             None, u 
                                                );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next (Procq.Rule.stop)
                                  ((Procq.Symbol.token (Tok.PKEYWORD ("Type")))))
                                 ((Procq.Symbol.token (Tok.PKEYWORD ("@{")))))
                                ((Procq.Symbol.nterm test_sort_qvar)))
                               ((Procq.Symbol.nterm reference)))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("|")))))
                             ((Procq.Symbol.nterm universe)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                           (fun _ u _ q _ _ _ loc -> 
# 143 "parsing/g_constr.mlg"
          Some (CQVar q), u 
                                                     );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("Type")))))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("@{")))))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("_")))))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                           (fun _ _ _ _ loc -> 
# 141 "parsing/g_constr.mlg"
                                    None, UAnonymous {rigid=UnivFlexible false} 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("Type")))))
                           (fun _ loc -> 
# 140 "parsing/g_constr.mlg"
                    None, UAnonymous {rigid=UnivRigid} 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("SProp")))))
                           (fun _ loc -> 
# 139 "parsing/g_constr.mlg"
                     None, UNamed [CSProp, 0] 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("Prop")))))
                           (fun _ loc -> 
# 138 "parsing/g_constr.mlg"
                    None, UNamed [CProp, 0] 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("Set")))))
                           (fun _ loc -> 
# 137 "parsing/g_constr.mlg"
                    None, UNamed [CSet, 0] 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty sort_family) in
  let () =
  Procq.grammar_extend sort_family
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("Type")))))
                            (fun _ loc -> 
# 150 "parsing/g_constr.mlg"
                    Sorts.InType 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("SProp")))))
                           (fun _ loc -> 
# 149 "parsing/g_constr.mlg"
                     Sorts.InSProp 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("Prop")))))
                           (fun _ loc -> 
# 148 "parsing/g_constr.mlg"
                    Sorts.InProp 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("Set")))))
                           (fun _ loc -> 
# 147 "parsing/g_constr.mlg"
                    Sorts.InSet 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty universe_increment) in
  let () =
  Procq.grammar_extend universe_increment
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 154 "parsing/g_constr.mlg"
             0 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("+")))))
                            ((Procq.Symbol.nterm natural)))
                           (fun n _ loc -> 
# 153 "parsing/g_constr.mlg"
                              n 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty universe_name) in
  let () =
  Procq.grammar_extend universe_name
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("Prop")))))
                            (fun _ loc -> 
# 159 "parsing/g_constr.mlg"
                    CProp 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("Set")))))
                           (fun _ loc -> 
# 158 "parsing/g_constr.mlg"
                    CSet 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm global)))
                           (fun id loc -> 
# 157 "parsing/g_constr.mlg"
                         CType id 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty universe_expr) in
  let () =
  Procq.grammar_extend universe_expr
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm universe_name)))
                             ((Procq.Symbol.nterm universe_increment)))
                            (fun n id loc -> 
# 162 "parsing/g_constr.mlg"
                                                        (id,n) 
                                             )])]))
  in let () = assert (Procq.Entry.is_empty universe) in
  let () =
  Procq.grammar_extend universe
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm universe_expr)))
                            (fun u loc -> 
# 167 "parsing/g_constr.mlg"
                               UNamed [u] 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("_")))))
                           (fun _ loc -> 
# 166 "parsing/g_constr.mlg"
                 UAnonymous { rigid = UnivFlexible false } 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("max"))))))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                             ((Procq.Symbol.list1sep ((Procq.Symbol.nterm universe_expr)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                           (fun _ ids _ _ loc -> 
# 165 "parsing/g_constr.mlg"
                                                                      UNamed ids 
                                                 )])]))
  in let () = assert (Procq.Entry.is_empty lconstr) in
  let () =
  Procq.grammar_extend lconstr
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterml term ("200"))))
                            (fun c loc -> 
# 170 "parsing/g_constr.mlg"
                                  c 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty constr) in
  let () =
  Procq.grammar_extend constr
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("@")))))
                              ((Procq.Symbol.nterm global)))
                             ((Procq.Symbol.nterm univ_annot)))
                            (fun i f _ loc -> 
# 174 "parsing/g_constr.mlg"
                                           CAst.make ~loc @@ CAppExpl((f,i),[]) 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterml term ("8"))))
                           (fun c loc -> 
# 173 "parsing/g_constr.mlg"
                                c 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty term) in
  let () =
  Procq.grammar_extend term
  (Procq.Fresh
  (Gramlib.Gramext.First, [(Some ("200"), Some (Gramlib.Gramext.RightA), []);
                          (Some ("100"), Some (Gramlib.Gramext.RightA),
                          [Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm term)))
                             ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                            ((Procq.Symbol.nterml term ("200"))))
                           (fun c2 _ c1 loc -> 
# 186 "parsing/g_constr.mlg"
                   CAst.make ~loc @@ CCast(c1, Some DEFAULTcast, c2) 
                                               );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm term)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (":>")))))
                           ((Procq.Symbol.nterml term ("200"))))
                          (fun c2 _ c1 loc -> 
# 184 "parsing/g_constr.mlg"
                   CAst.make ~loc @@ CCast(c1, None, c2) 
                                              );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm term)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("<<:")))))
                           ((Procq.Symbol.nterml term ("200"))))
                          (fun c2 _ c1 loc -> 
# 182 "parsing/g_constr.mlg"
                   CAst.make ~loc @@ CCast(c1, Some NATIVEcast, c2) 
                                              );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm term)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("<:")))))
                           ((Procq.Symbol.nterml term ("200"))))
                          (fun c2 _ c1 loc -> 
# 180 "parsing/g_constr.mlg"
                   CAst.make ~loc @@ CCast(c1, Some VMcast, c2) 
                                              )]);
                          (Some ("99"), Some (Gramlib.Gramext.RightA), []);
                          (Some ("90"), Some (Gramlib.Gramext.RightA), []);
                          (Some ("10"), Some (Gramlib.Gramext.LeftA),
                          [Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm binder_constr)))
                           (fun c loc -> 
# 196 "parsing/g_constr.mlg"
                               c 
                                         );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("@")))))
                            ((Procq.Symbol.nterm pattern_ident)))
                           ((Procq.Symbol.list1 ((Procq.Symbol.nterm identref)))))
                          (fun args lid _ loc -> 
# 193 "parsing/g_constr.mlg"
          let { CAst.loc = locid; v = id } = lid in
          let args = List.map (fun x -> CAst.make @@ CRef (qualid_of_ident ?loc:x.CAst.loc x.CAst.v, None), None) args in
          CAst.make ~loc @@ CApp(CAst.make ?loc:locid @@ CPatVar id,args) 
                                                 );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("@")))))
                             ((Procq.Symbol.nterm global)))
                            ((Procq.Symbol.nterm univ_annot)))
                           ((Procq.Symbol.list0 Procq.Symbol.next)))
                          (fun args i f _ loc -> 
# 191 "parsing/g_constr.mlg"
                                                                CAst.make ~loc @@ CAppExpl((f,i),args) 
                                                 );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm term)))
                           ((Procq.Symbol.list1 ((Procq.Symbol.nterm arg)))))
                          (fun args f loc -> 
# 190 "parsing/g_constr.mlg"
                                        CAst.make ~loc @@ CApp(f,args) 
                                             )]);
                          (Some ("9"), None,
                          [Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("..")))))
                             ((Procq.Symbol.nterml term ("0"))))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("..")))))
                           (fun _ c _ loc -> 
# 199 "parsing/g_constr.mlg"
          CAst.make ~loc @@ CAppExpl ((qualid_of_ident ~loc ldots_var, None),[c]) 
                                             )]);
                          (Some ("8"), None, []);
                          (Some ("1"), Some (Gramlib.Gramext.LeftA),
                          [Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm term)))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("%_")))))
                            ((Procq.Symbol.token (Tok.PIDENT (None)))))
                           (fun key _ c loc -> 
# 210 "parsing/g_constr.mlg"
          CAst.make ~loc @@ CDelimiters (DelimOnlyTmpScope,key,c) 
                                               );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm term)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("%")))))
                           ((Procq.Symbol.token (Tok.PIDENT (None)))))
                          (fun key _ c loc -> 
# 208 "parsing/g_constr.mlg"
          CAst.make ~loc @@ CDelimiters (DelimUnboundedScope,key,c) 
                                              );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.nterm term)))
                                ((Procq.Symbol.token (Tok.PKEYWORD (".(")))))
                               ((Procq.Symbol.token (Tok.PKEYWORD ("@")))))
                              ((Procq.Symbol.nterm global)))
                             ((Procq.Symbol.nterm univ_annot)))
                            ((Procq.Symbol.list0 (Procq.Symbol.nterml term ("9")))))
                           ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                          (fun _ args i f _ _ c loc -> 
# 206 "parsing/g_constr.mlg"
          CAst.make ~loc @@ CProj(true, (f,i), List.map (fun a -> (a,None)) args, c) 
                                                       );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.nterm term)))
                               ((Procq.Symbol.token (Tok.PKEYWORD (".(")))))
                              ((Procq.Symbol.nterm global)))
                             ((Procq.Symbol.nterm univ_annot)))
                            ((Procq.Symbol.list0 (Procq.Symbol.nterm arg))))
                           ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                          (fun _ args i f _ c loc -> 
# 203 "parsing/g_constr.mlg"
          CAst.make ~loc @@ CProj(false, (f,i), args, c) 
                                                     )]);
                          (Some ("0"), None,
                          [Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("`(")))))
                             ((Procq.Symbol.nterml term ("200"))))
                            ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                           (fun _ c _ loc -> 
# 234 "parsing/g_constr.mlg"
          CAst.make ~loc @@ CGeneralization (Explicit, c) 
                                             );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next
                                  (Procq.Rule.next
                                   (Procq.Rule.next
                                    (Procq.Rule.next (Procq.Rule.stop)
                                     ((Procq.Symbol.nterm test_array_opening)))
                                    ((Procq.Symbol.token (Tok.PKEYWORD ("[")))))
                                   ((Procq.Symbol.token (Tok.PKEYWORD ("|")))))
                                  ((Procq.Symbol.nterm array_elems)))
                                 ((Procq.Symbol.token (Tok.PKEYWORD ("|")))))
                                ((Procq.Symbol.nterm lconstr)))
                               ((Procq.Symbol.nterm type_cstr)))
                              ((Procq.Symbol.nterm test_array_closing)))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("|")))))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("]")))))
                           ((Procq.Symbol.nterm univ_annot)))
                          (fun u _ _ _ ty def _ ls _ _ _ loc -> 
# 229 "parsing/g_constr.mlg"
          let t = Array.make (List.length ls) def in
          List.iteri (fun i e -> t.(i) <- e) ls;
          CAst.make ~loc @@ CArray(u, t, def, ty)
        
                                                                );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("`{")))))
                            ((Procq.Symbol.nterml term ("200"))))
                           ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                          (fun _ c _ loc -> 
# 227 "parsing/g_constr.mlg"
          CAst.make ~loc @@ CGeneralization (MaxImplicit, c) 
                                            );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("{|")))))
                            ((Procq.Symbol.nterm record_declaration)))
                           ((Procq.Symbol.nterm bar_cbrace)))
                          (fun _ c _ loc -> 
# 225 "parsing/g_constr.mlg"
                                                      c 
                                            );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                            ((Procq.Symbol.nterml term ("200"))))
                           ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                          (fun _ c _ loc -> 
# 219 "parsing/g_constr.mlg"
          (* Preserve parentheses around numbers so that constrintern does not
             collapse -(3) into the number -3. *)
          (match c.CAst.v with
            | CPrim (Number (NumTok.SPlus,n)) ->
                CAst.make ~loc @@ CNotation(None,(InConstrEntry,"( _ )"),([c],[],[],[]))
            | _ -> c) 
                                            );
                          Procq.Production.make
                          (Procq.Rule.next (Procq.Rule.stop)
                           ((Procq.Symbol.nterm string)))
                          (fun s loc -> 
# 217 "parsing/g_constr.mlg"
                        CAst.make ~loc @@ CPrim (String s) 
                                        );
                          Procq.Production.make
                          (Procq.Rule.next (Procq.Rule.stop)
                           ((Procq.Symbol.token (Tok.PNUMBER None))))
                          (fun n loc -> 
# 216 "parsing/g_constr.mlg"
                       CAst.make ~loc @@ CPrim (Number (NumTok.SPlus,n)) 
                                        );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm reference)))
                           ((Procq.Symbol.nterm univ_annot)))
                          (fun i id loc -> 
# 215 "parsing/g_constr.mlg"
          CAst.make ~loc @@ CRef (id, i) 
                                           );
                          Procq.Production.make
                          (Procq.Rule.next (Procq.Rule.stop)
                           ((Procq.Symbol.nterm term_match)))
                          (fun c loc -> 
# 213 "parsing/g_constr.mlg"
                            c 
                                        );
                          Procq.Production.make
                          (Procq.Rule.next (Procq.Rule.stop)
                           ((Procq.Symbol.nterm atomic_constr)))
                          (fun c loc -> 
# 212 "parsing/g_constr.mlg"
                               c 
                                        )])]))
  in let () = assert (Procq.Entry.is_empty array_elems) in
  let () =
  Procq.grammar_extend array_elems
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.list0sep ((Procq.Symbol.nterm lconstr)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (";"))])) false)))
                            (fun fs loc -> 
# 237 "parsing/g_constr.mlg"
                                        fs 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty record_declaration) in
  let () =
  Procq.grammar_extend record_declaration
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm fields_def)))
                            (fun fs loc -> 
# 240 "parsing/g_constr.mlg"
                             CAst.make ~loc @@ CRecord fs 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty fields_def) in
  let () =
  Procq.grammar_extend fields_def
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 245 "parsing/g_constr.mlg"
             [] 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm field_def)))
                           (fun f loc -> 
# 244 "parsing/g_constr.mlg"
                           [f] 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm field_def)))
                             ((Procq.Symbol.token (Tok.PKEYWORD (";")))))
                            ((Procq.Symbol.nterm fields_def)))
                           (fun fs _ f loc -> 
# 243 "parsing/g_constr.mlg"
                                                 f :: fs 
                                              )])]))
  in let () = assert (Procq.Entry.is_empty field_def) in
  let () =
  Procq.grammar_extend field_def
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.nterm global)))
                               ((Procq.Symbol.nterm binders)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                             ((Procq.Symbol.nterm lconstr)))
                            (fun c _ bl id loc -> 
# 249 "parsing/g_constr.mlg"
          (id, mkLambdaCN ~loc bl c) 
                                                  )])]))
  in let () = assert (Procq.Entry.is_empty binder_constr) in
  let () =
  Procq.grammar_extend binder_constr
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("cofix")))))
                             ((Procq.Symbol.nterm cofix_decls)))
                            (fun c _ loc -> 
# 293 "parsing/g_constr.mlg"
                                      let (id,dcls) = c in CAst.make ~loc @@ CCoFix (id,dcls) 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("fix")))))
                            ((Procq.Symbol.nterm fix_decls)))
                           (fun c _ loc -> 
# 292 "parsing/g_constr.mlg"
                                  let (id,dcls) = c in CAst.make ~loc @@ CFix (id,dcls) 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next (Procq.Rule.stop)
                                  ((Procq.Symbol.token (Tok.PKEYWORD ("if")))))
                                 ((Procq.Symbol.nterml term ("200"))))
                                ((Procq.Symbol.nterm as_return_type)))
                               ((Procq.Symbol.token (Tok.PKEYWORD ("then")))))
                              ((Procq.Symbol.nterml term ("200"))))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("else")))))
                            ((Procq.Symbol.nterml term ("200"))))
                           (fun b2 _ b1 _ po c _ loc -> 
# 291 "parsing/g_constr.mlg"
          CAst.make ~loc @@ CIf (c, po, b1, b2) 
                                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next
                                  (Procq.Rule.next
                                   (Procq.Rule.next
                                    (Procq.Rule.next (Procq.Rule.stop)
                                     ((Procq.Symbol.token (Tok.PKEYWORD ("let")))))
                                    ((Procq.Symbol.token (Tok.PKEYWORD ("'")))))
                                   ((Procq.Symbol.nterml pattern ("200"))))
                                  ((Procq.Symbol.token (Tok.PKEYWORD ("in")))))
                                 ((Procq.Symbol.nterml pattern ("200"))))
                                ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                               ((Procq.Symbol.nterml term ("200"))))
                              ((Procq.Symbol.nterm case_type)))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("in")))))
                            ((Procq.Symbol.nterml term ("200"))))
                           (fun c2 _ rt c1 _ t _ p _ _ loc -> 
# 286 "parsing/g_constr.mlg"
          CAst.make ~loc @@
          CCases (LetPatternStyle, Some rt, [c1, aliasvar p, Some t], [CAst.make ~loc ([[p]], c2)]) 
                                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next
                                  (Procq.Rule.next (Procq.Rule.stop)
                                   ((Procq.Symbol.token (Tok.PKEYWORD ("let")))))
                                  ((Procq.Symbol.token (Tok.PKEYWORD ("'")))))
                                 ((Procq.Symbol.nterml pattern ("200"))))
                                ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                               ((Procq.Symbol.nterml term ("200"))))
                              ((Procq.Symbol.nterm case_type)))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("in")))))
                            ((Procq.Symbol.nterml term ("200"))))
                           (fun c2 _ rt c1 _ p _ _ loc -> 
# 281 "parsing/g_constr.mlg"
          CAst.make ~loc @@
          CCases (LetPatternStyle, Some rt, [c1, aliasvar p, None], [CAst.make ~loc ([[p]], c2)]) 
                                                          );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next (Procq.Rule.stop)
                                  ((Procq.Symbol.token (Tok.PKEYWORD ("let")))))
                                 ((Procq.Symbol.token (Tok.PKEYWORD ("'")))))
                                ((Procq.Symbol.nterml pattern ("200"))))
                               ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                              ((Procq.Symbol.nterml term ("200"))))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("in")))))
                            ((Procq.Symbol.nterml term ("200"))))
                           (fun c2 _ c1 _ p _ _ loc -> 
# 277 "parsing/g_constr.mlg"
          CAst.make ~loc @@
          CCases (LetPatternStyle, None,    [c1, None, None],       [CAst.make ~loc ([[p]], c2)]) 
                                                       );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next (Procq.Rule.stop)
                                  ((Procq.Symbol.token (Tok.PKEYWORD ("let")))))
                                 ((Procq.Symbol.rules [Procq.Rules.make 
                                                      (Procq.Rule.next_norec
                                                       (Procq.Rule.stop)
                                                       ((Procq.Symbol.token (Tok.PKEYWORD ("()")))))
                                                      (fun _ loc -> 
# 271 "parsing/g_constr.mlg"
                                                                         [] 
                                                                    );
                                                      Procq.Rules.make 
                                                      (Procq.Rule.next_norec
                                                       (Procq.Rule.next_norec
                                                        (Procq.Rule.next_norec
                                                         (Procq.Rule.stop)
                                                         ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                        ((Procq.Symbol.list0sep ((Procq.Symbol.nterm name)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                                                       ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                                                      (fun _ l _ loc -> 
                                                      
# 271 "parsing/g_constr.mlg"
                                                         l 
                                                      )])))
                                ((Procq.Symbol.nterm as_return_type)))
                               ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                              ((Procq.Symbol.nterml term ("200"))))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("in")))))
                            ((Procq.Symbol.nterml term ("200"))))
                           (fun c2 _ c1 _ po lb _ loc -> 
# 274 "parsing/g_constr.mlg"
          CAst.make ~loc @@ CLetTuple (lb,po,c1,c2) 
                                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PKEYWORD ("let")))))
                               ((Procq.Symbol.token (Tok.PKEYWORD ("cofix")))))
                              ((Procq.Symbol.nterm cofix_body)))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("in")))))
                            ((Procq.Symbol.nterml term ("200"))))
                           (fun c _ fx _ _ loc -> 
# 268 "parsing/g_constr.mlg"
          let {CAst.loc=locf;CAst.v=({CAst.loc=li;CAst.v=id} as lid,_,_,_,_ as dcl)} = fx in
          let cofix = CAst.make ?loc:locf @@ CCoFix (lid,[dcl]) in
          CAst.make ~loc @@ CLetIn( CAst.make ?loc:li @@ Name id,cofix,None,c) 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PKEYWORD ("let")))))
                               ((Procq.Symbol.token (Tok.PKEYWORD ("fix")))))
                              ((Procq.Symbol.nterm fix_decl)))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("in")))))
                            ((Procq.Symbol.nterml term ("200"))))
                           (fun c _ fx _ _ loc -> 
# 264 "parsing/g_constr.mlg"
          let {CAst.loc=locf;CAst.v=({CAst.loc=li;CAst.v=id} as lid,_,_,_,_,_ as dcl)} = fx in
          let fix = CAst.make ?loc:locf @@ CFix (lid,[dcl]) in
          CAst.make ~loc @@ CLetIn( CAst.make ?loc:li @@ Name id,fix,None,c) 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next
                                  (Procq.Rule.next (Procq.Rule.stop)
                                   ((Procq.Symbol.token (Tok.PKEYWORD ("let")))))
                                  ((Procq.Symbol.nterm name)))
                                 ((Procq.Symbol.nterm binders)))
                                ((Procq.Symbol.nterm let_type_cstr)))
                               ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                              ((Procq.Symbol.nterml term ("200"))))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("in")))))
                            ((Procq.Symbol.nterml term ("200"))))
                           (fun c2 _ c1 _ ty bl id _ loc -> 
# 258 "parsing/g_constr.mlg"
          let ty,c1 = match ty, c1 with
          | (_,None), { CAst.v = CCast(c, Some DEFAULTcast, t) } -> (Loc.tag ?loc:(constr_loc t) @@ Some t), c (* Tolerance, see G_vernac.def_body *)
          | _, _ -> ty, c1 in
          CAst.make ~loc @@ CLetIn(id,mkLambdaCN ?loc:(constr_loc c1) bl c1,
                 Option.map (mkProdCN ?loc:(fst ty) bl) (snd ty), c2) 
                                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("fun")))))
                              ((Procq.Symbol.nterm open_binders)))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("=>")))))
                            ((Procq.Symbol.nterml term ("200"))))
                           (fun c _ bl _ loc -> 
# 255 "parsing/g_constr.mlg"
          mkLambdaCN ~loc bl c 
                                                );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("forall")))))
                              ((Procq.Symbol.nterm open_binders)))
                             ((Procq.Symbol.token (Tok.PKEYWORD (",")))))
                            ((Procq.Symbol.nterml term ("200"))))
                           (fun c _ bl _ loc -> 
# 253 "parsing/g_constr.mlg"
          mkProdCN ~loc bl c 
                                                )])]))
  in let () = assert (Procq.Entry.is_empty arg) in
  let () =
  Procq.grammar_extend arg
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterml term ("9"))))
                            (fun c loc -> 
# 298 "parsing/g_constr.mlg"
                              (c,None) 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.nterm test_lpar_nat_coloneq)))
                                ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                               ((Procq.Symbol.nterm natural)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                             ((Procq.Symbol.nterm lconstr)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                           (fun _ c _ n _ _ loc -> 
# 297 "parsing/g_constr.mlg"
                                                                             (c,Some (CAst.make ~loc @@ ExplByPos n)) 
                                                   );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.nterm test_lpar_id_coloneq)))
                                ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                               ((Procq.Symbol.nterm identref)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                             ((Procq.Symbol.nterm lconstr)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                           (fun _ c _ id _ _ loc -> 
# 296 "parsing/g_constr.mlg"
                                                                              (c,Some (CAst.make ?loc:id.CAst.loc @@ ExplByName id.CAst.v)) 
                                                    )])]))
  in let () = assert (Procq.Entry.is_empty atomic_constr) in
  let () =
  Procq.grammar_extend atomic_constr
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm pattern_ident)))
                             ((Procq.Symbol.nterm evar_instance)))
                            (fun inst id loc -> 
# 305 "parsing/g_constr.mlg"
                                                      CAst.make ~loc @@ CEvar(id,inst) 
                                                );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("?")))))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("[")))))
                             ((Procq.Symbol.nterm pattern_ident)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("]")))))
                           (fun _ id _ _ loc -> 
# 304 "parsing/g_constr.mlg"
                                                CAst.make ~loc @@  CHole (Some (GNamedHole (true, id.CAst.v))) 
                                                );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("?")))))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("[")))))
                             ((Procq.Symbol.nterm identref)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("]")))))
                           (fun _ id _ _ loc -> 
# 303 "parsing/g_constr.mlg"
                                           CAst.make ~loc @@  CHole (Some (GNamedHole (false, id.CAst.v))) 
                                                );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("_")))))
                           (fun _ loc -> 
# 302 "parsing/g_constr.mlg"
                      CAst.make ~loc @@ CHole (None) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm sort)))
                           (fun s loc -> 
# 301 "parsing/g_constr.mlg"
                        CAst.make ~loc @@ CSort s 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty inst) in
  let () =
  Procq.grammar_extend inst
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.nterm identref)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                             ((Procq.Symbol.nterm lconstr)))
                            (fun c _ id loc -> 
# 308 "parsing/g_constr.mlg"
                                              (id,c) 
                                               )])]))
  in let () = assert (Procq.Entry.is_empty evar_instance) in
  let () =
  Procq.grammar_extend evar_instance
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 312 "parsing/g_constr.mlg"
             [] 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("@{")))))
                             ((Procq.Symbol.list1sep ((Procq.Symbol.nterm inst)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (";"))])) false)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                           (fun _ l _ loc -> 
# 311 "parsing/g_constr.mlg"
                                               l 
                                             )])]))
  in let () = assert (Procq.Entry.is_empty univ_annot) in
  let () =
  Procq.grammar_extend univ_annot
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 323 "parsing/g_constr.mlg"
             None 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("@{")))))
                              ((Procq.Symbol.list0 (Procq.Symbol.nterm univ_level_or_quality))))
                             ((Procq.Symbol.opt (Procq.Symbol.rules [Procq.Rules.make 
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("|")))))
                                                                    ((Procq.Symbol.list0 (Procq.Symbol.nterm univ_level_or_quality))))
                                                                    (fun l' _
                                                                    loc -> 
                                                                    
# 315 "parsing/g_constr.mlg"
                                                                                                     l' 
                                                                    )]))))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                           (fun _ l' l _ loc -> 
# 316 "parsing/g_constr.mlg"
         
          let l, l' = match l' with
            | None -> [], List.map (force_univ_level ~loc) l
            | Some l' -> List.map (force_quality ~loc) l, List.map (force_univ_level ~loc) l'
          in
          Some (l,l')
        
                                                )])]))
  in let () = assert (Procq.Entry.is_empty univ_level_or_quality) in
  let () =
  Procq.grammar_extend univ_level_or_quality
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm global)))
                            (fun id loc -> 
# 331 "parsing/g_constr.mlg"
                         Global id 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("_")))))
                           (fun _ loc -> 
# 330 "parsing/g_constr.mlg"
                 Anon loc 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("Type")))))
                           (fun _ loc -> 
# 329 "parsing/g_constr.mlg"
                    Type 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("Prop")))))
                           (fun _ loc -> 
# 328 "parsing/g_constr.mlg"
                    Prop 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("SProp")))))
                           (fun _ loc -> 
# 327 "parsing/g_constr.mlg"
                     SProp 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("Set")))))
                           (fun _ loc -> 
# 326 "parsing/g_constr.mlg"
                   Set 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty fix_decls) in
  let () =
  Procq.grammar_extend fix_decls
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.nterm fix_decl)))
                                ((Procq.Symbol.token (Tok.PKEYWORD ("with")))))
                               ((Procq.Symbol.list1sep ((Procq.Symbol.nterm fix_decl)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD ("with"))])) false)))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("for")))))
                             ((Procq.Symbol.nterm identref)))
                            (fun id _ dcls _ dcl loc -> 
# 336 "parsing/g_constr.mlg"
          (id,List.map (fun x -> x.CAst.v) (dcl::dcls)) 
                                                        );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm fix_decl)))
                           (fun dcl loc -> 
# 334 "parsing/g_constr.mlg"
                            let (id,_,_,_,_,_) = dcl.CAst.v in (id,[dcl.CAst.v]) 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty cofix_decls) in
  let () =
  Procq.grammar_extend cofix_decls
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.nterm cofix_body)))
                                ((Procq.Symbol.token (Tok.PKEYWORD ("with")))))
                               ((Procq.Symbol.list1sep ((Procq.Symbol.nterm cofix_body)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD ("with"))])) false)))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("for")))))
                             ((Procq.Symbol.nterm identref)))
                            (fun id _ dcls _ dcl loc -> 
# 341 "parsing/g_constr.mlg"
          (id,List.map (fun x -> x.CAst.v) (dcl::dcls)) 
                                                        );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm cofix_body)))
                           (fun dcl loc -> 
# 339 "parsing/g_constr.mlg"
                              let (id,_,_,_,_) = dcl.CAst.v in (id,[dcl.CAst.v]) 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty fix_decl) in
  let () =
  Procq.grammar_extend fix_decl
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.nterm identref)))
                                ((Procq.Symbol.nterm binders_fixannot)))
                               ((Procq.Symbol.nterm type_cstr)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                             ((Procq.Symbol.nterml term ("200"))))
                            (fun c _ ty bl id loc -> 
# 346 "parsing/g_constr.mlg"
          CAst.make ~loc (id,None,snd bl,fst bl,ty,c) 
                                                     )])]))
  in let () = assert (Procq.Entry.is_empty cofix_body) in
  let () =
  Procq.grammar_extend cofix_body
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.nterm identref)))
                                ((Procq.Symbol.nterm binders)))
                               ((Procq.Symbol.nterm type_cstr)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                             ((Procq.Symbol.nterml term ("200"))))
                            (fun c _ ty bl id loc -> 
# 351 "parsing/g_constr.mlg"
          CAst.make ~loc (id,None,bl,ty,c) 
                                                     )])]))
  in let () = assert (Procq.Entry.is_empty term_match) in
  let () =
  Procq.grammar_extend term_match
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next (Procq.Rule.stop)
                                  ((Procq.Symbol.token (Tok.PKEYWORD ("match")))))
                                 ((Procq.Symbol.list1sep ((Procq.Symbol.nterm case_item)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                                ((Procq.Symbol.opt (Procq.Symbol.nterm case_type))))
                               ((Procq.Symbol.token (Tok.PKEYWORD ("with")))))
                              ((Procq.Symbol.nterm branches)))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("end")))))
                            (fun _ br _ ty ci _ loc -> 
# 355 "parsing/g_constr.mlg"
                                  CAst.make ~loc @@ CCases(RegularStyle,ty,ci,br) 
                                                       )])]))
  in let () = assert (Procq.Entry.is_empty case_item) in
  let () =
  Procq.grammar_extend case_item
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.nterml term ("100"))))
                              ((Procq.Symbol.opt (Procq.Symbol.rules 
                              [Procq.Rules.make (Procq.Rule.next_norec
                                                 (Procq.Rule.next_norec
                                                  (Procq.Rule.stop)
                                                  ((Procq.Symbol.token (Tok.PKEYWORD ("as")))))
                                                 ((Procq.Symbol.nterm name)))
                                                (fun id _ loc -> 
# 359 "parsing/g_constr.mlg"
                                        id 
                                                                 )]))))
                             ((Procq.Symbol.opt (Procq.Symbol.rules [Procq.Rules.make 
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("in")))))
                                                                    ((Procq.Symbol.nterml pattern ("200"))))
                                                                    (fun t _
                                                                    loc -> 
                                                                    
# 360 "parsing/g_constr.mlg"
                                                     t 
                                                                    )]))))
                            (fun ty ona c loc -> 
# 361 "parsing/g_constr.mlg"
          (c,ona,ty) 
                                                 )])]))
  in let () = assert (Procq.Entry.is_empty case_type) in
  let () =
  Procq.grammar_extend case_type
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("return")))))
                             ((Procq.Symbol.nterml term ("100"))))
                            (fun ty _ loc -> 
# 364 "parsing/g_constr.mlg"
                                             ty 
                                             )])]))
  in let () = assert (Procq.Entry.is_empty as_return_type) in
  let () =
  Procq.grammar_extend as_return_type
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.opt (Procq.Symbol.rules [Procq.Rules.make 
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.opt (Procq.Symbol.rules 
                                                                    [Procq.Rules.make 
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("as")))))
                                                                    ((Procq.Symbol.nterm name)))
                                                                    (fun na _
                                                                    loc -> 
                                                                    
# 367 "parsing/g_constr.mlg"
                                                 na 
                                                                    )]))))
                                                                    ((Procq.Symbol.nterm case_type)))
                                                                    (fun ty
                                                                    na loc ->
                                                                    
# 368 "parsing/g_constr.mlg"
                                      (na,ty) 
                                                                    )]))))
                            (fun a loc -> 
# 369 "parsing/g_constr.mlg"
          match a with
          | None -> None, None
          | Some (na,t) -> (na, Some t) 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty branches) in
  let () =
  Procq.grammar_extend branches
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.opt (Procq.Symbol.token (Tok.PKEYWORD ("|"))))))
                             ((Procq.Symbol.list0sep ((Procq.Symbol.nterm eqn)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD ("|"))])) false)))
                            (fun br _ loc -> 
# 374 "parsing/g_constr.mlg"
                                            br 
                                             )])]))
  in let () = assert (Procq.Entry.is_empty mult_pattern) in
  let () =
  Procq.grammar_extend mult_pattern
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.list1sep ((Procq.Symbol.nterml pattern ("200"))) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                            (fun pl loc -> 
# 377 "parsing/g_constr.mlg"
                                                    pl 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty eqn) in
  let () =
  Procq.grammar_extend eqn
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.list1sep ((Procq.Symbol.nterm mult_pattern)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD ("|"))])) false)))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("=>")))))
                             ((Procq.Symbol.nterm lconstr)))
                            (fun rhs _ pll loc -> 
# 381 "parsing/g_constr.mlg"
                                 (CAst.make ~loc (pll,rhs)) 
                                                  )])]))
  in let () = assert (Procq.Entry.is_empty record_pattern) in
  let () =
  Procq.grammar_extend record_pattern
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.nterm global)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                             ((Procq.Symbol.nterml pattern ("200"))))
                            (fun pat _ id loc -> 
# 384 "parsing/g_constr.mlg"
                                                          (id, pat) 
                                                 )])]))
  in let () = assert (Procq.Entry.is_empty record_patterns) in
  let () =
  Procq.grammar_extend record_patterns
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 389 "parsing/g_constr.mlg"
             [] 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm record_pattern)))
                           (fun p loc -> 
# 388 "parsing/g_constr.mlg"
                               [p] 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm record_pattern)))
                             ((Procq.Symbol.token (Tok.PKEYWORD (";")))))
                            ((Procq.Symbol.nterm record_patterns)))
                           (fun ps _ p loc -> 
# 387 "parsing/g_constr.mlg"
                                                           p :: ps 
                                              )])]))
  in let () = assert (Procq.Entry.is_empty pattern) in
  let () =
  Procq.grammar_extend pattern
  (Procq.Fresh
  (Gramlib.Gramext.First, [(Some ("200"), Some (Gramlib.Gramext.RightA), []);
                          (Some ("100"), Some (Gramlib.Gramext.RightA),
                          [Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm pattern)))
                             ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                            ((Procq.Symbol.nterml term ("200"))))
                           (fun ty _ p loc -> 
# 395 "parsing/g_constr.mlg"
          CAst.make ~loc @@ CPatCast (p, ty) 
                                              )]);
                          (Some ("99"), Some (Gramlib.Gramext.RightA), []);
                          (Some ("90"), Some (Gramlib.Gramext.RightA), []);
                          (Some ("10"), Some (Gramlib.Gramext.LeftA),
                          [Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("@")))))
                             ((Procq.Symbol.nterm Prim.reference)))
                            ((Procq.Symbol.list0 Procq.Symbol.next)))
                           (fun lp r _ loc -> 
# 403 "parsing/g_constr.mlg"
          CAst.make ~loc @@ CPatCstr (r, Some lp, []) 
                                              );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm pattern)))
                           ((Procq.Symbol.list1 (Procq.Symbol.next))))
                          (fun lp p loc -> 
# 401 "parsing/g_constr.mlg"
                                          mkAppPattern ~loc p lp 
                                           );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm pattern)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("as")))))
                           ((Procq.Symbol.nterm name)))
                          (fun na _ p loc -> 
# 400 "parsing/g_constr.mlg"
          CAst.make ~loc @@ CPatAlias (p, na) 
                                             )]);
                          (Some ("1"), Some (Gramlib.Gramext.LeftA),
                          [Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm pattern)))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("%_")))))
                            ((Procq.Symbol.token (Tok.PIDENT (None)))))
                           (fun key _ c loc -> 
# 408 "parsing/g_constr.mlg"
          CAst.make ~loc @@ CPatDelimiters (DelimOnlyTmpScope,key,c) 
                                               );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm pattern)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("%")))))
                           ((Procq.Symbol.token (Tok.PIDENT (None)))))
                          (fun key _ c loc -> 
# 406 "parsing/g_constr.mlg"
          CAst.make ~loc @@ CPatDelimiters (DelimUnboundedScope,key,c) 
                                              )]);
                          (Some ("0"), None,
                          [Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm string)))
                           (fun s loc -> 
# 423 "parsing/g_constr.mlg"
                        CAst.make ~loc @@ CPatPrim (String s) 
                                         );
                          Procq.Production.make
                          (Procq.Rule.next (Procq.Rule.stop)
                           ((Procq.Symbol.token (Tok.PNUMBER None))))
                          (fun n loc -> 
# 422 "parsing/g_constr.mlg"
                       CAst.make ~loc @@ CPatPrim (Number (NumTok.SPlus,n)) 
                                        );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                              ((Procq.Symbol.nterml pattern ("200"))))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("|")))))
                            ((Procq.Symbol.list1sep ((Procq.Symbol.nterml pattern ("200"))) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD ("|"))])) false)))
                           ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                          (fun _ pl _ p _ loc -> 
# 421 "parsing/g_constr.mlg"
          CAst.make ~loc @@ CPatOr (p::pl) 
                                                 );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                            ((Procq.Symbol.nterml pattern ("200"))))
                           ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                          (fun _ p _ loc -> 
# 414 "parsing/g_constr.mlg"
          (* Preserve parentheses around numbers so that constrintern does not
             collapse -(3) into the number -3. *)
          match p.CAst.v with
          | CPatPrim (Number (NumTok.SPlus,n)) ->
              CAst.make ~loc @@ CPatNotation(None,(InConstrEntry,"( _ )"),([p],[],[]),[])
          | _ -> p 
                                            );
                          Procq.Production.make
                          (Procq.Rule.next (Procq.Rule.stop)
                           ((Procq.Symbol.token (Tok.PKEYWORD ("_")))))
                          (fun _ loc -> 
# 412 "parsing/g_constr.mlg"
                 CAst.make ~loc @@ CPatAtom None 
                                        );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("{|")))))
                            ((Procq.Symbol.nterm record_patterns)))
                           ((Procq.Symbol.nterm bar_cbrace)))
                          (fun _ pat _ loc -> 
# 411 "parsing/g_constr.mlg"
                                                     CAst.make ~loc @@ CPatRecord pat 
                                              );
                          Procq.Production.make
                          (Procq.Rule.next (Procq.Rule.stop)
                           ((Procq.Symbol.nterm Prim.reference)))
                          (fun r loc -> 
# 410 "parsing/g_constr.mlg"
                                               CAst.make ~loc @@ CPatAtom (Some r) 
                                        )])]))
  in let () = assert (Procq.Entry.is_empty fixannot) in
  let () =
  Procq.grammar_extend fixannot
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next (Procq.Rule.stop)
                                  ((Procq.Symbol.token (Tok.PKEYWORD ("{")))))
                                 ((Procq.Symbol.token (Tok.PIDENT (Some
                                 ("measure"))))))
                                ((Procq.Symbol.nterm constr)))
                               ((Procq.Symbol.opt (Procq.Symbol.nterm identref))))
                              ((Procq.Symbol.opt (Procq.Symbol.nterm constr))))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                            (fun _ rel id m _ _ loc -> 
# 429 "parsing/g_constr.mlg"
          CAst.make ~loc @@ CMeasureRec (id,m,rel) 
                                                       );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PKEYWORD ("{")))))
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("wf"))))))
                              ((Procq.Symbol.nterm constr)))
                             ((Procq.Symbol.nterm identref)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                           (fun _ id rel _ _ loc -> 
# 427 "parsing/g_constr.mlg"
                                                               CAst.make ~loc @@ CWfRec(id,rel) 
                                                    );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("{")))))
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("struct"))))))
                             ((Procq.Symbol.nterm identref)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                           (fun _ id _ _ loc -> 
# 426 "parsing/g_constr.mlg"
                                                     CAst.make ~loc @@ CStructRec id 
                                                )])]))
  in let () = assert (Procq.Entry.is_empty binders_fixannot) in
  let () =
  Procq.grammar_extend binders_fixannot
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 434 "parsing/g_constr.mlg"
             [], None 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm binder)))
                            ((Procq.Symbol.nterm binders_fixannot)))
                           (fun bl b loc -> 
# 433 "parsing/g_constr.mlg"
                                               b @ fst bl, snd bl 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm ensure_fixannot)))
                            ((Procq.Symbol.nterm fixannot)))
                           (fun f _ loc -> 
# 432 "parsing/g_constr.mlg"
                                           [], Some f 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty open_binders) in
  let () =
  Procq.grammar_extend open_binders
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm closed_binder)))
                             ((Procq.Symbol.nterm binders)))
                            (fun bl' bl loc -> 
# 448 "parsing/g_constr.mlg"
          bl@bl' 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm name)))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("..")))))
                            ((Procq.Symbol.nterm name)))
                           (fun id2 _ id1 loc -> 
# 445 "parsing/g_constr.mlg"
          [CLocalAssum ([id1;(CAst.make ~loc (Name ldots_var));id2],
                        None, Default Explicit, CAst.make ~loc @@ CHole (None))] 
                                                 );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm name)))
                             ((Procq.Symbol.list0 (Procq.Symbol.nterm name))))
                            ((Procq.Symbol.nterm binders)))
                           (fun bl idl id loc -> 
# 443 "parsing/g_constr.mlg"
          binders_of_names (id::idl) @ bl 
                                                 );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.nterm name)))
                              ((Procq.Symbol.list0 (Procq.Symbol.nterm name))))
                             ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                            ((Procq.Symbol.nterm lconstr)))
                           (fun c _ idl id loc -> 
# 440 "parsing/g_constr.mlg"
          [CLocalAssum (id::idl,None,Default Explicit,c)] 
                                                  )])]))
  in let () = assert (Procq.Entry.is_empty binders) in
  let () =
  Procq.grammar_extend binders
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.list0 (Procq.Symbol.nterm binder))))
                            (fun l loc -> 
# 451 "parsing/g_constr.mlg"
                              List.flatten l 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty binder) in
  let () =
  Procq.grammar_extend binder
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm closed_binder)))
                            (fun bl loc -> 
# 455 "parsing/g_constr.mlg"
                                bl 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm name)))
                           (fun id loc -> 
# 454 "parsing/g_constr.mlg"
                       [CLocalAssum ([id], None, Default Explicit, CAst.make ~loc @@ CHole (None))] 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty closed_binder) in
  let () =
  Procq.grammar_extend closed_binder
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("'")))))
                             ((Procq.Symbol.nterml pattern ("0"))))
                            (fun p _ loc -> 
# 493 "parsing/g_constr.mlg"
                                        [CLocalPattern p] 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("`[")))))
                             ((Procq.Symbol.list1sep ((Procq.Symbol.nterm typeclass_constraint)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("]")))))
                           (fun _ tc _ loc -> 
# 492 "parsing/g_constr.mlg"
          List.map (fun (n, b, t) -> CLocalAssum ([n], None, Generalized (NonMaxImplicit, b), t)) tc 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("`{")))))
                             ((Procq.Symbol.list1sep ((Procq.Symbol.nterm typeclass_constraint)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                           (fun _ tc _ loc -> 
# 490 "parsing/g_constr.mlg"
          List.map (fun (n, b, t) -> CLocalAssum ([n], None, Generalized (MaxImplicit, b), t)) tc 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("`(")))))
                             ((Procq.Symbol.list1sep ((Procq.Symbol.nterm typeclass_constraint)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                           (fun _ tc _ loc -> 
# 488 "parsing/g_constr.mlg"
          List.map (fun (n, b, t) -> CLocalAssum ([n], None, Generalized (Explicit, b), t)) tc 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("[")))))
                              ((Procq.Symbol.nterm name)))
                             ((Procq.Symbol.list1 ((Procq.Symbol.nterm name)))))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("]")))))
                           (fun _ idl id _ loc -> 
# 486 "parsing/g_constr.mlg"
          List.map (fun id -> CLocalAssum ([id], None, Default NonMaxImplicit, CAst.make ~loc @@ CHole (None))) (id::idl) 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PKEYWORD ("[")))))
                               ((Procq.Symbol.nterm name)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                             ((Procq.Symbol.nterm lconstr)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("]")))))
                           (fun _ c _ id _ loc -> 
# 484 "parsing/g_constr.mlg"
          [CLocalAssum ([id],None, Default NonMaxImplicit,c)] 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.token (Tok.PKEYWORD ("[")))))
                                ((Procq.Symbol.nterm name)))
                               ((Procq.Symbol.list1 ((Procq.Symbol.nterm name)))))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                             ((Procq.Symbol.nterm lconstr)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("]")))))
                           (fun _ c _ idl id _ loc -> 
# 482 "parsing/g_constr.mlg"
          [CLocalAssum (id::idl,None,Default NonMaxImplicit,c)] 
                                                      );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("[")))))
                             ((Procq.Symbol.nterm name)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("]")))))
                           (fun _ id _ loc -> 
# 480 "parsing/g_constr.mlg"
          [CLocalAssum ([id], None, Default NonMaxImplicit, CAst.make ~loc @@ CHole (None))] 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("{")))))
                              ((Procq.Symbol.nterm name)))
                             ((Procq.Symbol.list1 ((Procq.Symbol.nterm name)))))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                           (fun _ idl id _ loc -> 
# 478 "parsing/g_constr.mlg"
          List.map (fun id -> CLocalAssum ([id], None, Default MaxImplicit, CAst.make ~loc @@ CHole (None))) (id::idl) 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PKEYWORD ("{")))))
                               ((Procq.Symbol.nterm name)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                             ((Procq.Symbol.nterm lconstr)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                           (fun _ c _ id _ loc -> 
# 476 "parsing/g_constr.mlg"
          [CLocalAssum ([id],None,Default MaxImplicit,c)] 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.token (Tok.PKEYWORD ("{")))))
                                ((Procq.Symbol.nterm name)))
                               ((Procq.Symbol.list1 ((Procq.Symbol.nterm name)))))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                             ((Procq.Symbol.nterm lconstr)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                           (fun _ c _ idl id _ loc -> 
# 474 "parsing/g_constr.mlg"
          [CLocalAssum (id::idl,None,Default MaxImplicit,c)] 
                                                      );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("{")))))
                             ((Procq.Symbol.nterm name)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                           (fun _ id _ loc -> 
# 472 "parsing/g_constr.mlg"
          [CLocalAssum ([id], None, Default MaxImplicit, CAst.make ~loc @@ CHole (None))] 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next (Procq.Rule.stop)
                                  ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                                 ((Procq.Symbol.nterm name)))
                                ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                               ((Procq.Symbol.nterm lconstr)))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("|")))))
                             ((Procq.Symbol.nterm lconstr)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                           (fun _ c _ t _ id _ loc -> 
# 469 "parsing/g_constr.mlg"
          let typ = mkAppC (sigref loc, [mkLambdaC ([id], default_binder_kind, t, c)]) in
          [CLocalAssum ([id], None, default_binder_kind, typ)] 
                                                      );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next (Procq.Rule.stop)
                                  ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                                 ((Procq.Symbol.nterm name)))
                                ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                               ((Procq.Symbol.nterm lconstr)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                             ((Procq.Symbol.nterm lconstr)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                           (fun _ c _ t _ id _ loc -> 
# 467 "parsing/g_constr.mlg"
          [CLocalDef (id,None,c,Some t)] 
                                                      );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                               ((Procq.Symbol.nterm name)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                             ((Procq.Symbol.nterm lconstr)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                           (fun _ c _ id _ loc -> 
# 463 "parsing/g_constr.mlg"
          match c.CAst.v with
          | CCast(c, Some DEFAULTcast, t) -> [CLocalDef (id,None,c,Some t)]
          | _ -> [CLocalDef (id,None,c,None)] 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                               ((Procq.Symbol.nterm name)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                             ((Procq.Symbol.nterm lconstr)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                           (fun _ c _ id _ loc -> 
# 461 "parsing/g_constr.mlg"
          [CLocalAssum ([id],None,Default Explicit,c)] 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                                ((Procq.Symbol.nterm name)))
                               ((Procq.Symbol.list1 ((Procq.Symbol.nterm name)))))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                             ((Procq.Symbol.nterm lconstr)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                           (fun _ c _ idl id _ loc -> 
# 459 "parsing/g_constr.mlg"
          [CLocalAssum (id::idl,None,Default Explicit,c)] 
                                                      )])]))
  in let () = assert (Procq.Entry.is_empty one_open_binder) in
  let () =
  Procq.grammar_extend one_open_binder
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm one_closed_binder)))
                            (fun b loc -> 
# 498 "parsing/g_constr.mlg"
                                   b 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm name)))
                             ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                            ((Procq.Symbol.nterm lconstr)))
                           (fun t _ na loc -> 
# 497 "parsing/g_constr.mlg"
                                         (CAst.make ~loc @@ CPatCast (pat_of_name na, t), Explicit) 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm name)))
                           (fun na loc -> 
# 496 "parsing/g_constr.mlg"
                       (pat_of_name na, Explicit) 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty one_closed_binder) in
  let () =
  Procq.grammar_extend one_closed_binder
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("'")))))
                             ((Procq.Symbol.nterml pattern ("0"))))
                            (fun p _ loc -> 
# 506 "parsing/g_constr.mlg"
                                        (p, Explicit) 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PKEYWORD ("[")))))
                               ((Procq.Symbol.nterm name)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                             ((Procq.Symbol.nterm lconstr)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("]")))))
                           (fun _ t _ na _ loc -> 
# 505 "parsing/g_constr.mlg"
                                                   (CAst.make ~loc @@ CPatCast (pat_of_name na, t), NonMaxImplicit) 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("[")))))
                             ((Procq.Symbol.nterm name)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("]")))))
                           (fun _ na _ loc -> 
# 504 "parsing/g_constr.mlg"
                                 (pat_of_name na, NonMaxImplicit) 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PKEYWORD ("{")))))
                               ((Procq.Symbol.nterm name)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                             ((Procq.Symbol.nterm lconstr)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                           (fun _ t _ na _ loc -> 
# 503 "parsing/g_constr.mlg"
                                                   (CAst.make ~loc @@ CPatCast (pat_of_name na, t), MaxImplicit) 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("{")))))
                             ((Procq.Symbol.nterm name)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                           (fun _ na _ loc -> 
# 502 "parsing/g_constr.mlg"
                                 (pat_of_name na, MaxImplicit) 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                               ((Procq.Symbol.nterm name)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                             ((Procq.Symbol.nterm lconstr)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                           (fun _ t _ na _ loc -> 
# 501 "parsing/g_constr.mlg"
                                                   (CAst.make ~loc @@ CPatCast (pat_of_name na, t), Explicit) 
                                                  )])]))
  in let () = assert (Procq.Entry.is_empty typeclass_constraint) in
  let () =
  Procq.grammar_extend typeclass_constraint
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterml term ("200"))))
                            (fun c loc -> 
# 515 "parsing/g_constr.mlg"
            (CAst.make ~loc Anonymous), false, c 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.nterm test_name_colon)))
                               ((Procq.Symbol.nterm name)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                             ((Procq.Symbol.rules [Procq.Rules.make (Procq.Rule.stop)
                                                                    (fun
                                                                    loc -> 
                                                                    
# 512 "parsing/g_constr.mlg"
                                                                           false 
                                                                    );
                                                  Procq.Rules.make (Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("!")))))
                                                                   (fun _
                                                                   loc -> 
                                                                   
# 512 "parsing/g_constr.mlg"
                                                             true 
                                                                   )])))
                            ((Procq.Symbol.nterml term ("200"))))
                           (fun c expl _ iid _ loc -> 
# 513 "parsing/g_constr.mlg"
            iid, expl, c 
                                                      );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.token (Tok.PKEYWORD ("{")))))
                                ((Procq.Symbol.nterm name)))
                               ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                             ((Procq.Symbol.rules [Procq.Rules.make (Procq.Rule.stop)
                                                                    (fun
                                                                    loc -> 
                                                                    
# 510 "parsing/g_constr.mlg"
                                                                   false 
                                                                    );
                                                  Procq.Rules.make (Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("!")))))
                                                                   (fun _
                                                                   loc -> 
                                                                   
# 510 "parsing/g_constr.mlg"
                                                     true 
                                                                   )])))
                            ((Procq.Symbol.nterml term ("200"))))
                           (fun c expl _ _ id _ loc -> 
# 511 "parsing/g_constr.mlg"
            id, expl, c 
                                                       );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("!")))))
                            ((Procq.Symbol.nterml term ("200"))))
                           (fun c _ loc -> 
# 509 "parsing/g_constr.mlg"
                                        (CAst.make ~loc Anonymous), true, c 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty type_cstr) in
  let () =
  Procq.grammar_extend type_cstr
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 519 "parsing/g_constr.mlg"
             CAst.make ~loc @@ CHole (None) 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                            ((Procq.Symbol.nterm lconstr)))
                           (fun c _ loc -> 
# 518 "parsing/g_constr.mlg"
                              c 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty let_type_cstr) in
  let () =
  Procq.grammar_extend let_type_cstr
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.opt (Procq.Symbol.rules [Procq.Rules.make 
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                                                                    ((Procq.Symbol.nterm lconstr)))
                                                                    (fun c _
                                                                    loc -> 
                                                                    
# 522 "parsing/g_constr.mlg"
                                       c 
                                                                    )]))))
                            (fun c loc -> 
# 522 "parsing/g_constr.mlg"
                                                  Loc.tag ~loc c 
                                          )])]))
  in ()

