
# 13 "vernac/g_proofs.mlg"
 

open Constrexpr
open Vernacexpr
open Hints

module C = Constr

open Procq
open Procq.Prim
open Procq.Constr
open Pvernac.Vernac_

let thm_token = G_vernac.thm_token

let hint = Entry.make "hint"

let warn_deprecated_focus =
  CWarnings.create ~name:"deprecated-focus" ~category:Deprecation.Version.v8_8
    Pp.(function
      | None ->
        strbrk "The Focus command is deprecated; use bullets or focusing brackets instead."
      | Some n ->
        str "The Focus command is deprecated;" ++ spc ()
        ++ str "use '" ++ int n ++ str ": {' instead.")

let warn_deprecated_focus_n ?loc n = warn_deprecated_focus ?loc (Some n)

let warn_deprecated_focus ?loc () = warn_deprecated_focus ?loc None

let warn_deprecated_unfocus =
  CWarnings.create ~name:"deprecated-unfocus" ~category:Deprecation.Version.v8_8
         (fun () -> Pp.strbrk "The Unfocus command is deprecated")



let _ =
  let opt_hintbases = Procq.Entry.make "opt_hintbases"
  and reference_or_constr = Procq.Entry.make "reference_or_constr"
  and constr_body = Procq.Entry.make "constr_body"
  and mode = Procq.Entry.make "mode"
  in
  let () = assert (Procq.Entry.is_empty opt_hintbases) in
  let () =
  Procq.grammar_extend opt_hintbases
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                             ((Procq.Symbol.list1 ((Procq.Symbol.rules 
                             [Procq.Rules.make (Procq.Rule.next_norec
                                                (Procq.Rule.stop)
                                                ((Procq.Symbol.token (Tok.PIDENT (None)))))
                                               (fun id loc -> 
# 55 "vernac/g_proofs.mlg"
                                      id 
                                                              )])))))
                            (fun l _ loc -> 
# 55 "vernac/g_proofs.mlg"
                                                  l 
                                            );
                           Procq.Production.make (Procq.Rule.stop)
                           (fun loc -> 
# 54 "vernac/g_proofs.mlg"
           [] 
                                       )])]))
  in let () =
  Procq.grammar_extend command
  (Procq.Reuse (None, [Procq.Production.make
                       (Procq.Rule.next
                        (Procq.Rule.next
                         (Procq.Rule.next (Procq.Rule.stop)
                          ((Procq.Symbol.token (Tok.PIDENT (Some ("Hint"))))))
                         ((Procq.Symbol.nterm hint)))
                        ((Procq.Symbol.nterm opt_hintbases)))
                       (fun dbnames h _ loc -> 
# 107 "vernac/g_proofs.mlg"
            VernacSynPure (VernacHints (dbnames, h)) 
                                               );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next
                         (Procq.Rule.next (Procq.Rule.stop)
                          ((Procq.Symbol.token (Tok.PIDENT (Some
                          ("Remove"))))))
                         ((Procq.Symbol.token (Tok.PIDENT (Some ("Hints"))))))
                        ((Procq.Symbol.list1 ((Procq.Symbol.nterm global)))))
                       ((Procq.Symbol.nterm opt_hintbases)))
                      (fun dbnames ids _ _ loc -> 
# 105 "vernac/g_proofs.mlg"
            VernacSynPure (VernacRemoveHints (dbnames, ids)) 
                                                  );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next
                         (Procq.Rule.next (Procq.Rule.stop)
                          ((Procq.Symbol.token (Tok.PIDENT (Some
                          ("Create"))))))
                         ((Procq.Symbol.token (Tok.PIDENT (Some
                         ("HintDb"))))))
                        ((Procq.Symbol.token (Tok.PIDENT (None)))))
                       ((Procq.Symbol.rules [Procq.Rules.make (Procq.Rule.stop)
                                                              (fun loc -> 
                                                              
# 102 "vernac/g_proofs.mlg"
                                                                      false 
                                                              );
                                            Procq.Rules.make (Procq.Rule.next_norec
                                                              (Procq.Rule.stop)
                                                              ((Procq.Symbol.token (Tok.PIDENT (Some
                                                              ("discriminated"))))))
                                                             (fun _ loc -> 
                                                             
# 102 "vernac/g_proofs.mlg"
                                                        true 
                                                             )])))
                      (fun b id _ _ loc -> 
# 103 "vernac/g_proofs.mlg"
              VernacSynPure (VernacCreateHintDb (id, b)) 
                                           );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some
                        ("Validate"))))))
                       ((Procq.Symbol.token (Tok.PIDENT (Some ("Proof"))))))
                      (fun _ _ loc -> 
# 99 "vernac/g_proofs.mlg"
                                             VernacSynPure VernacValidateProof 
                                      );
                      Procq.Production.make
                      (Procq.Rule.next (Procq.Rule.stop)
                       ((Procq.Symbol.token (Tok.PIDENT (Some ("Guarded"))))))
                      (fun _ loc -> 
# 98 "vernac/g_proofs.mlg"
                             VernacSynPure VernacCheckGuard 
                                    );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next (Procq.Rule.stop)
                         ((Procq.Symbol.token (Tok.PIDENT (Some ("Show"))))))
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Match"))))))
                       ((Procq.Symbol.nterm reference)))
                      (fun id _ _ loc -> 
# 97 "vernac/g_proofs.mlg"
                                                         VernacSynPure (VernacShow (ShowMatch id)) 
                                         );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Show"))))))
                       ((Procq.Symbol.token (Tok.PIDENT (Some ("Intros"))))))
                      (fun _ _ loc -> 
# 96 "vernac/g_proofs.mlg"
                                          VernacSynPure (VernacShow (ShowIntros true)) 
                                      );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Show"))))))
                       ((Procq.Symbol.token (Tok.PIDENT (Some ("Intro"))))))
                      (fun _ _ loc -> 
# 95 "vernac/g_proofs.mlg"
                                         VernacSynPure (VernacShow (ShowIntros false)) 
                                      );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Show"))))))
                       ((Procq.Symbol.token (Tok.PIDENT (Some ("Proof"))))))
                      (fun _ _ loc -> 
# 94 "vernac/g_proofs.mlg"
                                         VernacSynPure (VernacShow ShowProof) 
                                      );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Show"))))))
                       ((Procq.Symbol.token (Tok.PIDENT (Some
                       ("Conjectures"))))))
                      (fun _ _ loc -> 
# 93 "vernac/g_proofs.mlg"
                                               VernacSynPure (VernacShow ShowProofNames) 
                                      );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Show"))))))
                       ((Procq.Symbol.token (Tok.PIDENT (Some
                       ("Universes"))))))
                      (fun _ _ loc -> 
# 92 "vernac/g_proofs.mlg"
                                             VernacSynPure (VernacShow ShowUniverses) 
                                      );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Show"))))))
                       ((Procq.Symbol.token (Tok.PIDENT (Some
                       ("Existentials"))))))
                      (fun _ _ loc -> 
# 91 "vernac/g_proofs.mlg"
                                                VernacSynPure (VernacShow ShowExistentials) 
                                      );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Show"))))))
                       ((Procq.Symbol.nterm ident)))
                      (fun id _ loc -> 
# 90 "vernac/g_proofs.mlg"
                                      VernacSynPure (VernacShow (ShowGoal (GoalId id))) 
                                       );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Show"))))))
                       ((Procq.Symbol.nterm natural)))
                      (fun n _ loc -> 
# 89 "vernac/g_proofs.mlg"
                                       VernacSynPure (VernacShow (ShowGoal (NthGoal n))) 
                                      );
                      Procq.Production.make
                      (Procq.Rule.next (Procq.Rule.stop)
                       ((Procq.Symbol.token (Tok.PIDENT (Some ("Show"))))))
                      (fun _ loc -> 
# 88 "vernac/g_proofs.mlg"
                          VernacSynPure (VernacShow (ShowGoal OpenSubgoals)) 
                                    );
                      Procq.Production.make
                      (Procq.Rule.next (Procq.Rule.stop)
                       ((Procq.Symbol.token (Tok.PIDENT (Some
                       ("Unfocused"))))))
                      (fun _ loc -> 
# 87 "vernac/g_proofs.mlg"
                               VernacSynPure VernacUnfocused 
                                    );
                      Procq.Production.make
                      (Procq.Rule.next (Procq.Rule.stop)
                       ((Procq.Symbol.token (Tok.PIDENT (Some ("Unfocus"))))))
                      (fun _ loc -> 
# 85 "vernac/g_proofs.mlg"
           warn_deprecated_unfocus ~loc ();
         VernacSynPure VernacUnfocus 
                                    );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Focus"))))))
                       ((Procq.Symbol.nterm natural)))
                      (fun n _ loc -> 
# 82 "vernac/g_proofs.mlg"
           warn_deprecated_focus_n n ~loc;
         VernacSynPure (VernacFocus (Some n)) 
                                      );
                      Procq.Production.make
                      (Procq.Rule.next (Procq.Rule.stop)
                       ((Procq.Symbol.token (Tok.PIDENT (Some ("Focus"))))))
                      (fun _ loc -> 
# 79 "vernac/g_proofs.mlg"
           warn_deprecated_focus ~loc ();
         VernacSynPure (VernacFocus None) 
                                    );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next (Procq.Rule.stop)
                         ((Procq.Symbol.token (Tok.PIDENT (Some ("Undo"))))))
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("To"))))))
                       ((Procq.Symbol.nterm natural)))
                      (fun n _ _ loc -> 
# 77 "vernac/g_proofs.mlg"
                                                   VernacSynPure (VernacUndoTo n) 
                                        );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Undo"))))))
                       ((Procq.Symbol.nterm natural)))
                      (fun n _ loc -> 
# 76 "vernac/g_proofs.mlg"
                                       VernacSynPure (VernacUndo n) 
                                      );
                      Procq.Production.make
                      (Procq.Rule.next (Procq.Rule.stop)
                       ((Procq.Symbol.token (Tok.PIDENT (Some ("Undo"))))))
                      (fun _ loc -> 
# 75 "vernac/g_proofs.mlg"
                          VernacSynPure (VernacUndo 1) 
                                    );
                      Procq.Production.make
                      (Procq.Rule.next (Procq.Rule.stop)
                       ((Procq.Symbol.token (Tok.PIDENT (Some ("Restart"))))))
                      (fun _ loc -> 
# 74 "vernac/g_proofs.mlg"
                             VernacSynPure VernacRestart 
                                    );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some
                        ("Defined"))))))
                       ((Procq.Symbol.nterm identref)))
                      (fun id _ loc -> 
# 73 "vernac/g_proofs.mlg"
            VernacSynPure (VernacEndProof (Proved (Transparent,Some id))) 
                                       );
                      Procq.Production.make
                      (Procq.Rule.next (Procq.Rule.stop)
                       ((Procq.Symbol.token (Tok.PIDENT (Some ("Defined"))))))
                      (fun _ loc -> 
# 71 "vernac/g_proofs.mlg"
                             VernacSynPure (VernacEndProof (Proved (Transparent,None))) 
                                    );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Save"))))))
                       ((Procq.Symbol.nterm identref)))
                      (fun id _ loc -> 
# 70 "vernac/g_proofs.mlg"
            VernacSynPure (VernacEndProof (Proved (Opaque, Some id))) 
                                       );
                      Procq.Production.make
                      (Procq.Rule.next (Procq.Rule.stop)
                       ((Procq.Symbol.token (Tok.PIDENT (Some ("Qed"))))))
                      (fun _ loc -> 
# 68 "vernac/g_proofs.mlg"
                         VernacSynPure (VernacEndProof (Proved (Opaque,None))) 
                                    );
                      Procq.Production.make
                      (Procq.Rule.next (Procq.Rule.stop)
                       ((Procq.Symbol.token (Tok.PIDENT (Some
                       ("Admitted"))))))
                      (fun _ loc -> 
# 67 "vernac/g_proofs.mlg"
                              VernacSynPure (VernacEndProof Admitted) 
                                    );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Abort"))))))
                       ((Procq.Symbol.token (Tok.PIDENT (Some ("All"))))))
                      (fun _ _ loc -> 
# 66 "vernac/g_proofs.mlg"
                                        VernacSynPure VernacAbortAll 
                                      );
                      Procq.Production.make
                      (Procq.Rule.next (Procq.Rule.stop)
                       ((Procq.Symbol.token (Tok.PIDENT (Some ("Abort"))))))
                      (fun _ loc -> 
# 65 "vernac/g_proofs.mlg"
                           VernacSynPure VernacAbort 
                                    );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Proof"))))))
                       ((Procq.Symbol.nterm lconstr)))
                      (fun c _ loc -> 
# 64 "vernac/g_proofs.mlg"
                                        VernacSynPure (VernacExactProof c) 
                                      );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next (Procq.Rule.stop)
                         ((Procq.Symbol.token (Tok.PIDENT (Some ("Proof"))))))
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Mode"))))))
                       ((Procq.Symbol.nterm string)))
                      (fun mn _ _ loc -> 
# 63 "vernac/g_proofs.mlg"
                                                        VernacSynterp (VernacProofMode mn) 
                                         );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next (Procq.Rule.stop)
                         ((Procq.Symbol.token (Tok.PIDENT (Some ("Proof"))))))
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("using"))))))
                       ((Procq.Symbol.nterm G_vernac.section_subset_expr)))
                      (fun l _ _ loc -> 
# 62 "vernac/g_proofs.mlg"
            VernacSynPure (VernacProof (None,Some l)) 
                                        );
                      Procq.Production.make
                      (Procq.Rule.next (Procq.Rule.stop)
                       ((Procq.Symbol.token (Tok.PIDENT (Some ("Proof"))))))
                      (fun _ loc -> 
# 60 "vernac/g_proofs.mlg"
                           VernacSynPure (VernacProof (None,None)) 
                                    );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Goal"))))))
                       ((Procq.Symbol.nterm lconstr)))
                      (fun c _ loc -> 
# 59 "vernac/g_proofs.mlg"
          VernacSynPure (VernacDefinition (Decls.(NoDischarge, Definition), ((CAst.make ~loc Names.Anonymous), None), ProveBody ([], c))) 
                                      )]))
  in let () = assert (Procq.Entry.is_empty reference_or_constr) in
  let () =
  Procq.grammar_extend reference_or_constr
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm constr)))
                            (fun c loc -> 
# 111 "vernac/g_proofs.mlg"
                       HintsConstr c 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm global)))
                           (fun r loc -> 
# 110 "vernac/g_proofs.mlg"
                       HintsReference r 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty hint) in
  let () =
  Procq.grammar_extend hint
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Constructors"))))))
                             ((Procq.Symbol.list1 ((Procq.Symbol.nterm global)))))
                            (fun lc _ loc -> 
# 131 "vernac/g_proofs.mlg"
                                                     HintsConstructors lc 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Unfold"))))))
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm global)))))
                           (fun lqid _ loc -> 
# 130 "vernac/g_proofs.mlg"
                                                 HintsUnfold lqid 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Mode"))))))
                             ((Procq.Symbol.nterm global)))
                            ((Procq.Symbol.nterm mode)))
                           (fun m l _ loc -> 
# 129 "vernac/g_proofs.mlg"
                                                HintsMode (l, m) 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Opaque"))))))
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm global)))))
                           (fun lc _ loc -> 
# 128 "vernac/g_proofs.mlg"
                                               HintsTransparency (HintsReferences lc, false) 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Transparent"))))))
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm global)))))
                           (fun lc _ loc -> 
# 127 "vernac/g_proofs.mlg"
                                                    HintsTransparency (HintsReferences lc, true) 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Projections"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Opaque"))))))
                           (fun _ _ loc -> 
# 126 "vernac/g_proofs.mlg"
                                                 HintsTransparency (HintsProjections, false) 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Projections"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Transparent"))))))
                           (fun _ _ loc -> 
# 125 "vernac/g_proofs.mlg"
                                                      HintsTransparency (HintsProjections, true) 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Constants"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Opaque"))))))
                           (fun _ _ loc -> 
# 124 "vernac/g_proofs.mlg"
                                               HintsTransparency (HintsConstants, false) 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Constants"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Transparent"))))))
                           (fun _ _ loc -> 
# 123 "vernac/g_proofs.mlg"
                                                    HintsTransparency (HintsConstants, true) 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Variables"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Opaque"))))))
                           (fun _ _ loc -> 
# 122 "vernac/g_proofs.mlg"
                                               HintsTransparency (HintsVariables, false) 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Variables"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Transparent"))))))
                           (fun _ _ loc -> 
# 121 "vernac/g_proofs.mlg"
                                                    HintsTransparency (HintsVariables, true) 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Immediate"))))))
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm reference_or_constr)))))
                           (fun lc _ loc -> 
# 120 "vernac/g_proofs.mlg"
                                                               HintsImmediate lc 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Resolve"))))))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("<-")))))
                             ((Procq.Symbol.list1 ((Procq.Symbol.nterm global)))))
                            ((Procq.Symbol.opt (Procq.Symbol.nterm natural))))
                           (fun n lc _ _ loc -> 
# 119 "vernac/g_proofs.mlg"
            HintsResolveIFF (false, lc, n) 
                                                );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Resolve"))))))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("->")))))
                             ((Procq.Symbol.list1 ((Procq.Symbol.nterm global)))))
                            ((Procq.Symbol.opt (Procq.Symbol.nterm natural))))
                           (fun n lc _ _ loc -> 
# 117 "vernac/g_proofs.mlg"
            HintsResolveIFF (true, lc, n) 
                                                );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Resolve"))))))
                             ((Procq.Symbol.list1 ((Procq.Symbol.nterm reference_or_constr)))))
                            ((Procq.Symbol.nterm hint_info)))
                           (fun info lc _ loc -> 
# 115 "vernac/g_proofs.mlg"
            HintsResolve (List.map (fun x -> (info, true, x)) lc) 
                                                 )])]))
  in let () = assert (Procq.Entry.is_empty constr_body) in
  let () =
  Procq.grammar_extend constr_body
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                               ((Procq.Symbol.nterm lconstr)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                             ((Procq.Symbol.nterm lconstr)))
                            (fun c _ t _ loc -> 
# 135 "vernac/g_proofs.mlg"
                                                 CAst.make ~loc @@ CCast(c,Some C.DEFAULTcast, t) 
                                                );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                            ((Procq.Symbol.nterm lconstr)))
                           (fun c _ loc -> 
# 134 "vernac/g_proofs.mlg"
                               c 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty mode) in
  let () =
  Procq.grammar_extend mode
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.list1 ((Procq.Symbol.rules 
                             [Procq.Rules.make (Procq.Rule.next_norec
                                                (Procq.Rule.stop)
                                                ((Procq.Symbol.token (Tok.PKEYWORD ("-")))))
                                               (fun _ loc -> 
# 140 "vernac/g_proofs.mlg"
                             ModeOutput 
                                                             );
                             Procq.Rules.make (Procq.Rule.next_norec
                                               (Procq.Rule.stop)
                                               ((Procq.Symbol.token (Tok.PKEYWORD ("!")))))
                                              (fun _ loc -> 
# 139 "vernac/g_proofs.mlg"
                             ModeNoHeadEvar 
                                                            );
                             Procq.Rules.make (Procq.Rule.next_norec
                                               (Procq.Rule.stop)
                                               ((Procq.Symbol.token (Tok.PKEYWORD ("+")))))
                                              (fun _ loc -> 
# 138 "vernac/g_proofs.mlg"
                             ModeInput 
                                                            )])))))
                            (fun l loc -> 
# 140 "vernac/g_proofs.mlg"
                                                 l 
                                          )])]))
  in ()

