
# 13 "vernac/g_vernac.mlg"
 

open Pp
open Util
open Names
open Glob_term
open Vernacexpr
open Constrexpr
open Constrexpr_ops
open Extend
open Decls
open Declaremods

module C = Constr

open Procq
open Procq.Prim
open Procq.Constr
open Procq.Module
open Pvernac.Vernac_
open Attributes

(* Rem: do not join the different GEXTEND into one, it breaks native *)
(* compilation on PowerPC and Sun architectures *)

let query_command = Entry.make "query_command"

let search_query = Entry.make "search_query"
let search_queries = Entry.make "search_queries"

let subprf = Entry.make "subprf"
let subprf_with_selector = Entry.make "subprf_with_selector"

let quoted_attributes = Entry.make "quoted_attributes"
let attribute_list = Entry.make "attribute_list"
let coercion_class = Entry.make "coercion_class"
let thm_token = Entry.make "thm_token"
let def_token = Entry.make "def_token"
let assumption_token = Entry.make "assumption_token"
let def_body = Entry.make "def_body"
let notation_declaration = Entry.make "notation_declaration"
let decl_notations = Entry.make "decl_notations"
let record_field = Entry.make "record_field"
let of_type = Entry.make "of_type"
let of_type_inst = Entry.make "of_type_inst"
let section_subset_expr = Entry.make "section_subset_expr"
let scope_delimiter = Entry.make "scope_delimiter"
let syntax_modifiers = Entry.make "syntax_modifiers"
let goal_selector = Entry.make "goal_selector"
let toplevel_selector = Entry.make "toplevel_selector"

let make_bullet s =
  let open Proof_bullet in
  let n = String.length s in
  match s.[0] with
  | '-' -> Dash n
  | '+' -> Plus n
  | '*' -> Star n
  | _ -> assert false

(* For now we just keep the top-level location of the whole
   vernacular, that is to say, including attributes and control flags;
   this is not very convenient for advanced clients tho, so in the
   future it'd be cool to actually locate the attributes and control
   flags individually too. *)
let add_control_flag ~loc ~flag { CAst.v = cmd } =
  CAst.make ~loc { cmd with control = flag :: cmd.control }

let test_hash_ident =
  let open Procq.Lookahead in
  to_entry "test_hash_ident" begin
    lk_kw "#" >> lk_ident >> check_no_space
  end

let test_id_colon =
  let open Procq.Lookahead in
  to_entry "test_id_colon" begin
    lk_ident >> lk_kw ":"
  end

let warn_chdir_pwd = CWarnings.create ~name:"change-dir-pwd-deprecated" ~category:Deprecation.Version.v8_20
    (fun () -> strbrk "Command \"Cd\" as a synonym of \"Pwd\" is deprecated." ++ spc () ++
               strbrk "Use \"Pwd\" instead.")



let _ =
  let decorated_vernac = Procq.Entry.make "decorated_vernac"
  and attribute = Procq.Entry.make "attribute"
  and attr_value = Procq.Entry.make "attr_value"
  and legacy_attr = Procq.Entry.make "legacy_attr"
  and vernac = Procq.Entry.make "vernac"
  and vernac_aux = Procq.Entry.make "vernac_aux"
  in
  let () =
  Procq.grammar_extend vernac_control
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm decorated_vernac)))
                            (fun v loc -> 
# 117 "vernac/g_vernac.mlg"
          let (attrs, expr) = v in CAst.make ~loc { control = []; attrs; expr = expr } 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Succeed"))))))
                            ((Procq.Symbol.nterm vernac_control)))
                           (fun c _ loc -> 
# 115 "vernac/g_vernac.mlg"
          add_control_flag ~loc ~flag:ControlSucceed c 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Fail"))))))
                            ((Procq.Symbol.nterm vernac_control)))
                           (fun c _ loc -> 
# 113 "vernac/g_vernac.mlg"
          add_control_flag ~loc ~flag:ControlFail c 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Timeout"))))))
                             ((Procq.Symbol.nterm natural)))
                            ((Procq.Symbol.nterm vernac_control)))
                           (fun c n _ loc -> 
# 111 "vernac/g_vernac.mlg"
          add_control_flag ~loc ~flag:(ControlTimeout n) c 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Redirect"))))))
                             ((Procq.Symbol.nterm ne_string)))
                            ((Procq.Symbol.nterm vernac_control)))
                           (fun c s _ loc -> 
# 109 "vernac/g_vernac.mlg"
          add_control_flag ~loc ~flag:(ControlRedirect s) c 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Profile"))))))
                             ((Procq.Symbol.opt (Procq.Symbol.token (Tok.PSTRING (None))))))
                            ((Procq.Symbol.nterm vernac_control)))
                           (fun c f _ loc -> 
# 107 "vernac/g_vernac.mlg"
          add_control_flag ~loc ~flag:(ControlProfile f) c 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Instructions"))))))
                            ((Procq.Symbol.nterm vernac_control)))
                           (fun c _ loc -> 
# 105 "vernac/g_vernac.mlg"
          add_control_flag ~loc ~flag:ControlInstructions c 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Time"))))))
                            ((Procq.Symbol.nterm vernac_control)))
                           (fun c _ loc -> 
# 103 "vernac/g_vernac.mlg"
          add_control_flag ~loc ~flag:ControlTime c 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty decorated_vernac) in
  let () =
  Procq.grammar_extend decorated_vernac
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm quoted_attributes)))
                             ((Procq.Symbol.nterm vernac)))
                            (fun fv a loc -> 
# 122 "vernac/g_vernac.mlg"
          let (f, v) = fv in (List.append a f, v) 
                                             )])]))
  in let () = assert (Procq.Entry.is_empty quoted_attributes) in
  let () =
  Procq.grammar_extend quoted_attributes
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.list0 (Procq.Symbol.rules 
                             [Procq.Rules.make (Procq.Rule.next_norec
                                                (Procq.Rule.next_norec
                                                 (Procq.Rule.next_norec
                                                  (Procq.Rule.stop)
                                                  ((Procq.Symbol.token (Tok.PKEYWORD ("#[")))))
                                                 ((Procq.Symbol.nterm attribute_list)))
                                                ((Procq.Symbol.token (Tok.PKEYWORD ("]")))))
                                               (fun _ a _ loc -> 
# 125 "vernac/g_vernac.mlg"
                                                         a 
                                                                 )]))))
                            (fun l loc -> 
# 125 "vernac/g_vernac.mlg"
                                                                    List.flatten l 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty attribute_list) in
  let () =
  Procq.grammar_extend attribute_list
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.list1sep ((Procq.Symbol.nterm attribute)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                            (fun a loc -> 
# 129 "vernac/g_vernac.mlg"
                                         a 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty attribute) in
  let () =
  Procq.grammar_extend attribute
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("using"))))))
                             ((Procq.Symbol.nterm attr_value)))
                            (fun v _ loc -> 
# 135 "vernac/g_vernac.mlg"
                                            CAst.make ~loc ("using", v) 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm ident)))
                            ((Procq.Symbol.nterm attr_value)))
                           (fun v k loc -> 
# 133 "vernac/g_vernac.mlg"
                                        CAst.make ~loc (Names.Id.to_string k, v) 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty attr_value) in
  let () =
  Procq.grammar_extend attr_value
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 142 "vernac/g_vernac.mlg"
             VernacFlagEmpty 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                             ((Procq.Symbol.nterm attribute_list)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                           (fun _ a _ loc -> 
# 141 "vernac/g_vernac.mlg"
                                            VernacFlagList a 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("=")))))
                            ((Procq.Symbol.nterm qualid)))
                           (fun v _ loc -> 
# 140 "vernac/g_vernac.mlg"
                              VernacFlagLeaf (FlagQualid v) 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("=")))))
                            ((Procq.Symbol.nterm string)))
                           (fun v _ loc -> 
# 139 "vernac/g_vernac.mlg"
                              VernacFlagLeaf (FlagString v) 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty legacy_attr) in
  let () =
  Procq.grammar_extend legacy_attr
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Program"))))))
                            (fun _ loc -> 
# 161 "vernac/g_vernac.mlg"
          CAst.make ~loc ("program", VernacFlagEmpty) 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Private"))))))
                           (fun _ loc -> 
# 159 "vernac/g_vernac.mlg"
          CAst.make ~loc ("private", VernacFlagList [CAst.make ~loc ("matching", VernacFlagEmpty)]) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("NonCumulative"))))))
                           (fun _ loc -> 
# 157 "vernac/g_vernac.mlg"
          CAst.make ~loc ("universes", VernacFlagList [CAst.make ~loc ("cumulative", VernacFlagLeaf (FlagQualid (Libnames.qualid_of_string "no")))]) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Cumulative"))))))
                           (fun _ loc -> 
# 155 "vernac/g_vernac.mlg"
          CAst.make ~loc ("universes", VernacFlagList [CAst.make ~loc ("cumulative", VernacFlagEmpty)]) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Monomorphic"))))))
                           (fun _ loc -> 
# 153 "vernac/g_vernac.mlg"
          Attributes.vernac_monomorphic_flag (Some loc) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Polymorphic"))))))
                           (fun _ loc -> 
# 151 "vernac/g_vernac.mlg"
          Attributes.vernac_polymorphic_flag (Some loc) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Global"))))))
                           (fun _ loc -> 
# 149 "vernac/g_vernac.mlg"
          CAst.make ~loc ("global", VernacFlagEmpty) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Local"))))))
                           (fun _ loc -> 
# 147 "vernac/g_vernac.mlg"
          CAst.make ~loc ("local", VernacFlagEmpty) 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty vernac) in
  let () =
  Procq.grammar_extend vernac
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.list0 (Procq.Symbol.nterm legacy_attr))))
                             ((Procq.Symbol.nterm vernac_aux)))
                            (fun v attrs loc -> 
# 165 "vernac/g_vernac.mlg"
                                                       (attrs, v) 
                                                )])]))
  in let () = assert (Procq.Entry.is_empty vernac_aux) in
  let () =
  Procq.grammar_extend vernac_aux
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm syntax)))
                             ((Procq.Symbol.token (Tok.PKEYWORD (".")))))
                            (fun _ c loc -> 
# 173 "vernac/g_vernac.mlg"
                             c 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm command)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (".")))))
                           (fun _ c loc -> 
# 172 "vernac/g_vernac.mlg"
                              c 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm gallina_ext)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (".")))))
                           (fun _ g loc -> 
# 171 "vernac/g_vernac.mlg"
                                  g 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm gallina)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (".")))))
                           (fun _ g loc -> 
# 170 "vernac/g_vernac.mlg"
                              VernacSynPure g 
                                           )])]))
  in let () =
  Procq.grammar_extend vernac_aux
  (Procq.Fresh
  (Gramlib.Gramext.Last, [(None, None,
                          [Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm command_entry)))
                           (fun prfcom loc -> 
# 177 "vernac/g_vernac.mlg"
                                    prfcom 
                                              )])]))
  in let () = assert (Procq.Entry.is_empty noedit_mode) in
  let () =
  Procq.grammar_extend noedit_mode
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm query_command)))
                            (fun c loc -> 
# 180 "vernac/g_vernac.mlg"
                                          VernacSynPure (c None) 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty subprf) in
  let () =
  Procq.grammar_extend subprf
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                            (fun _ loc -> 
# 185 "vernac/g_vernac.mlg"
               VernacEndSubproof 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PBULLET (None)))))
                           (fun s loc -> 
# 184 "vernac/g_vernac.mlg"
                      VernacBullet (make_bullet s) 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty subprf_with_selector) in
  let () =
  Procq.grammar_extend subprf_with_selector
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm query_command)))
                            (fun c loc -> 
# 191 "vernac/g_vernac.mlg"
                             c 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("{")))))
                           (fun _ loc -> 
# 189 "vernac/g_vernac.mlg"
               fun g -> VernacSubproof g 
                                         )])]))
  in ()


# 197 "vernac/g_vernac.mlg"
 

let warn_plural_command =
  CWarnings.create ~name:"plural-command" ~category:CWarnings.CoreCategories.pedantic ~default:CWarnings.Disabled
         (fun kwd -> strbrk (Printf.sprintf "Command \"%s\" expects more than one assumption." kwd))

let test_plural_form loc kwd = function
  | [(_,([_],_))] ->
     warn_plural_command ~loc kwd
  | _ -> ()

let test_plural_form_types loc kwd = function
  | [([_],_)] ->
     warn_plural_command ~loc kwd
  | _ -> ()

let test_plural_form_rules loc kwd = function
  | [_] ->
     warn_plural_command ~loc kwd
  | _ -> ()

let lname_of_lident : lident -> lname =
  CAst.map (fun s -> Name s)

let name_of_ident_decl : ident_decl -> name_decl =
  on_fst lname_of_lident

let test_variance_ident =
  let open Procq.Lookahead in
  to_entry "test_variance_ident" begin
    lk_kws ["=";"+";"*"] >> lk_ident
  end

let test_univ_decl =
  let open Procq.Lookahead in
  to_entry "test_univ_decl" (lk_ident_list >> lk_kw "|" >> lk_ident_list >> (lk_kw "+" <+> lk_empty) >> (lk_kw "|" <+> lk_kw "|}"))

let test_cumul_univ_decl =
  let open Procq.Lookahead in
  let lk_list_variance_ident = lk_list (lk_kw "+" <+> lk_kw "*" <+> lk_kw "=" <+> lk_ident) in
  to_entry "test_cumul_univ_decl" (lk_ident_list >> lk_kw "|" >> lk_list_variance_ident >> (lk_kw "+" <+> lk_empty) >> (lk_kw "|" <+> lk_kw "|}"))




let _ =
  let register_token = Procq.Entry.make "register_token"
  and assumptions_token = Procq.Entry.make "assumptions_token"
  and inline = Procq.Entry.make "inline"
  and univ_constraint = Procq.Entry.make "univ_constraint"
  and univ_decl_constraints = Procq.Entry.make "univ_decl_constraints"
  and variance = Procq.Entry.make "variance"
  and variance_identref = Procq.Entry.make "variance_identref"
  and cumul_univ_decl = Procq.Entry.make "cumul_univ_decl"
  and cumul_ident_decl = Procq.Entry.make "cumul_ident_decl"
  and inductive_token = Procq.Entry.make "inductive_token"
  and finite_token = Procq.Entry.make "finite_token"
  and reduce = Procq.Entry.make "reduce"
  and decl_sep = Procq.Entry.make "decl_sep"
  and opt_constructors_or_fields =
    Procq.Entry.make "opt_constructors_or_fields"
  and constructors_or_record = Procq.Entry.make "constructors_or_record"
  and default_inhabitant_ident = Procq.Entry.make "default_inhabitant_ident"
  and opt_coercion = Procq.Entry.make "opt_coercion"
  and cofix_definition = Procq.Entry.make "cofix_definition"
  and rw_pattern = Procq.Entry.make "rw_pattern"
  and rewrite_rule = Procq.Entry.make "rewrite_rule"
  and scheme = Procq.Entry.make "scheme"
  and scheme_kind = Procq.Entry.make "scheme_kind"
  and scheme_type = Procq.Entry.make "scheme_type"
  and record_fields = Procq.Entry.make "record_fields"
  and field_body = Procq.Entry.make "field_body"
  and record_binder = Procq.Entry.make "record_binder"
  and assum_list = Procq.Entry.make "assum_list"
  and assum_coe = Procq.Entry.make "assum_coe"
  and assumpt = Procq.Entry.make "assumpt"
  and constructor_type = Procq.Entry.make "constructor_type"
  and constructor = Procq.Entry.make "constructor"
  in
  let () = assert (Procq.Entry.is_empty gallina) in
  let () =
  Procq.grammar_extend gallina
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next (Procq.Rule.stop)
                                  ((Procq.Symbol.token (Tok.PIDENT (Some
                                  ("Rewrite"))))))
                                 ((Procq.Symbol.token (Tok.PIDENT (Some
                                 ("Rules"))))))
                                ((Procq.Symbol.nterm identref)))
                               ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                              ((Procq.Symbol.opt (Procq.Symbol.token (Tok.PKEYWORD ("|"))))))
                             ((Procq.Symbol.list1sep ((Procq.Symbol.nterm rewrite_rule)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD ("|"))])) false)))
                            (fun rules _ _ id _ _ loc -> 
# 303 "vernac/g_vernac.mlg"
            test_plural_form_rules loc "Rewrite Rules" rules;
            VernacAddRewRule (id, rules) 
                                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.token (Tok.PIDENT (Some
                                 ("Rewrite"))))))
                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                ("Rule"))))))
                               ((Procq.Symbol.nterm identref)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                             ((Procq.Symbol.opt (Procq.Symbol.token (Tok.PKEYWORD ("|"))))))
                            ((Procq.Symbol.list1sep ((Procq.Symbol.nterm rewrite_rule)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD ("|"))])) false)))
                           (fun rules _ _ id _ _ loc -> 
# 301 "vernac/g_vernac.mlg"
            VernacAddRewRule (id, rules) 
                                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Constraint"))))))
                            ((Procq.Symbol.list1sep ((Procq.Symbol.nterm univ_constraint)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                           (fun l _ loc -> 
# 299 "vernac/g_vernac.mlg"
                                                                   VernacConstraint l 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Universes"))))))
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm identref)))))
                           (fun l _ loc -> 
# 298 "vernac/g_vernac.mlg"
                                                   VernacUniverse l 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Universe"))))))
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm identref)))))
                           (fun l _ loc -> 
# 297 "vernac/g_vernac.mlg"
                                                  VernacUniverse l 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                ("Primitive"))))))
                               ((Procq.Symbol.nterm ident_decl)))
                              ((Procq.Symbol.opt (Procq.Symbol.rules 
                              [Procq.Rules.make (Procq.Rule.next_norec
                                                 (Procq.Rule.next_norec
                                                  (Procq.Rule.stop)
                                                  ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                                                 ((Procq.Symbol.nterm lconstr)))
                                                (fun typ _ loc -> 
# 295 "vernac/g_vernac.mlg"
                                                                                   typ 
                                                                  )]))))
                             ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                            ((Procq.Symbol.nterm register_token)))
                           (fun r _ typopt id _ loc -> 
# 296 "vernac/g_vernac.mlg"
            VernacPrimitive(id, r, typopt) 
                                                       );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Register"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Inline"))))))
                            ((Procq.Symbol.nterm global)))
                           (fun g _ _ loc -> 
# 294 "vernac/g_vernac.mlg"
            VernacRegister(g, RegisterInline) 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next (Procq.Rule.stop)
                                  ((Procq.Symbol.token (Tok.PIDENT (Some
                                  ("Register"))))))
                                 ((Procq.Symbol.token (Tok.PIDENT (Some
                                 ("Scheme"))))))
                                ((Procq.Symbol.nterm global)))
                               ((Procq.Symbol.token (Tok.PKEYWORD ("as")))))
                              ((Procq.Symbol.nterm qualid)))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("for"))))))
                            ((Procq.Symbol.nterm global)))
                           (fun g' _ qid _ g _ _ loc -> 
# 292 "vernac/g_vernac.mlg"
          VernacRegister(g, RegisterScheme {inductive = g'; scheme_kind = qid}) 
                                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Register"))))))
                              ((Procq.Symbol.nterm global)))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("as")))))
                            ((Procq.Symbol.nterm qualid)))
                           (fun quid _ g _ loc -> 
# 290 "vernac/g_vernac.mlg"
            VernacRegister(g, RegisterCoqlib quid) 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                ("Combined"))))))
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Scheme"))))))
                              ((Procq.Symbol.nterm identref)))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("from"))))))
                            ((Procq.Symbol.list1sep ((Procq.Symbol.nterm identref)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                           (fun l _ id _ _ loc -> 
# 288 "vernac/g_vernac.mlg"
                                              VernacCombinedScheme (id, l) 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                ("Scheme"))))))
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Boolean"))))))
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Equality"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("for"))))))
                            ((Procq.Symbol.nterm smart_global)))
                           (fun id _ _ _ _ loc -> 
# 286 "vernac/g_vernac.mlg"
            VernacSchemeEquality (SchemeBooleanEquality,id) 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Scheme"))))))
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Equality"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("for"))))))
                            ((Procq.Symbol.nterm smart_global)))
                           (fun id _ _ _ loc -> 
# 284 "vernac/g_vernac.mlg"
            VernacSchemeEquality (SchemeEquality,id) 
                                                );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Scheme"))))))
                            ((Procq.Symbol.list1sep ((Procq.Symbol.nterm scheme)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD ("with"))])) false)))
                           (fun l _ loc -> 
# 282 "vernac/g_vernac.mlg"
                                                         VernacScheme l 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Let"))))))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("CoFixpoint")))))
                            ((Procq.Symbol.list1sep ((Procq.Symbol.nterm cofix_definition)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD ("with"))])) false)))
                           (fun corecs _ _ loc -> 
# 281 "vernac/g_vernac.mlg"
            VernacCoFixpoint (DoDischarge, corecs) 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("CoFixpoint")))))
                            ((Procq.Symbol.list1sep ((Procq.Symbol.nterm cofix_definition)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD ("with"))])) false)))
                           (fun corecs _ loc -> 
# 279 "vernac/g_vernac.mlg"
            VernacCoFixpoint (NoDischarge, corecs) 
                                                );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Let"))))))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("Fixpoint")))))
                            ((Procq.Symbol.list1sep ((Procq.Symbol.nterm fix_definition)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD ("with"))])) false)))
                           (fun recs _ _ loc -> 
# 277 "vernac/g_vernac.mlg"
            VernacFixpoint (DoDischarge, List.split recs) 
                                                );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("Fixpoint")))))
                            ((Procq.Symbol.list1sep ((Procq.Symbol.nterm fix_definition)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD ("with"))])) false)))
                           (fun recs _ loc -> 
# 275 "vernac/g_vernac.mlg"
            VernacFixpoint (NoDischarge, List.split recs) 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm inductive_token)))
                            ((Procq.Symbol.list1sep ((Procq.Symbol.nterm inductive_or_record_definition)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD ("with"))])) false)))
                           (fun indl f loc -> 
# 273 "vernac/g_vernac.mlg"
            VernacInductive (f, indl) 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm finite_token)))
                            ((Procq.Symbol.nterm inductive_or_record_definition)))
                           (fun ind f loc -> 
# 271 "vernac/g_vernac.mlg"
            VernacInductive (f, [ind]) 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Let"))))))
                             ((Procq.Symbol.nterm ident_decl)))
                            ((Procq.Symbol.nterm def_body)))
                           (fun b id _ loc -> 
# 268 "vernac/g_vernac.mlg"
            VernacDefinition ((DoDischarge, Let), name_of_ident_decl id, b) 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Symbols"))))))
                            ((Procq.Symbol.nterm assum_list)))
                           (fun bl _ loc -> 
# 265 "vernac/g_vernac.mlg"
            test_plural_form loc "Symbols" bl;
            VernacSymbol bl 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Symbol"))))))
                            ((Procq.Symbol.nterm assum_list)))
                           (fun bl _ loc -> 
# 263 "vernac/g_vernac.mlg"
            VernacSymbol bl 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm def_token)))
                             ((Procq.Symbol.nterm ident_decl)))
                            ((Procq.Symbol.nterm def_body)))
                           (fun b id d loc -> 
# 261 "vernac/g_vernac.mlg"
            VernacDefinition (d, name_of_ident_decl id, b) 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm assumptions_token)))
                             ((Procq.Symbol.nterm inline)))
                            ((Procq.Symbol.nterm assum_list)))
                           (fun bl nl tk loc -> 
# 257 "vernac/g_vernac.mlg"
            let (kwd,stre) = tk in
            test_plural_form loc kwd bl;
            VernacAssumption (stre, nl, bl) 
                                                );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm assumption_token)))
                             ((Procq.Symbol.nterm inline)))
                            ((Procq.Symbol.nterm assum_list)))
                           (fun bl nl stre loc -> 
# 255 "vernac/g_vernac.mlg"
            VernacAssumption (stre, nl, bl) 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.nterm thm_token)))
                                ((Procq.Symbol.nterm ident_decl)))
                               ((Procq.Symbol.nterm binders)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                             ((Procq.Symbol.nterm lconstr)))
                            ((Procq.Symbol.list0 (Procq.Symbol.rules 
                            [Procq.Rules.make (Procq.Rule.next_norec
                                               (Procq.Rule.next_norec
                                                (Procq.Rule.next_norec
                                                 (Procq.Rule.next_norec
                                                  (Procq.Rule.next_norec
                                                   (Procq.Rule.stop)
                                                   ((Procq.Symbol.token (Tok.PKEYWORD ("with")))))
                                                  ((Procq.Symbol.nterm ident_decl)))
                                                 ((Procq.Symbol.nterm binders)))
                                                ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                                               ((Procq.Symbol.nterm lconstr)))
                                              (fun c _ bl id _ loc -> 
                                              
# 252 "vernac/g_vernac.mlg"
            (id,(bl,c)) 
                                              )]))))
                           (fun l c _ bl id thm loc -> 
# 253 "vernac/g_vernac.mlg"
            VernacStartTheoremProof (thm, (id,(bl,c))::l) 
                                                       )])]))
  in let () = assert (Procq.Entry.is_empty register_token) in
  let () =
  Procq.grammar_extend register_token
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.nterm test_hash_ident)))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("#")))))
                             ((Procq.Symbol.token (Tok.PIDENT (None)))))
                            (fun r _ _ loc -> 
# 308 "vernac/g_vernac.mlg"
                                             CPrimitives.parse_op_or_type ~loc r 
                                              )])]))
  in let () = assert (Procq.Entry.is_empty thm_token) in
  let () =
  Procq.grammar_extend thm_token
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Property"))))))
                            (fun _ loc -> 
# 317 "vernac/g_vernac.mlg"
                              Property 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Proposition"))))))
                           (fun _ loc -> 
# 316 "vernac/g_vernac.mlg"
                                 Proposition 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Corollary"))))))
                           (fun _ loc -> 
# 315 "vernac/g_vernac.mlg"
                               Corollary 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Remark"))))))
                           (fun _ loc -> 
# 314 "vernac/g_vernac.mlg"
                            Remark 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Fact"))))))
                           (fun _ loc -> 
# 313 "vernac/g_vernac.mlg"
                          Fact 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Lemma"))))))
                           (fun _ loc -> 
# 312 "vernac/g_vernac.mlg"
                           Lemma 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("Theorem")))))
                           (fun _ loc -> 
# 311 "vernac/g_vernac.mlg"
                       Theorem 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty def_token) in
  let () =
  Procq.grammar_extend def_token
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("SubClass"))))))
                            (fun _ loc -> 
# 322 "vernac/g_vernac.mlg"
                              (NoDischarge,SubClass) 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Example"))))))
                           (fun _ loc -> 
# 321 "vernac/g_vernac.mlg"
                             (NoDischarge,Example) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("Definition")))))
                           (fun _ loc -> 
# 320 "vernac/g_vernac.mlg"
                          (NoDischarge,Definition) 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty assumption_token) in
  let () =
  Procq.grammar_extend assumption_token
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Conjecture"))))))
                            (fun _ loc -> 
# 329 "vernac/g_vernac.mlg"
                                (NoDischarge, Conjectural) 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("Parameter")))))
                           (fun _ loc -> 
# 328 "vernac/g_vernac.mlg"
                         (NoDischarge, Definitional) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("Axiom")))))
                           (fun _ loc -> 
# 327 "vernac/g_vernac.mlg"
                     (NoDischarge, Logical) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("Variable")))))
                           (fun _ loc -> 
# 326 "vernac/g_vernac.mlg"
                        (DoDischarge, Definitional) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("Hypothesis")))))
                           (fun _ loc -> 
# 325 "vernac/g_vernac.mlg"
                          (DoDischarge, Logical) 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty assumptions_token) in
  let () =
  Procq.grammar_extend assumptions_token
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Conjectures"))))))
                            (fun _ loc -> 
# 336 "vernac/g_vernac.mlg"
                                 ("Conjectures", (NoDischarge, Conjectural)) 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Parameters"))))))
                           (fun _ loc -> 
# 335 "vernac/g_vernac.mlg"
                                ("Parameters", (NoDischarge, Definitional)) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Axioms"))))))
                           (fun _ loc -> 
# 334 "vernac/g_vernac.mlg"
                            ("Axioms", (NoDischarge, Logical)) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Variables"))))))
                           (fun _ loc -> 
# 333 "vernac/g_vernac.mlg"
                               ("Variables", (DoDischarge, Definitional)) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Hypotheses"))))))
                           (fun _ loc -> 
# 332 "vernac/g_vernac.mlg"
                                ("Hypotheses", (DoDischarge, Logical)) 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty inline) in
  let () =
  Procq.grammar_extend inline
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 341 "vernac/g_vernac.mlg"
             NoInline 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Inline"))))))
                           (fun _ loc -> 
# 340 "vernac/g_vernac.mlg"
                            DefaultInline 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Inline"))))))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                             ((Procq.Symbol.nterm natural)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                           (fun _ i _ _ loc -> 
# 339 "vernac/g_vernac.mlg"
                                                   InlineAt i 
                                               )])]))
  in let () = assert (Procq.Entry.is_empty univ_constraint) in
  let () =
  Procq.grammar_extend univ_constraint
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.nterm universe_name)))
                              ((Procq.Symbol.rules [Procq.Rules.make 
                                                   (Procq.Rule.next_norec
                                                    (Procq.Rule.stop)
                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("<=")))))
                                                   (fun _ loc -> 
# 344 "vernac/g_vernac.mlg"
                                                                                       Univ.Le 
                                                                 );
                                                   Procq.Rules.make (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("=")))))
                                                                    (fun _
                                                                    loc -> 
                                                                    
# 344 "vernac/g_vernac.mlg"
                                                                 Univ.Eq 
                                                                    );
                                                   Procq.Rules.make (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("<")))))
                                                                    (fun _
                                                                    loc -> 
                                                                    
# 344 "vernac/g_vernac.mlg"
                                            Univ.Lt 
                                                                    )])))
                             ((Procq.Symbol.nterm universe_name)))
                            (fun r ord l loc -> 
# 345 "vernac/g_vernac.mlg"
                               (l, ord, r) 
                                                )])]))
  in let () = assert (Procq.Entry.is_empty univ_decl_constraints) in
  let () =
  Procq.grammar_extend univ_decl_constraints
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.rules [Procq.Rules.make (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.nterm bar_cbrace)))
                                                                    (fun _
                                                                    loc -> 
                                                                    
# 350 "vernac/g_vernac.mlg"
                                                  false 
                                                                    );
                                                  Procq.Rules.make (Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                                                                   (fun _
                                                                   loc -> 
                                                                   
# 350 "vernac/g_vernac.mlg"
                         true 
                                                                   )])))
                            (fun ext loc -> 
# 350 "vernac/g_vernac.mlg"
                                                                 ([], ext) 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("|")))))
                              ((Procq.Symbol.list0sep ((Procq.Symbol.nterm univ_constraint)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                             ((Procq.Symbol.rules [Procq.Rules.make (Procq.Rule.stop)
                                                                    (fun
                                                                    loc -> 
                                                                    
# 349 "vernac/g_vernac.mlg"
                                       false 
                                                                    );
                                                  Procq.Rules.make (Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("+")))))
                                                                   (fun _
                                                                   loc -> 
                                                                   
# 349 "vernac/g_vernac.mlg"
                         true 
                                                                   )])))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                           (fun _ ext l' _ loc -> 
# 349 "vernac/g_vernac.mlg"
                                                           (l',ext) 
                                                  )])]))
  in let () = assert (Procq.Entry.is_empty univ_decl) in
  let () =
  Procq.grammar_extend univ_decl
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PKEYWORD ("@{")))))
                               ((Procq.Symbol.list0 (Procq.Symbol.nterm identref))))
                              ((Procq.Symbol.rules [Procq.Rules.make 
                                                   (Procq.Rule.stop) (fun
                                                   loc -> 
# 363 "vernac/g_vernac.mlg"
                                                                false 
                                                          );
                                                   Procq.Rules.make (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("+")))))
                                                                    (fun _
                                                                    loc -> 
                                                                    
# 363 "vernac/g_vernac.mlg"
                                                  true 
                                                                    )])))
                             ((Procq.Symbol.nterm univ_decl_constraints)))
                            (fun cs ext l _ loc -> 
# 366 "vernac/g_vernac.mlg"
           let open UState in
         { univdecl_qualities = [];
           univdecl_extensible_qualities = false;
           univdecl_instance = l;
           univdecl_extensible_instance = ext;
           univdecl_constraints = fst cs;
           univdecl_extensible_constraints = snd cs } 
                                                   );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next (Procq.Rule.stop)
                                  ((Procq.Symbol.token (Tok.PKEYWORD ("@{")))))
                                 ((Procq.Symbol.nterm test_univ_decl)))
                                ((Procq.Symbol.list0 (Procq.Symbol.nterm identref))))
                               ((Procq.Symbol.token (Tok.PKEYWORD ("|")))))
                              ((Procq.Symbol.list0 (Procq.Symbol.nterm identref))))
                             ((Procq.Symbol.rules [Procq.Rules.make (Procq.Rule.stop)
                                                                    (fun
                                                                    loc -> 
                                                                    
# 353 "vernac/g_vernac.mlg"
                                                                                                             false 
                                                                    );
                                                  Procq.Rules.make (Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("+")))))
                                                                   (fun _
                                                                   loc -> 
                                                                   
# 353 "vernac/g_vernac.mlg"
                                                                                               true 
                                                                   )])))
                            ((Procq.Symbol.nterm univ_decl_constraints)))
                           (fun cs ext l _ l0 _ _ loc -> 
# 356 "vernac/g_vernac.mlg"
           let open UState in
         { univdecl_qualities = l0;
           univdecl_extensible_qualities = false;
           univdecl_instance = l;
           univdecl_extensible_instance = ext;
           univdecl_constraints = fst cs;
           univdecl_extensible_constraints = snd cs } 
                                                         )])]))
  in let () = assert (Procq.Entry.is_empty variance) in
  let () =
  Procq.grammar_extend variance
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("*")))))
                            (fun _ loc -> 
# 378 "vernac/g_vernac.mlg"
                 UVars.Variance.Irrelevant 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("=")))))
                           (fun _ loc -> 
# 377 "vernac/g_vernac.mlg"
                 UVars.Variance.Invariant 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("+")))))
                           (fun _ loc -> 
# 376 "vernac/g_vernac.mlg"
                 UVars.Variance.Covariant 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty variance_identref) in
  let () =
  Procq.grammar_extend variance_identref
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.nterm test_variance_ident)))
                              ((Procq.Symbol.nterm variance)))
                             ((Procq.Symbol.nterm identref)))
                            (fun id v _ loc -> 
# 383 "vernac/g_vernac.mlg"
                                                              (id, Some v) 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm identref)))
                           (fun id loc -> 
# 382 "vernac/g_vernac.mlg"
                           (id, None) 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty cumul_univ_decl) in
  let () =
  Procq.grammar_extend cumul_univ_decl
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PKEYWORD ("@{")))))
                               ((Procq.Symbol.list0 (Procq.Symbol.nterm variance_identref))))
                              ((Procq.Symbol.rules [Procq.Rules.make 
                                                   (Procq.Rule.stop) (fun
                                                   loc -> 
# 401 "vernac/g_vernac.mlg"
                                                                            false 
                                                          );
                                                   Procq.Rules.make (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("+")))))
                                                                    (fun _
                                                                    loc -> 
                                                                    
# 401 "vernac/g_vernac.mlg"
                                                              true 
                                                                    )])))
                             ((Procq.Symbol.nterm univ_decl_constraints)))
                            (fun cs ext l _ loc -> 
# 404 "vernac/g_vernac.mlg"
           let open UState in
         { univdecl_qualities = [];
           univdecl_extensible_qualities = false;
           univdecl_instance = l;
           univdecl_extensible_instance = ext;
           univdecl_constraints = fst cs;
           univdecl_extensible_constraints = snd cs } 
                                                   );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next (Procq.Rule.stop)
                                  ((Procq.Symbol.token (Tok.PKEYWORD ("@{")))))
                                 ((Procq.Symbol.nterm test_cumul_univ_decl)))
                                ((Procq.Symbol.list0 (Procq.Symbol.nterm identref))))
                               ((Procq.Symbol.token (Tok.PKEYWORD ("|")))))
                              ((Procq.Symbol.list0 (Procq.Symbol.nterm variance_identref))))
                             ((Procq.Symbol.rules [Procq.Rules.make (Procq.Rule.stop)
                                                                    (fun
                                                                    loc -> 
                                                                    
# 390 "vernac/g_vernac.mlg"
                                                                    false 
                                                                    );
                                                  Procq.Rules.make (Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("+")))))
                                                                   (fun _
                                                                   loc -> 
                                                                   
# 390 "vernac/g_vernac.mlg"
                                                      true 
                                                                   )])))
                            ((Procq.Symbol.nterm univ_decl_constraints)))
                           (fun cs ext l _ l0 _ _ loc -> 
# 393 "vernac/g_vernac.mlg"
           let open UState in
         { univdecl_qualities = l0;
           univdecl_extensible_qualities = false;
           univdecl_instance = l;
           univdecl_extensible_instance = ext;
           univdecl_constraints = fst cs;
           univdecl_extensible_constraints = snd cs } 
                                                         )])]))
  in let () = assert (Procq.Entry.is_empty ident_decl) in
  let () =
  Procq.grammar_extend ident_decl
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm identref)))
                             ((Procq.Symbol.opt (Procq.Symbol.nterm univ_decl))))
                            (fun l i loc -> 
# 414 "vernac/g_vernac.mlg"
                                             (i, l) 
                                            )])]))
  in let () = assert (Procq.Entry.is_empty cumul_ident_decl) in
  let () =
  Procq.grammar_extend cumul_ident_decl
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm identref)))
                             ((Procq.Symbol.opt (Procq.Symbol.nterm cumul_univ_decl))))
                            (fun l i loc -> 
# 418 "vernac/g_vernac.mlg"
                                                   (i, l) 
                                            )])]))
  in let () = assert (Procq.Entry.is_empty inductive_token) in
  let () =
  Procq.grammar_extend inductive_token
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("CoInductive"))))))
                            (fun _ loc -> 
# 423 "vernac/g_vernac.mlg"
                                 CoInductive 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Inductive"))))))
                           (fun _ loc -> 
# 422 "vernac/g_vernac.mlg"
                               Inductive_kw 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty finite_token) in
  let () =
  Procq.grammar_extend finite_token
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Class"))))))
                            (fun _ loc -> 
# 429 "vernac/g_vernac.mlg"
                           Class true 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Structure"))))))
                           (fun _ loc -> 
# 428 "vernac/g_vernac.mlg"
                               Structure 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Record"))))))
                           (fun _ loc -> 
# 427 "vernac/g_vernac.mlg"
                            Record 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Variant"))))))
                           (fun _ loc -> 
# 426 "vernac/g_vernac.mlg"
                             Variant 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty def_body) in
  let () =
  Procq.grammar_extend def_body
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.nterm binders)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                             ((Procq.Symbol.nterm lconstr)))
                            (fun t _ bl loc -> 
# 440 "vernac/g_vernac.mlg"
          ProveBody (bl, t) 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.nterm binders)))
                                ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                               ((Procq.Symbol.nterm lconstr)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                             ((Procq.Symbol.nterm reduce)))
                            ((Procq.Symbol.nterm lconstr)))
                           (fun c red _ t _ bl loc -> 
# 438 "vernac/g_vernac.mlg"
          DefineBody (bl, red, c, Some t) 
                                                      );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.nterm binders)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                             ((Procq.Symbol.nterm reduce)))
                            ((Procq.Symbol.nterm lconstr)))
                           (fun c red _ bl loc -> 
# 434 "vernac/g_vernac.mlg"
          match c.CAst.v with
          | CCast(c, Some C.DEFAULTcast, t) -> DefineBody (bl, red, c, Some t)
          | _ -> DefineBody (bl, red, c, None) 
                                                  )])]))
  in let () = assert (Procq.Entry.is_empty reduce) in
  let () =
  Procq.grammar_extend reduce
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 444 "vernac/g_vernac.mlg"
             None 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Eval"))))))
                             ((Procq.Symbol.nterm red_expr)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("in")))))
                           (fun _ r _ loc -> 
# 443 "vernac/g_vernac.mlg"
                                              Some r 
                                             )])]))
  in let () = assert (Procq.Entry.is_empty notation_declaration) in
  let () =
  Procq.grammar_extend notation_declaration
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.nterm lstring)))
                                ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                               ((Procq.Symbol.nterm constr)))
                              ((Procq.Symbol.nterm syntax_modifiers)))
                             ((Procq.Symbol.opt (Procq.Symbol.rules [Procq.Rules.make 
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                                                                    ((Procq.Symbol.token (Tok.PIDENT (None)))))
                                                                    (fun sc _
                                                                    loc -> 
                                                                    
# 449 "vernac/g_vernac.mlg"
                                           sc 
                                                                    )]))))
                            (fun scopt modl c _ ntn loc -> 
# 450 "vernac/g_vernac.mlg"
        { ntn_decl_string = ntn; ntn_decl_interp = c;
          ntn_decl_scope = scopt;
          ntn_decl_modifiers = modl;
      } 
                                                           )])]))
  in let () = assert (Procq.Entry.is_empty decl_sep) in
  let () =
  Procq.grammar_extend decl_sep
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("and"))))))
                            (fun _ loc -> 
# 456 "vernac/g_vernac.mlg"
                         () 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty decl_notations) in
  let () =
  Procq.grammar_extend decl_notations
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 460 "vernac/g_vernac.mlg"
           [] 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("where")))))
                            ((Procq.Symbol.list1sep ((Procq.Symbol.nterm notation_declaration)) ((Procq.Symbol.rules 
                            [Procq.Rules.make (Procq.Rule.next_norec
                                               (Procq.Rule.stop)
                                               ((Procq.Symbol.nterm decl_sep)))
                                              (fun _ loc -> ())])) false)))
                           (fun l _ loc -> 
# 459 "vernac/g_vernac.mlg"
                                                                  l 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty opt_constructors_or_fields)
  in
  let () =
  Procq.grammar_extend opt_constructors_or_fields
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 466 "vernac/g_vernac.mlg"
             RecordDecl (None, [], None) 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                           (fun _ loc -> 
# 465 "vernac/g_vernac.mlg"
                  Constructors [] 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                            ((Procq.Symbol.nterm constructors_or_record)))
                           (fun lc _ loc -> 
# 464 "vernac/g_vernac.mlg"
                                               lc 
                                            )])]))
  in let () = assert (Procq.Entry.is_empty inductive_or_record_definition)
  in
  let () =
  Procq.grammar_extend inductive_or_record_definition
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next
                                  (Procq.Rule.next (Procq.Rule.stop)
                                   ((Procq.Symbol.nterm opt_coercion)))
                                  ((Procq.Symbol.nterm cumul_ident_decl)))
                                 ((Procq.Symbol.nterm binders)))
                                ((Procq.Symbol.opt (Procq.Symbol.rules 
                                [Procq.Rules.make (Procq.Rule.next_norec
                                                   (Procq.Rule.next_norec
                                                    (Procq.Rule.stop)
                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("|")))))
                                                   ((Procq.Symbol.nterm binders)))
                                                  (fun p _ loc -> 
# 470 "vernac/g_vernac.mlg"
                                               p 
                                                                  )]))))
                               ((Procq.Symbol.opt (Procq.Symbol.rules 
                               [Procq.Rules.make (Procq.Rule.next_norec
                                                  (Procq.Rule.next_norec
                                                   (Procq.Rule.stop)
                                                   ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                                                  ((Procq.Symbol.nterm lconstr)))
                                                 (fun c _ loc -> 
# 471 "vernac/g_vernac.mlg"
                                        c 
                                                                 )]))))
                              ((Procq.Symbol.nterm opt_constructors_or_fields)))
                             ((Procq.Symbol.nterm decl_notations)))
                            (fun ntn lc c extrapar indpar id oc loc -> 
# 473 "vernac/g_vernac.mlg"
             (((oc,id),(indpar,extrapar),c,lc),ntn) 
                                                                    )])]))
  in let () = assert (Procq.Entry.is_empty constructors_or_record) in
  let () =
  Procq.grammar_extend constructors_or_record
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PKEYWORD ("{")))))
                               ((Procq.Symbol.nterm record_fields)))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                             ((Procq.Symbol.nterm default_inhabitant_ident)))
                            (fun id _ fs _ loc -> 
# 486 "vernac/g_vernac.mlg"
                                                                        RecordDecl (None,fs,id) 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.nterm quoted_attributes)))
                                ((Procq.Symbol.nterm identref)))
                               ((Procq.Symbol.token (Tok.PKEYWORD ("{")))))
                              ((Procq.Symbol.nterm record_fields)))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                            ((Procq.Symbol.nterm default_inhabitant_ident)))
                           (fun id _ fs _ cstr attr loc -> 
# 484 "vernac/g_vernac.mlg"
            let () = unsupported_attributes attr in
            RecordDecl (Some cstr,fs,id) 
                                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm quoted_attributes)))
                             ((Procq.Symbol.nterm identref)))
                            ((Procq.Symbol.nterm constructor_type)))
                           (fun c id attr loc -> 
# 481 "vernac/g_vernac.mlg"
            Constructors [ c attr id ] 
                                                 );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.nterm quoted_attributes)))
                               ((Procq.Symbol.nterm identref)))
                              ((Procq.Symbol.nterm constructor_type)))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("|")))))
                            ((Procq.Symbol.list1sep ((Procq.Symbol.nterm constructor)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD ("|"))])) false)))
                           (fun l _ c id attr loc -> 
# 479 "vernac/g_vernac.mlg"
            Constructors ((c attr id)::l) 
                                                     );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("|")))))
                            ((Procq.Symbol.list1sep ((Procq.Symbol.nterm constructor)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD ("|"))])) false)))
                           (fun l _ loc -> 
# 476 "vernac/g_vernac.mlg"
                                                Constructors l 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty default_inhabitant_ident)
  in
  let () =
  Procq.grammar_extend default_inhabitant_ident
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 490 "vernac/g_vernac.mlg"
             None 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("as")))))
                            ((Procq.Symbol.nterm identref)))
                           (fun id _ loc -> 
# 489 "vernac/g_vernac.mlg"
                                 Some id 
                                            )])]))
  in let () = assert (Procq.Entry.is_empty opt_coercion) in
  let () =
  Procq.grammar_extend opt_coercion
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 499 "vernac/g_vernac.mlg"
              NoCoercion 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD (">")))))
                           (fun _ loc -> 
# 498 "vernac/g_vernac.mlg"
                 AddCoercion 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty fix_definition) in
  let () =
  Procq.grammar_extend fix_definition
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.nterm ident_decl)))
                                ((Procq.Symbol.nterm binders_fixannot)))
                               ((Procq.Symbol.nterm type_cstr)))
                              ((Procq.Symbol.opt (Procq.Symbol.rules 
                              [Procq.Rules.make (Procq.Rule.next_norec
                                                 (Procq.Rule.next_norec
                                                  (Procq.Rule.stop)
                                                  ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                                                 ((Procq.Symbol.nterm lconstr)))
                                                (fun def _ loc -> 
# 506 "vernac/g_vernac.mlg"
                                                 def 
                                                                  )]))))
                             ((Procq.Symbol.nterm decl_notations)))
                            (fun notations body_def rtype bl id_decl loc -> 
# 507 "vernac/g_vernac.mlg"
            let binders, rec_order = bl in
            ((rec_order : Constrexpr.fixpoint_order_expr option), {fname = fst id_decl; univs = snd id_decl; binders; rtype; body_def; notations})
          
                                                                    )])]))
  in let () = assert (Procq.Entry.is_empty cofix_definition) in
  let () =
  Procq.grammar_extend cofix_definition
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.nterm ident_decl)))
                                ((Procq.Symbol.nterm binders)))
                               ((Procq.Symbol.nterm type_cstr)))
                              ((Procq.Symbol.opt (Procq.Symbol.rules 
                              [Procq.Rules.make (Procq.Rule.next_norec
                                                 (Procq.Rule.next_norec
                                                  (Procq.Rule.stop)
                                                  ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                                                 ((Procq.Symbol.nterm lconstr)))
                                                (fun def _ loc -> 
# 513 "vernac/g_vernac.mlg"
                                                 def 
                                                                  )]))))
                             ((Procq.Symbol.nterm decl_notations)))
                            (fun notations body_def rtype binders id_decl
                            loc -> 
# 514 "vernac/g_vernac.mlg"
          {fname = fst id_decl; univs = snd id_decl; binders; rtype; body_def; notations}
        
                                   )])]))
  in let () = assert (Procq.Entry.is_empty rw_pattern) in
  let () =
  Procq.grammar_extend rw_pattern
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm lconstr)))
                            (fun p loc -> 
# 519 "vernac/g_vernac.mlg"
                         p 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty rewrite_rule) in
  let () =
  Procq.grammar_extend rewrite_rule
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.opt (Procq.Symbol.rules 
                                [Procq.Rules.make (Procq.Rule.next_norec
                                                   (Procq.Rule.next_norec
                                                    (Procq.Rule.stop)
                                                    ((Procq.Symbol.nterm univ_decl)))
                                                   ((Procq.Symbol.token (Tok.PKEYWORD ("|-")))))
                                                  (fun _ u loc -> 
# 521 "vernac/g_vernac.mlg"
                                           u 
                                                                  )]))))
                               ((Procq.Symbol.nterm rw_pattern)))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("=>")))))
                             ((Procq.Symbol.nterm lconstr)))
                            (fun rhs _ lhs u loc -> 
# 523 "vernac/g_vernac.mlg"
          (u, lhs, rhs)
        
                                                    )])]))
  in let () = assert (Procq.Entry.is_empty scheme) in
  let () =
  Procq.grammar_extend scheme
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.nterm identref)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                             ((Procq.Symbol.nterm scheme_kind)))
                            (fun kind _ id loc -> 
# 529 "vernac/g_vernac.mlg"
                                                     (Some id,kind) 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm scheme_kind)))
                           (fun kind loc -> 
# 528 "vernac/g_vernac.mlg"
                                (None,kind) 
                                            )])]))
  in let () = assert (Procq.Entry.is_empty scheme_kind) in
  let () =
  Procq.grammar_extend scheme_kind
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.nterm scheme_type)))
                                ((Procq.Symbol.token (Tok.PKEYWORD ("for")))))
                               ((Procq.Symbol.nterm smart_global)))
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Sort"))))))
                             ((Procq.Symbol.nterm sort_family)))
                            (fun sch_sort _ sch_qualid _ sch_type loc -> 
# 533 "vernac/g_vernac.mlg"
           {sch_type; sch_qualid; sch_sort} 
                                                                    )])]))
  in let () = assert (Procq.Entry.is_empty scheme_type) in
  let () =
  Procq.grammar_extend scheme_type
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Case"))))))
                            (fun _ loc -> 
# 540 "vernac/g_vernac.mlg"
                          SchemeCase 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Elimination"))))))
                           (fun _ loc -> 
# 539 "vernac/g_vernac.mlg"
                                 SchemeElimination 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Minimality"))))))
                           (fun _ loc -> 
# 538 "vernac/g_vernac.mlg"
                                SchemeMinimality 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Induction"))))))
                           (fun _ loc -> 
# 537 "vernac/g_vernac.mlg"
                               SchemeInduction 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty record_field) in
  let () =
  Procq.grammar_extend record_field
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.nterm quoted_attributes)))
                               ((Procq.Symbol.nterm record_binder)))
                              ((Procq.Symbol.opt (Procq.Symbol.rules 
                              [Procq.Rules.make (Procq.Rule.next_norec
                                                 (Procq.Rule.next_norec
                                                  (Procq.Rule.stop)
                                                  ((Procq.Symbol.token (Tok.PKEYWORD ("|")))))
                                                 ((Procq.Symbol.nterm natural)))
                                                (fun n _ loc -> 
# 558 "vernac/g_vernac.mlg"
                                                                     n 
                                                                )]))))
                             ((Procq.Symbol.nterm decl_notations)))
                            (fun rfu_notation rfu_priority bd rfu_attrs
                            loc -> 
# 559 "vernac/g_vernac.mlg"
                                        
      let (rfu_coercion, rfu_instance), rf_decl = bd in
      rf_decl,
      { rfu_attrs ; rfu_coercion ; rfu_instance ; rfu_priority ;
        rfu_notation } 
                                   )])]))
  in let () = assert (Procq.Entry.is_empty record_fields) in
  let () =
  Procq.grammar_extend record_fields
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 568 "vernac/g_vernac.mlg"
             [] 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm record_field)))
                           (fun f loc -> 
# 567 "vernac/g_vernac.mlg"
                              [f] 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm record_field)))
                             ((Procq.Symbol.token (Tok.PKEYWORD (";")))))
                            ((Procq.Symbol.nterm record_fields)))
                           (fun fs _ f loc -> 
# 566 "vernac/g_vernac.mlg"
                                                       f :: fs 
                                              )])]))
  in let () = assert (Procq.Entry.is_empty field_body) in
  let () =
  Procq.grammar_extend field_body
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.nterm binders)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                             ((Procq.Symbol.nterm lconstr)))
                            (fun b _ l loc -> 
# 577 "vernac/g_vernac.mlg"
                                            fun id ->
        (* Why are we dropping cast info here? *)
         match b.CAst.v with
         | CCast(b', _, t) ->
             ((NoCoercion,NoInstance),DefExpr(id,l,b',Some t))
         | _ ->
             ((NoCoercion,NoInstance),DefExpr(id,l,b,None)) 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.nterm binders)))
                               ((Procq.Symbol.nterm of_type_inst)))
                              ((Procq.Symbol.nterm lconstr)))
                             ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                            ((Procq.Symbol.nterm lconstr)))
                           (fun b _ t oc l loc -> 
# 575 "vernac/g_vernac.mlg"
                                             fun id ->
           (oc,DefExpr (id,l,b,Some t)) 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm binders)))
                             ((Procq.Symbol.nterm of_type_inst)))
                            ((Procq.Symbol.nterm lconstr)))
                           (fun t oc l loc -> 
# 573 "vernac/g_vernac.mlg"
                          fun id -> (oc,AssumExpr (id,l,t)) 
                                              )])]))
  in let () = assert (Procq.Entry.is_empty record_binder) in
  let () =
  Procq.grammar_extend record_binder
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm name)))
                             ((Procq.Symbol.nterm field_body)))
                            (fun f id loc -> 
# 587 "vernac/g_vernac.mlg"
                                       f id 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm name)))
                           (fun id loc -> 
# 586 "vernac/g_vernac.mlg"
                       ((NoCoercion,NoInstance),AssumExpr(id, [], CAst.make ~loc @@ CHole (None))) 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty assum_list) in
  let () =
  Procq.grammar_extend assum_list
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm assumpt)))
                            (fun b loc -> 
# 590 "vernac/g_vernac.mlg"
                                                          [b] 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm assum_coe)))))
                           (fun bl loc -> 
# 590 "vernac/g_vernac.mlg"
                                  bl 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty assum_coe) in
  let () =
  Procq.grammar_extend assum_coe
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                              ((Procq.Symbol.nterm assumpt)))
                             ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                            (fun _ a _ loc -> 
# 593 "vernac/g_vernac.mlg"
                                   a 
                                              )])]))
  in let () = assert (Procq.Entry.is_empty assumpt) in
  let () =
  Procq.grammar_extend assumpt
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.list1 ((Procq.Symbol.nterm ident_decl)))))
                              ((Procq.Symbol.nterm of_type)))
                             ((Procq.Symbol.nterm lconstr)))
                            (fun c oc idl loc -> 
# 597 "vernac/g_vernac.mlg"
          (oc,(idl,c)) 
                                                 )])]))
  in let () = assert (Procq.Entry.is_empty constructor_type) in
  let () =
  Procq.grammar_extend constructor_type
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm binders)))
                             ((Procq.Symbol.rules [Procq.Rules.make (Procq.Rule.stop)
                                                                    (fun
                                                                    loc -> 
                                                                    
# 605 "vernac/g_vernac.mlg"
                   fun l attr id -> ((attr,NoCoercion,NoInstance),(id,mkProdCN ~loc l (CAst.make ~loc @@ CHole (None)))) 
                                                                    );
                                                  Procq.Rules.make (Procq.Rule.next_norec
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.nterm of_type_inst)))
                                                                    ((Procq.Symbol.nterm lconstr)))
                                                                   (fun c coe
                                                                   loc -> 
                                                                   
# 603 "vernac/g_vernac.mlg"
                      fun l attr id -> ((attr, fst coe, snd coe),(id,mkProdCN ~loc l c)) 
                                                                   )])))
                            (fun t l loc -> 
# 606 "vernac/g_vernac.mlg"
              t l 
                                            )])]))
  in let () = assert (Procq.Entry.is_empty constructor) in
  let () =
  Procq.grammar_extend constructor
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.nterm quoted_attributes)))
                              ((Procq.Symbol.nterm identref)))
                             ((Procq.Symbol.nterm constructor_type)))
                            (fun c id attr loc -> 
# 612 "vernac/g_vernac.mlg"
                                               c attr id 
                                                  )])]))
  in let () = assert (Procq.Entry.is_empty of_type) in
  let () =
  Procq.grammar_extend of_type
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                            (fun _ loc -> 
# 617 "vernac/g_vernac.mlg"
                   NoCoercion 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                            ((Procq.Symbol.token (Tok.PKEYWORD (">")))))
                           (fun _ _ loc -> 
# 616 "vernac/g_vernac.mlg"
                        AddCoercion 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD (":>")))))
                           (fun _ loc -> 
# 615 "vernac/g_vernac.mlg"
                    AddCoercion 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty of_type_inst) in
  let () =
  Procq.grammar_extend of_type_inst
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                            (fun _ loc -> 
# 624 "vernac/g_vernac.mlg"
                   (NoCoercion, NoInstance) 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("::>")))))
                           (fun _ loc -> 
# 623 "vernac/g_vernac.mlg"
                     (AddCoercion, BackInstance) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("::")))))
                           (fun _ loc -> 
# 622 "vernac/g_vernac.mlg"
                    (NoCoercion, BackInstance) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                            ((Procq.Symbol.token (Tok.PKEYWORD (">")))))
                           (fun _ _ loc -> 
# 621 "vernac/g_vernac.mlg"
                        (AddCoercion, NoInstance) 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD (":>")))))
                           (fun _ loc -> 
# 620 "vernac/g_vernac.mlg"
                    (AddCoercion, NoInstance) 
                                         )])]))
  in ()


# 628 "vernac/g_vernac.mlg"
 

let test_only_starredidentrefs =
  let open Procq.Lookahead in
  to_entry "test_only_starredidentrefs" begin
    lk_list (lk_ident <+> lk_kws ["Type"; "*"]) >> (lk_kws [".";")"])
  end

let starredidentreflist_to_expr l =
  match l with
  | [] -> SsEmpty
  | x :: xs -> List.fold_right (fun i acc -> SsUnion(i,acc)) xs x

let warn_deprecated_include_type =
  CWarnings.create ~name:"deprecated-include-type" ~category:Deprecation.Version.v8_3
         (fun () -> strbrk "Include Type is deprecated; use Include instead")

let warn_deprecated_as_ident_kind =
  CWarnings.create ~name:"deprecated-as-ident-kind" ~category:Deprecation.Version.v8_14
         (fun () -> strbrk "grammar kind \"as ident\" no longer accepts \"_\"; use \"as name\" instead to accept \"_\", too, or silence the warning if you actually intended to accept only identifiers.")



let _ =
  let import_categories = Procq.Entry.make "import_categories"
  and filtered_import = Procq.Entry.make "filtered_import"
  and one_import_filter_name = Procq.Entry.make "one_import_filter_name"
  and export_token = Procq.Entry.make "export_token"
  and ext_module_type = Procq.Entry.make "ext_module_type"
  and ext_module_expr = Procq.Entry.make "ext_module_expr"
  and check_module_type = Procq.Entry.make "check_module_type"
  and check_module_types = Procq.Entry.make "check_module_types"
  and of_module_type = Procq.Entry.make "of_module_type"
  and is_module_type = Procq.Entry.make "is_module_type"
  and is_module_expr = Procq.Entry.make "is_module_expr"
  and functor_app_annot = Procq.Entry.make "functor_app_annot"
  and module_expr_inl = Procq.Entry.make "module_expr_inl"
  and module_type_inl = Procq.Entry.make "module_type_inl"
  and module_binder = Procq.Entry.make "module_binder"
  and module_expr_atom = Procq.Entry.make "module_expr_atom"
  and with_declaration = Procq.Entry.make "with_declaration"
  and starredidentref = Procq.Entry.make "starredidentref"
  and ssexpr = Procq.Entry.make "ssexpr"
  in
  let () = assert (Procq.Entry.is_empty gallina_ext) in
  let () =
  Procq.grammar_extend gallina_ext
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                ("Include"))))))
                               ((Procq.Symbol.token (Tok.PKEYWORD ("Type")))))
                              ((Procq.Symbol.nterm module_type_inl)))
                             ((Procq.Symbol.list0 (Procq.Symbol.nterm ext_module_type))))
                            (fun l e _ _ loc -> 
# 697 "vernac/g_vernac.mlg"
           warn_deprecated_include_type ~loc ();
        VernacSynterp (VernacInclude(e::l)) 
                                                );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Include"))))))
                             ((Procq.Symbol.nterm module_type_inl)))
                            ((Procq.Symbol.list0 (Procq.Symbol.nterm ext_module_type))))
                           (fun l e _ loc -> 
# 695 "vernac/g_vernac.mlg"
            VernacSynterp (VernacInclude(e::l)) 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Export"))))))
                             ((Procq.Symbol.opt (Procq.Symbol.nterm import_categories))))
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm filtered_import)))))
                           (fun qidl cats _ loc -> 
# 693 "vernac/g_vernac.mlg"
          VernacSynterp (VernacImport ((Export,cats),qidl)) 
                                                   );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Import"))))))
                             ((Procq.Symbol.opt (Procq.Symbol.nterm import_categories))))
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm filtered_import)))))
                           (fun qidl cats _ loc -> 
# 691 "vernac/g_vernac.mlg"
          VernacSynterp (VernacImport ((Import,cats),qidl)) 
                                                   );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                ("From"))))))
                               ((Procq.Symbol.nterm global)))
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Require"))))))
                             ((Procq.Symbol.nterm export_token)))
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm filtered_import)))))
                           (fun qidl export _ ns _ loc -> 
# 689 "vernac/g_vernac.mlg"
          VernacSynterp (VernacRequire (Some ns, export, qidl)) 
                                                          );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Require"))))))
                             ((Procq.Symbol.nterm export_token)))
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm filtered_import)))))
                           (fun qidl export _ loc -> 
# 686 "vernac/g_vernac.mlg"
            VernacSynterp (VernacRequire (None, export, qidl)) 
                                                     );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.token (Tok.PIDENT (Some
                                 ("From"))))))
                                ((Procq.Symbol.nterm global)))
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Extra"))))))
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Dependency"))))))
                             ((Procq.Symbol.nterm ne_string)))
                            ((Procq.Symbol.opt (Procq.Symbol.rules [Procq.Rules.make 
                                                                   (Procq.Rule.next_norec
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("as")))))
                                                                    ((Procq.Symbol.token (Tok.PIDENT (None)))))
                                                                   (fun id _
                                                                   loc -> 
                                                                   
# 681 "vernac/g_vernac.mlg"
                                         id 
                                                                   )]))))
                           (fun id f _ _ ns _ loc -> 
# 682 "vernac/g_vernac.mlg"
            VernacSynterp (VernacExtraDependency (ns, f, Option.map Id.of_string id)) 
                                                     );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Collection"))))))
                              ((Procq.Symbol.nterm identref)))
                             ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                            ((Procq.Symbol.nterm section_subset_expr)))
                           (fun expr _ id _ loc -> 
# 676 "vernac/g_vernac.mlg"
            VernacSynPure (VernacNameSectionHypSet (id, expr)) 
                                                   );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("End"))))))
                            ((Procq.Symbol.nterm identref)))
                           (fun id _ loc -> 
# 672 "vernac/g_vernac.mlg"
                                        VernacSynterp (VernacEndSegment id) 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Section"))))))
                            ((Procq.Symbol.nterm identref)))
                           (fun id _ loc -> 
# 669 "vernac/g_vernac.mlg"
                                            VernacSynterp (VernacBeginSection id) 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next (Procq.Rule.stop)
                                  ((Procq.Symbol.token (Tok.PIDENT (Some
                                  ("Declare"))))))
                                 ((Procq.Symbol.token (Tok.PIDENT (Some
                                 ("Module"))))))
                                ((Procq.Symbol.nterm export_token)))
                               ((Procq.Symbol.nterm identref)))
                              ((Procq.Symbol.list0 (Procq.Symbol.nterm module_binder))))
                             ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                            ((Procq.Symbol.nterm module_type_inl)))
                           (fun mty _ bl id export _ _ loc -> 
# 667 "vernac/g_vernac.mlg"
            VernacSynterp (VernacDeclareModule (export, id, bl, mty)) 
                                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.token (Tok.PIDENT (Some
                                 ("Module"))))))
                                ((Procq.Symbol.token (Tok.PKEYWORD ("Type")))))
                               ((Procq.Symbol.nterm identref)))
                              ((Procq.Symbol.list0 (Procq.Symbol.nterm module_binder))))
                             ((Procq.Symbol.nterm check_module_types)))
                            ((Procq.Symbol.nterm is_module_type)))
                           (fun body sign bl id _ _ loc -> 
# 664 "vernac/g_vernac.mlg"
            VernacSynterp (VernacDeclareModuleType (id, bl, sign, body)) 
                                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.token (Tok.PIDENT (Some
                                 ("Module"))))))
                                ((Procq.Symbol.nterm export_token)))
                               ((Procq.Symbol.nterm identref)))
                              ((Procq.Symbol.list0 (Procq.Symbol.nterm module_binder))))
                             ((Procq.Symbol.nterm of_module_type)))
                            ((Procq.Symbol.nterm is_module_expr)))
                           (fun body sign bl id export _ loc -> 
# 660 "vernac/g_vernac.mlg"
            VernacSynterp (VernacDefineModule (export, id, bl, sign, body)) 
                                                                )])]))
  in let () = assert (Procq.Entry.is_empty import_categories) in
  let () =
  Procq.grammar_extend import_categories
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.opt (Procq.Symbol.token (Tok.PKEYWORD ("-"))))))
                               ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                              ((Procq.Symbol.list1sep ((Procq.Symbol.nterm qualid)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                             ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                            (fun _ cats _ negative loc -> 
# 702 "vernac/g_vernac.mlg"
        let cats = List.map (fun cat -> CAst.make ?loc:cat.CAst.loc (Libnames.string_of_qualid cat)) cats in
        { negative=Option.has_some negative; import_cats = cats } 
                                                          )])]))
  in let () = assert (Procq.Entry.is_empty filtered_import) in
  let () =
  Procq.grammar_extend filtered_import
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.nterm global)))
                               ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                              ((Procq.Symbol.list1sep ((Procq.Symbol.nterm one_import_filter_name)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                             ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                            (fun _ ns _ m loc -> 
# 708 "vernac/g_vernac.mlg"
                                                                             (m, ImportNames ns) 
                                                 );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm global)))
                           (fun m loc -> 
# 707 "vernac/g_vernac.mlg"
                        (m, ImportAll) 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty one_import_filter_name) in
  let () =
  Procq.grammar_extend one_import_filter_name
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm global)))
                             ((Procq.Symbol.opt (Procq.Symbol.rules [Procq.Rules.make 
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("..")))))
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                                                                    (fun _ _
                                                                    _ loc ->
                                                                    
# 711 "vernac/g_vernac.mlg"
                                                    () 
                                                                    )]))))
                            (fun etc n loc -> 
# 711 "vernac/g_vernac.mlg"
                                                                n, Option.has_some etc 
                                              )])]))
  in let () = assert (Procq.Entry.is_empty export_token) in
  let () =
  Procq.grammar_extend export_token
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 716 "vernac/g_vernac.mlg"
              None 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Export"))))))
                            ((Procq.Symbol.opt (Procq.Symbol.nterm import_categories))))
                           (fun cats _ loc -> 
# 715 "vernac/g_vernac.mlg"
                                                          Some (Export,cats) 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Import"))))))
                            ((Procq.Symbol.opt (Procq.Symbol.nterm import_categories))))
                           (fun cats _ loc -> 
# 714 "vernac/g_vernac.mlg"
                                                          Some (Import,cats) 
                                              )])]))
  in let () = assert (Procq.Entry.is_empty ext_module_type) in
  let () =
  Procq.grammar_extend ext_module_type
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("<+")))))
                             ((Procq.Symbol.nterm module_type_inl)))
                            (fun mty _ loc -> 
# 719 "vernac/g_vernac.mlg"
                                         mty 
                                              )])]))
  in let () = assert (Procq.Entry.is_empty ext_module_expr) in
  let () =
  Procq.grammar_extend ext_module_expr
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("<+")))))
                             ((Procq.Symbol.nterm module_expr_inl)))
                            (fun mexpr _ loc -> 
# 722 "vernac/g_vernac.mlg"
                                           mexpr 
                                                )])]))
  in let () = assert (Procq.Entry.is_empty check_module_type) in
  let () =
  Procq.grammar_extend check_module_type
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("<:")))))
                             ((Procq.Symbol.nterm module_type_inl)))
                            (fun mty _ loc -> 
# 725 "vernac/g_vernac.mlg"
                                         mty 
                                              )])]))
  in let () = assert (Procq.Entry.is_empty check_module_types) in
  let () =
  Procq.grammar_extend check_module_types
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.list0 (Procq.Symbol.nterm check_module_type))))
                            (fun mtys loc -> 
# 728 "vernac/g_vernac.mlg"
                                            mtys 
                                             )])]))
  in let () = assert (Procq.Entry.is_empty of_module_type) in
  let () =
  Procq.grammar_extend of_module_type
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm check_module_types)))
                            (fun mtys loc -> 
# 732 "vernac/g_vernac.mlg"
                                       Check mtys 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                            ((Procq.Symbol.nterm module_type_inl)))
                           (fun mty _ loc -> 
# 731 "vernac/g_vernac.mlg"
                                        Enforce mty 
                                             )])]))
  in let () = assert (Procq.Entry.is_empty is_module_type) in
  let () =
  Procq.grammar_extend is_module_type
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 736 "vernac/g_vernac.mlg"
             [] 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                             ((Procq.Symbol.nterm module_type_inl)))
                            ((Procq.Symbol.list0 (Procq.Symbol.nterm ext_module_type))))
                           (fun l mty _ loc -> 
# 735 "vernac/g_vernac.mlg"
                                                                     (mty::l) 
                                               )])]))
  in let () = assert (Procq.Entry.is_empty is_module_expr) in
  let () =
  Procq.grammar_extend is_module_expr
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 740 "vernac/g_vernac.mlg"
             [] 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                             ((Procq.Symbol.nterm module_expr_inl)))
                            ((Procq.Symbol.list0 (Procq.Symbol.nterm ext_module_expr))))
                           (fun l mexpr _ loc -> 
# 739 "vernac/g_vernac.mlg"
                                                                      (mexpr::l) 
                                                 )])]))
  in let () = assert (Procq.Entry.is_empty functor_app_annot) in
  let () =
  Procq.grammar_extend functor_app_annot
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 746 "vernac/g_vernac.mlg"
             DefaultInline 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("[")))))
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("no"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("inline"))))))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("]")))))
                           (fun _ _ _ _ loc -> 
# 745 "vernac/g_vernac.mlg"
                                                  NoInline 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.token (Tok.PKEYWORD ("[")))))
                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                ("inline"))))))
                               ((Procq.Symbol.token (Tok.PKEYWORD ("at")))))
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("level"))))))
                             ((Procq.Symbol.nterm natural)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("]")))))
                           (fun _ i _ _ _ _ loc -> 
# 744 "vernac/g_vernac.mlg"
          InlineAt i 
                                                   )])]))
  in let () = assert (Procq.Entry.is_empty module_expr_inl) in
  let () =
  Procq.grammar_extend module_expr_inl
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm module_expr)))
                             ((Procq.Symbol.nterm functor_app_annot)))
                            (fun a me loc -> 
# 751 "vernac/g_vernac.mlg"
                                                     (me,a) 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("!")))))
                            ((Procq.Symbol.nterm module_expr)))
                           (fun me _ loc -> 
# 750 "vernac/g_vernac.mlg"
                                   (me,NoInline) 
                                            )])]))
  in let () = assert (Procq.Entry.is_empty module_type_inl) in
  let () =
  Procq.grammar_extend module_type_inl
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm module_type)))
                             ((Procq.Symbol.nterm functor_app_annot)))
                            (fun a me loc -> 
# 755 "vernac/g_vernac.mlg"
                                                     (me,a) 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("!")))))
                            ((Procq.Symbol.nterm module_type)))
                           (fun me _ loc -> 
# 754 "vernac/g_vernac.mlg"
                                   (me,NoInline) 
                                            )])]))
  in let () = assert (Procq.Entry.is_empty module_binder) in
  let () =
  Procq.grammar_extend module_binder
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next (Procq.Rule.stop)
                                  ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                                 ((Procq.Symbol.nterm export_token)))
                                ((Procq.Symbol.list1 ((Procq.Symbol.nterm identref)))))
                               ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                              ((Procq.Symbol.nterm module_type_inl)))
                             ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                            (fun _ mty _ idl export _ loc -> 
# 760 "vernac/g_vernac.mlg"
                                         (export,idl,mty) 
                                                             )])]))
  in let () = assert (Procq.Entry.is_empty module_expr) in
  let () =
  Procq.grammar_extend module_expr
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm module_expr)))
                             ((Procq.Symbol.nterm module_expr_atom)))
                            (fun me2 me1 loc -> 
# 765 "vernac/g_vernac.mlg"
                                                       CAst.make ~loc @@ CMapply (me1,me2) 
                                                );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm module_expr_atom)))
                           (fun me loc -> 
# 764 "vernac/g_vernac.mlg"
                                   CAst.make ~loc @@ CMident me 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty module_expr_atom) in
  let () =
  Procq.grammar_extend module_expr_atom
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                              ((Procq.Symbol.nterm module_expr_atom)))
                             ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                            (fun _ me _ loc -> 
# 769 "vernac/g_vernac.mlg"
                                                                       me 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm qualid)))
                           (fun qid loc -> 
# 769 "vernac/g_vernac.mlg"
                          qid 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty with_declaration) in
  let () =
  Procq.grammar_extend with_declaration
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                ("Module"))))))
                               ((Procq.Symbol.nterm fullyqualid)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                             ((Procq.Symbol.nterm qualid)))
                            (fun qid _ fqid _ loc -> 
# 775 "vernac/g_vernac.mlg"
            CWith_Module (fqid,qid) 
                                                     );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PKEYWORD ("Definition")))))
                               ((Procq.Symbol.nterm fullyqualid)))
                              ((Procq.Symbol.opt (Procq.Symbol.nterm univ_decl))))
                             ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                            ((Procq.Symbol.nterm Constr.lconstr)))
                           (fun c _ udecl fqid _ loc -> 
# 773 "vernac/g_vernac.mlg"
            CWith_Definition (fqid,udecl,c) 
                                                        )])]))
  in let () = assert (Procq.Entry.is_empty module_type) in
  let () =
  Procq.grammar_extend module_type
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.nterm module_type)))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("with")))))
                             ((Procq.Symbol.nterm with_declaration)))
                            (fun decl _ mty loc -> 
# 784 "vernac/g_vernac.mlg"
            CAst.make ~loc @@ CMwith (mty,decl) 
                                                   );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm module_type)))
                            ((Procq.Symbol.nterm module_expr_atom)))
                           (fun me mty loc -> 
# 782 "vernac/g_vernac.mlg"
            CAst.make ~loc @@ CMapply (mty,me) 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                             ((Procq.Symbol.nterm module_type)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                           (fun _ mt _ loc -> 
# 780 "vernac/g_vernac.mlg"
                                        mt 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm qualid)))
                           (fun qid loc -> 
# 779 "vernac/g_vernac.mlg"
                          CAst.make ~loc @@ CMident qid 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty section_subset_expr) in
  let () =
  Procq.grammar_extend section_subset_expr
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm ssexpr)))
                            (fun e loc -> 
# 791 "vernac/g_vernac.mlg"
                        e 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm test_only_starredidentrefs)))
                            ((Procq.Symbol.list0 (Procq.Symbol.nterm starredidentref))))
                           (fun l _ loc -> 
# 790 "vernac/g_vernac.mlg"
            starredidentreflist_to_expr l 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty starredidentref) in
  let () =
  Procq.grammar_extend starredidentref
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("Type")))))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("*")))))
                            (fun _ _ loc -> 
# 797 "vernac/g_vernac.mlg"
                         SsFwdClose SsType 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("Type")))))
                           (fun _ loc -> 
# 796 "vernac/g_vernac.mlg"
                    SsType 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm identref)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("*")))))
                           (fun _ i loc -> 
# 795 "vernac/g_vernac.mlg"
                               SsFwdClose(SsSingl i) 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm identref)))
                           (fun i loc -> 
# 794 "vernac/g_vernac.mlg"
                          SsSingl i 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty ssexpr) in
  let () =
  Procq.grammar_extend ssexpr
  (Procq.Fresh
  (Gramlib.Gramext.First, [(Some ("35"), None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("-")))))
                             ((Procq.Symbol.nterm ssexpr)))
                            (fun e _ loc -> 
# 801 "vernac/g_vernac.mlg"
                             SsCompl e 
                                            )]);
                          (Some ("50"), None,
                          [Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm ssexpr)))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("+")))))
                            ((Procq.Symbol.nterm ssexpr)))
                           (fun e2 _ e1 loc -> 
# 804 "vernac/g_vernac.mlg"
                                          SsUnion(e1,e2) 
                                               );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm ssexpr)))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("-")))))
                           ((Procq.Symbol.nterm ssexpr)))
                          (fun e2 _ e1 loc -> 
# 803 "vernac/g_vernac.mlg"
                                          SsSubstr(e1,e2) 
                                              )]);
                          (Some ("0"), None,
                          [Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                              ((Procq.Symbol.nterm ssexpr)))
                             ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("*")))))
                           (fun _ _ e _ loc -> 
# 813 "vernac/g_vernac.mlg"
                                       SsFwdClose e 
                                               );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                            ((Procq.Symbol.nterm ssexpr)))
                           ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                          (fun _ e _ loc -> 
# 812 "vernac/g_vernac.mlg"
                                 e 
                                            );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                              ((Procq.Symbol.nterm test_only_starredidentrefs)))
                             ((Procq.Symbol.list0 (Procq.Symbol.nterm starredidentref))))
                            ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                           ((Procq.Symbol.token (Tok.PKEYWORD ("*")))))
                          (fun _ _ l _ _ loc -> 
# 811 "vernac/g_vernac.mlg"
            SsFwdClose(starredidentreflist_to_expr l) 
                                                );
                          Procq.Production.make
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                             ((Procq.Symbol.nterm test_only_starredidentrefs)))
                            ((Procq.Symbol.list0 (Procq.Symbol.nterm starredidentref))))
                           ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                          (fun _ l _ _ loc -> 
# 809 "vernac/g_vernac.mlg"
            starredidentreflist_to_expr l 
                                              );
                          Procq.Production.make
                          (Procq.Rule.next (Procq.Rule.stop)
                           ((Procq.Symbol.token (Tok.PKEYWORD ("()")))))
                          (fun _ loc -> 
# 807 "vernac/g_vernac.mlg"
                  SsEmpty 
                                        );
                          Procq.Production.make
                          (Procq.Rule.next (Procq.Rule.stop)
                           ((Procq.Symbol.nterm starredidentref)))
                          (fun i loc -> 
# 806 "vernac/g_vernac.mlg"
                                 i 
                                        )])]))
  in ()

let _ =
  let args_modifier = Procq.Entry.make "args_modifier"
  and argument_spec = Procq.Entry.make "argument_spec"
  and arg_specs = Procq.Entry.make "arg_specs"
  and implicits_alt = Procq.Entry.make "implicits_alt"
  and instance_name = Procq.Entry.make "instance_name"
  and reserv_list = Procq.Entry.make "reserv_list"
  and reserv_tuple = Procq.Entry.make "reserv_tuple"
  and simple_reserv = Procq.Entry.make "simple_reserv"
  in
  let () =
  Procq.grammar_extend gallina_ext
  (Procq.Reuse (None, [Procq.Production.make
                       (Procq.Rule.next
                        (Procq.Rule.next (Procq.Rule.stop)
                         ((Procq.Symbol.token (Tok.PIDENT (Some
                         ("Generalizable"))))))
                        ((Procq.Symbol.rules [Procq.Rules.make (Procq.Rule.next_norec
                                                                (Procq.Rule.next_norec
                                                                 (Procq.Rule.stop)
                                                                 ((Procq.Symbol.rules 
                                                                 [Procq.Rules.make 
                                                                 (Procq.Rule.next_norec
                                                                  (Procq.Rule.stop)
                                                                  ((Procq.Symbol.token (Tok.PIDENT (Some
                                                                  ("Variables"))))))
                                                                 (fun _
                                                                 loc -> 
                                                                 
# 901 "vernac/g_vernac.mlg"
                                                              () 
                                                                 );
                                                                 Procq.Rules.make 
                                                                 (Procq.Rule.next_norec
                                                                  (Procq.Rule.stop)
                                                                  ((Procq.Symbol.token (Tok.PKEYWORD ("Variable")))))
                                                                 (fun _
                                                                 loc -> 
                                                                 
# 901 "vernac/g_vernac.mlg"
                                () 
                                                                 )])))
                                                                ((Procq.Symbol.list1 ((Procq.Symbol.nterm identref)))))
                                                               (fun idl _
                                                               loc -> 
                                                               
# 902 "vernac/g_vernac.mlg"
                                            Some idl 
                                                               );
                                             Procq.Rules.make (Procq.Rule.next_norec
                                                               (Procq.Rule.next_norec
                                                                (Procq.Rule.stop)
                                                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                                                ("No"))))))
                                                               ((Procq.Symbol.token (Tok.PIDENT (Some
                                                               ("Variables"))))))
                                                              (fun _ _ loc ->
                                                              
# 900 "vernac/g_vernac.mlg"
                                                  None 
                                                              );
                                             Procq.Rules.make (Procq.Rule.next_norec
                                                               (Procq.Rule.next_norec
                                                                (Procq.Rule.stop)
                                                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                                                ("All"))))))
                                                               ((Procq.Symbol.token (Tok.PIDENT (Some
                                                               ("Variables"))))))
                                                              (fun _ _ loc ->
                                                              
# 899 "vernac/g_vernac.mlg"
                                                      Some [] 
                                                              )])))
                       (fun gen _ loc -> 
# 903 "vernac/g_vernac.mlg"
               VernacSynPure (VernacGeneralizable gen) 
                                         );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next (Procq.Rule.stop)
                         ((Procq.Symbol.token (Tok.PIDENT (Some
                         ("Implicit"))))))
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Types"))))))
                       ((Procq.Symbol.nterm reserv_list)))
                      (fun bl _ _ loc -> 
# 895 "vernac/g_vernac.mlg"
            test_plural_form_types loc "Implicit Types" bl;
           VernacSynPure (VernacReserve bl) 
                                         );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next (Procq.Rule.stop)
                         ((Procq.Symbol.token (Tok.PIDENT (Some
                         ("Implicit"))))))
                        ((Procq.Symbol.token (Tok.PKEYWORD ("Type")))))
                       ((Procq.Symbol.nterm reserv_list)))
                      (fun bl _ _ loc -> 
# 892 "vernac/g_vernac.mlg"
             VernacSynPure (VernacReserve bl) 
                                         );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next
                         (Procq.Rule.next
                          (Procq.Rule.next (Procq.Rule.stop)
                           ((Procq.Symbol.token (Tok.PIDENT (Some
                           ("Arguments"))))))
                          ((Procq.Symbol.nterm smart_global)))
                         ((Procq.Symbol.list0 (Procq.Symbol.nterm arg_specs))))
                        ((Procq.Symbol.opt (Procq.Symbol.rules [Procq.Rules.make 
                                                               (Procq.Rule.next_norec
                                                                (Procq.Rule.next_norec
                                                                 (Procq.Rule.stop)
                                                                 ((Procq.Symbol.token (Tok.PKEYWORD (",")))))
                                                                ((Procq.Symbol.list1sep ((Procq.Symbol.rules 
                                                                [Procq.Rules.make 
                                                                (Procq.Rule.next_norec
                                                                 (Procq.Rule.stop)
                                                                 ((Procq.Symbol.list0 (Procq.Symbol.nterm implicits_alt))))
                                                                (fun impl
                                                                loc -> 
                                                                
# 883 "vernac/g_vernac.mlg"
                                              List.flatten impl 
                                                                )])) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                                                               (fun impl _
                                                               loc -> 
                                                               
# 884 "vernac/g_vernac.mlg"
                         impl 
                                                               )]))))
                       ((Procq.Symbol.opt (Procq.Symbol.rules [Procq.Rules.make 
                                                              (Procq.Rule.next_norec
                                                               (Procq.Rule.next_norec
                                                                (Procq.Rule.stop)
                                                                ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                                                               ((Procq.Symbol.list1sep ((Procq.Symbol.nterm args_modifier)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                                                              (fun l _ loc ->
                                                              
# 886 "vernac/g_vernac.mlg"
                                                               l 
                                                              )]))))
                      (fun mods more_implicits args qid _ loc -> 
# 887 "vernac/g_vernac.mlg"
           let mods = match mods with None -> [] | Some l -> List.flatten l in
         let more_implicits = Option.default [] more_implicits in
         VernacSynPure (VernacArguments (qid, List.flatten args, more_implicits, mods)) 
                                                                 );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next (Procq.Rule.stop)
                         ((Procq.Symbol.token (Tok.PIDENT (Some
                         ("Existing"))))))
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Class"))))))
                       ((Procq.Symbol.nterm global)))
                      (fun is _ _ loc -> 
# 876 "vernac/g_vernac.mlg"
                                                          VernacSynPure (VernacExistingClass is) 
                                         );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next
                         (Procq.Rule.next (Procq.Rule.stop)
                          ((Procq.Symbol.token (Tok.PIDENT (Some
                          ("Existing"))))))
                         ((Procq.Symbol.token (Tok.PIDENT (Some
                         ("Instances"))))))
                        ((Procq.Symbol.list1 ((Procq.Symbol.nterm global)))))
                       ((Procq.Symbol.opt (Procq.Symbol.rules [Procq.Rules.make 
                                                              (Procq.Rule.next_norec
                                                               (Procq.Rule.next_norec
                                                                (Procq.Rule.stop)
                                                                ((Procq.Symbol.token (Tok.PKEYWORD ("|")))))
                                                               ((Procq.Symbol.nterm natural)))
                                                              (fun i _ loc ->
                                                              
# 871 "vernac/g_vernac.mlg"
                                          i 
                                                              )]))))
                      (fun pri ids _ _ loc -> 
# 872 "vernac/g_vernac.mlg"
           let info = { Typeclasses.hint_priority = pri; hint_pattern = None } in
         let insts = List.map (fun i -> (i, info)) ids in
          VernacSynPure (VernacExistingInstance insts) 
                                              );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next
                         (Procq.Rule.next (Procq.Rule.stop)
                          ((Procq.Symbol.token (Tok.PIDENT (Some
                          ("Existing"))))))
                         ((Procq.Symbol.token (Tok.PIDENT (Some
                         ("Instance"))))))
                        ((Procq.Symbol.nterm global)))
                       ((Procq.Symbol.nterm hint_info)))
                      (fun info id _ _ loc -> 
# 868 "vernac/g_vernac.mlg"
            VernacSynPure (VernacExistingInstance [id, info]) 
                                              );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next
                         (Procq.Rule.next
                          (Procq.Rule.next
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Instance"))))))
                           ((Procq.Symbol.nterm instance_name)))
                          ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                         ((Procq.Symbol.nterml term ("200"))))
                        ((Procq.Symbol.nterm hint_info)))
                       ((Procq.Symbol.rules [Procq.Rules.make (Procq.Rule.stop)
                                                              (fun loc -> 
                                                              
# 863 "vernac/g_vernac.mlg"
                                                            None 
                                                              );
                                            Procq.Rules.make (Procq.Rule.next_norec
                                                              (Procq.Rule.next_norec
                                                               (Procq.Rule.stop)
                                                               ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                                                              ((Procq.Symbol.nterm lconstr)))
                                                             (fun c _ loc ->
                                                             
# 863 "vernac/g_vernac.mlg"
                                    Some (false,c) 
                                                             );
                                            Procq.Rules.make (Procq.Rule.next_norec
                                                              (Procq.Rule.next_norec
                                                               (Procq.Rule.next_norec
                                                                (Procq.Rule.next_norec
                                                                 (Procq.Rule.stop)
                                                                 ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                                                                ((Procq.Symbol.token (Tok.PKEYWORD ("{")))))
                                                               ((Procq.Symbol.nterm record_declaration)))
                                                              ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                                                             (fun _ r _ _
                                                             loc -> 
# 862 "vernac/g_vernac.mlg"
                                                               Some (true,r) 
                                                                    )])))
                      (fun props info t _ namesup _ loc -> 
# 864 "vernac/g_vernac.mlg"
             VernacSynPure (VernacInstance (fst namesup,snd namesup,t,props,info)) 
                                                           );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some
                        ("Context"))))))
                       ((Procq.Symbol.list1 ((Procq.Symbol.nterm binder)))))
                      (fun c _ loc -> 
# 857 "vernac/g_vernac.mlg"
            VernacSynPure (VernacContext (List.flatten c)) 
                                      );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next
                         (Procq.Rule.next
                          (Procq.Rule.next
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Coercion"))))))
                           ((Procq.Symbol.nterm by_notation)))
                          ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                         ((Procq.Symbol.nterm coercion_class)))
                        ((Procq.Symbol.token (Tok.PKEYWORD (">->")))))
                       ((Procq.Symbol.nterm coercion_class)))
                      (fun t _ s _ ntn _ loc -> 
# 854 "vernac/g_vernac.mlg"
            VernacSynPure (VernacCoercion (CAst.make ~loc @@ ByNotation ntn, Some (s, t))) 
                                                );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next
                         (Procq.Rule.next
                          (Procq.Rule.next
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Coercion"))))))
                           ((Procq.Symbol.nterm global)))
                          ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                         ((Procq.Symbol.nterm coercion_class)))
                        ((Procq.Symbol.token (Tok.PKEYWORD (">->")))))
                       ((Procq.Symbol.nterm coercion_class)))
                      (fun t _ s _ qid _ loc -> 
# 851 "vernac/g_vernac.mlg"
            VernacSynPure (VernacCoercion (CAst.make ~loc @@ AN qid, Some(s, t))) 
                                                );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next
                         (Procq.Rule.next
                          (Procq.Rule.next
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Identity"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Coercion"))))))
                           ((Procq.Symbol.nterm identref)))
                          ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                         ((Procq.Symbol.nterm coercion_class)))
                        ((Procq.Symbol.token (Tok.PKEYWORD (">->")))))
                       ((Procq.Symbol.nterm coercion_class)))
                      (fun t _ s _ f _ _ loc -> 
# 848 "vernac/g_vernac.mlg"
             VernacSynPure (VernacIdentityCoercion (f, s, t)) 
                                                );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next (Procq.Rule.stop)
                         ((Procq.Symbol.token (Tok.PIDENT (Some
                         ("Coercion"))))))
                        ((Procq.Symbol.nterm global)))
                       ((Procq.Symbol.opt (Procq.Symbol.rules [Procq.Rules.make 
                                                              (Procq.Rule.next_norec
                                                               (Procq.Rule.next_norec
                                                                (Procq.Rule.stop)
                                                                ((Procq.Symbol.opt (Procq.Symbol.nterm univ_decl))))
                                                               ((Procq.Symbol.nterm def_body)))
                                                              (fun d u loc ->
                                                              
# 842 "vernac/g_vernac.mlg"
                                                                                        u, d 
                                                              )]))))
                      (fun ud qid _ loc -> 
# 843 "vernac/g_vernac.mlg"
            match ud with Some (u, d) -> let s = coerce_reference_to_id qid in
          VernacSynPure (VernacDefinition ((NoDischarge,Coercion),((CAst.make ?loc:qid.CAst.loc (Name s)),u),d))
          | None -> VernacSynPure (VernacCoercion (CAst.make ~loc @@ AN qid, None)) 
                                           );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next (Procq.Rule.stop)
                         ((Procq.Symbol.token (Tok.PIDENT (Some
                         ("Canonical"))))))
                        ((Procq.Symbol.opt (Procq.Symbol.rules [Procq.Rules.make 
                                                               (Procq.Rule.next_norec
                                                                (Procq.Rule.stop)
                                                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                                                ("Structure"))))))
                                                               (fun _ loc ->
                                                               
# 838 "vernac/g_vernac.mlg"
                                                       ()
                                                               )]))))
                       ((Procq.Symbol.nterm by_notation)))
                      (fun ntn _ _ loc -> 
# 839 "vernac/g_vernac.mlg"
            VernacSynPure (VernacCanonical CAst.(make ~loc @@ ByNotation ntn)) 
                                          );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next
                         (Procq.Rule.next (Procq.Rule.stop)
                          ((Procq.Symbol.token (Tok.PIDENT (Some
                          ("Canonical"))))))
                         ((Procq.Symbol.opt (Procq.Symbol.rules [Procq.Rules.make 
                                                                (Procq.Rule.next_norec
                                                                 (Procq.Rule.stop)
                                                                 ((Procq.Symbol.token (Tok.PIDENT (Some
                                                                 ("Structure"))))))
                                                                (fun _ loc ->
                                                                
# 831 "vernac/g_vernac.mlg"
                                                       ()
                                                                )]))))
                        ((Procq.Symbol.nterm global)))
                       ((Procq.Symbol.opt (Procq.Symbol.rules [Procq.Rules.make 
                                                              (Procq.Rule.next_norec
                                                               (Procq.Rule.next_norec
                                                                (Procq.Rule.stop)
                                                                ((Procq.Symbol.opt (Procq.Symbol.nterm univ_decl))))
                                                               ((Procq.Symbol.nterm def_body)))
                                                              (fun d u loc ->
                                                              
# 831 "vernac/g_vernac.mlg"
                                                                                                                            (u,d) 
                                                              )]))))
                      (fun ud qid _ _ loc -> 
# 832 "vernac/g_vernac.mlg"
            match ud with
           | None ->
             VernacSynPure (VernacCanonical CAst.(make ?loc:qid.CAst.loc @@ AN qid))
           | Some (u,d) ->
             let s = coerce_reference_to_id qid in
             VernacSynPure (VernacDefinition ((NoDischarge,CanonicalStructure),((CAst.make ?loc:qid.CAst.loc (Name s)),u),d)) 
                                             );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some
                        ("Strategy"))))))
                       ((Procq.Symbol.list1 ((Procq.Symbol.rules [Procq.Rules.make 
                                                                 (Procq.Rule.next_norec
                                                                  (Procq.Rule.next_norec
                                                                   (Procq.Rule.next_norec
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.nterm strategy_level)))
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("[")))))
                                                                   ((Procq.Symbol.list1 ((Procq.Symbol.nterm smart_global)))))
                                                                  ((Procq.Symbol.token (Tok.PKEYWORD ("]")))))
                                                                 (fun _ q _ v
                                                                 loc -> 
                                                                 
# 828 "vernac/g_vernac.mlg"
                                                                        (v,q) 
                                                                 )])))))
                      (fun l _ loc -> 
# 829 "vernac/g_vernac.mlg"
              VernacSynPure (VernacSetStrategy l) 
                                      );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next (Procq.Rule.stop)
                         ((Procq.Symbol.token (Tok.PIDENT (Some
                         ("Opaque"))))))
                        ((Procq.Symbol.opt (Procq.Symbol.rules [Procq.Rules.make 
                                                               (Procq.Rule.next_norec
                                                                (Procq.Rule.stop)
                                                                ((Procq.Symbol.token (Tok.PKEYWORD ("!")))))
                                                               (fun _ loc ->
                                                               
# 825 "vernac/g_vernac.mlg"
                                          ()
                                                               )]))))
                       ((Procq.Symbol.list1 ((Procq.Symbol.nterm smart_global)))))
                      (fun l o _ loc -> 
# 826 "vernac/g_vernac.mlg"
            VernacSynPure (VernacSetOpacity ((Conv_oracle.Opaque, l), o <> None)) 
                                        );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next (Procq.Rule.stop)
                         ((Procq.Symbol.token (Tok.PIDENT (Some
                         ("Transparent"))))))
                        ((Procq.Symbol.opt (Procq.Symbol.rules [Procq.Rules.make 
                                                               (Procq.Rule.next_norec
                                                                (Procq.Rule.stop)
                                                                ((Procq.Symbol.token (Tok.PKEYWORD ("!")))))
                                                               (fun _ loc ->
                                                               
# 823 "vernac/g_vernac.mlg"
                                               ()
                                                               )]))))
                       ((Procq.Symbol.list1 ((Procq.Symbol.nterm smart_global)))))
                      (fun l o _ loc -> 
# 824 "vernac/g_vernac.mlg"
            VernacSynPure (VernacSetOpacity ((Conv_oracle.transparent, l), o <> None)) 
                                        )]))
  in let () = assert (Procq.Entry.is_empty args_modifier) in
  let () =
  Procq.grammar_extend args_modifier
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                ("clear"))))))
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("implicits"))))))
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("and"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("scopes"))))))
                            (fun _ _ _ _ loc -> 
# 919 "vernac/g_vernac.mlg"
            [`ClearImplicits; `ClearScopes] 
                                                );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("clear"))))))
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("scopes"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("and"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("implicits"))))))
                           (fun _ _ _ _ loc -> 
# 917 "vernac/g_vernac.mlg"
            [`ClearImplicits; `ClearScopes] 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("extra"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("scopes"))))))
                           (fun _ _ loc -> 
# 915 "vernac/g_vernac.mlg"
                                           [`ExtraScopes] 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("assert"))))))
                           (fun _ loc -> 
# 914 "vernac/g_vernac.mlg"
                            [`Assert] 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("rename"))))))
                           (fun _ loc -> 
# 913 "vernac/g_vernac.mlg"
                            [`Rename] 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("clear"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("bidirectionality"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("hint"))))))
                           (fun _ _ _ loc -> 
# 912 "vernac/g_vernac.mlg"
                                                                   [`ClearBidiHint] 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("clear"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("scopes"))))))
                           (fun _ _ loc -> 
# 911 "vernac/g_vernac.mlg"
                                           [`ClearScopes] 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("clear"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("implicits"))))))
                           (fun _ _ loc -> 
# 910 "vernac/g_vernac.mlg"
                                              [`ClearImplicits] 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("default"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("implicits"))))))
                           (fun _ _ loc -> 
# 909 "vernac/g_vernac.mlg"
                                                [`DefaultImplicits] 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("clear"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("simpl"))))))
                           (fun _ _ loc -> 
# 908 "vernac/g_vernac.mlg"
                                          [`ClearReduction] 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("simpl"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("never"))))))
                           (fun _ _ loc -> 
# 907 "vernac/g_vernac.mlg"
                                          [`SimplNeverUnfold] 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("simpl"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("nomatch"))))))
                           (fun _ _ loc -> 
# 906 "vernac/g_vernac.mlg"
                                            [`SimplDontExposeCase] 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty scope_delimiter) in
  let () =
  Procq.grammar_extend scope_delimiter
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("%_")))))
                             ((Procq.Symbol.token (Tok.PIDENT (None)))))
                            (fun key _ loc -> 
# 924 "vernac/g_vernac.mlg"
                               DelimOnlyTmpScope, key 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("%")))))
                            ((Procq.Symbol.token (Tok.PIDENT (None)))))
                           (fun key _ loc -> 
# 923 "vernac/g_vernac.mlg"
                              DelimUnboundedScope, key 
                                             )])]))
  in let () = assert (Procq.Entry.is_empty argument_spec) in
  let () =
  Procq.grammar_extend argument_spec
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.opt (Procq.Symbol.token (Tok.PKEYWORD ("!"))))))
                              ((Procq.Symbol.nterm name)))
                             ((Procq.Symbol.list0 (Procq.Symbol.nterm scope_delimiter))))
                            (fun s id b loc -> 
# 928 "vernac/g_vernac.mlg"
         id.CAst.v, not (Option.is_empty b), List.map (fun x -> CAst.make ~loc x) s 
                                               )])]))
  in let () = assert (Procq.Entry.is_empty arg_specs) in
  let () =
  Procq.grammar_extend arg_specs
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PKEYWORD ("{")))))
                               ((Procq.Symbol.list1 ((Procq.Symbol.nterm argument_spec)))))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                             ((Procq.Symbol.list0 (Procq.Symbol.nterm scope_delimiter))))
                            (fun scl _ items _ loc -> 
# 953 "vernac/g_vernac.mlg"
         let scl = List.map (CAst.make ~loc) scl in
       List.map (fun (name,recarg_like,notation_scope) ->
           RealArg { name=name; recarg_like=recarg_like;
                 notation_scope = notation_scope @ scl;
                 implicit_status = MaxImplicit}) items 
                                                      );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("[")))))
                              ((Procq.Symbol.list1 ((Procq.Symbol.nterm argument_spec)))))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("]")))))
                            ((Procq.Symbol.list0 (Procq.Symbol.nterm scope_delimiter))))
                           (fun scl _ items _ loc -> 
# 947 "vernac/g_vernac.mlg"
         let scl = List.map (CAst.make ~loc) scl in
       List.map (fun (name,recarg_like,notation_scope) ->
           RealArg { name=name; recarg_like=recarg_like;
                 notation_scope = notation_scope @ scl;
                 implicit_status = NonMaxImplicit}) items 
                                                     );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                              ((Procq.Symbol.list1 ((Procq.Symbol.nterm argument_spec)))))
                             ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                            ((Procq.Symbol.list0 (Procq.Symbol.nterm scope_delimiter))))
                           (fun scl _ items _ loc -> 
# 941 "vernac/g_vernac.mlg"
         let scl = List.map (CAst.make ~loc) scl in
       List.map (fun (name,recarg_like,notation_scope) ->
           RealArg { name=name; recarg_like=recarg_like;
                 notation_scope = notation_scope @ scl;
                 implicit_status = Explicit}) items 
                                                     );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("&")))))
                           (fun _ loc -> 
# 939 "vernac/g_vernac.mlg"
               [BidiArg] 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("/")))))
                           (fun _ loc -> 
# 938 "vernac/g_vernac.mlg"
               [VolatileArg] 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm argument_spec)))
                           (fun item loc -> 
# 934 "vernac/g_vernac.mlg"
        let name, recarg_like, notation_scope = item in
      [RealArg { name=name; recarg_like=recarg_like;
             notation_scope=notation_scope;
             implicit_status = Explicit}] 
                                            )])]))
  in let () = assert (Procq.Entry.is_empty implicits_alt) in
  let () =
  Procq.grammar_extend implicits_alt
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("{")))))
                              ((Procq.Symbol.list1 ((Procq.Symbol.nterm name)))))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                            (fun _ items _ loc -> 
# 966 "vernac/g_vernac.mlg"
         List.map (fun name -> (name.CAst.v, MaxImplicit)) items 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("[")))))
                             ((Procq.Symbol.list1 ((Procq.Symbol.nterm name)))))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("]")))))
                           (fun _ items _ loc -> 
# 964 "vernac/g_vernac.mlg"
         List.map (fun name -> (name.CAst.v, NonMaxImplicit)) items 
                                                 );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm name)))
                           (fun name loc -> 
# 962 "vernac/g_vernac.mlg"
                       [(name.CAst.v, Explicit)] 
                                            )])]))
  in let () = assert (Procq.Entry.is_empty instance_name) in
  let () =
  Procq.grammar_extend instance_name
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 972 "vernac/g_vernac.mlg"
             ((CAst.make ~loc Anonymous), None), []  
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm ident_decl)))
                            ((Procq.Symbol.nterm binders)))
                           (fun bl name loc -> 
# 971 "vernac/g_vernac.mlg"
            (CAst.map (fun id -> Name id) (fst name), snd name), bl 
                                               )])]))
  in let () = assert (Procq.Entry.is_empty hint_info) in
  let () =
  Procq.grammar_extend hint_info
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 977 "vernac/g_vernac.mlg"
             { Typeclasses.hint_priority = None; hint_pattern = None } 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("|")))))
                             ((Procq.Symbol.opt (Procq.Symbol.nterm natural))))
                            ((Procq.Symbol.opt (Procq.Symbol.nterm constr_pattern))))
                           (fun pat i _ loc -> 
# 976 "vernac/g_vernac.mlg"
           { Typeclasses.hint_priority = i; hint_pattern = pat } 
                                               )])]))
  in let () = assert (Procq.Entry.is_empty reserv_list) in
  let () =
  Procq.grammar_extend reserv_list
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm simple_reserv)))
                            (fun b loc -> 
# 980 "vernac/g_vernac.mlg"
                                                                   [b] 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm reserv_tuple)))))
                           (fun bl loc -> 
# 980 "vernac/g_vernac.mlg"
                                     bl 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty reserv_tuple) in
  let () =
  Procq.grammar_extend reserv_tuple
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                              ((Procq.Symbol.nterm simple_reserv)))
                             ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                            (fun _ a _ loc -> 
# 983 "vernac/g_vernac.mlg"
                                         a 
                                              )])]))
  in let () = assert (Procq.Entry.is_empty simple_reserv) in
  let () =
  Procq.grammar_extend simple_reserv
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.list1 ((Procq.Symbol.nterm identref)))))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                             ((Procq.Symbol.nterm lconstr)))
                            (fun c _ idl loc -> 
# 986 "vernac/g_vernac.mlg"
                                                    (idl,c) 
                                                )])]))
  in ()


# 991 "vernac/g_vernac.mlg"
 

(* Hack to parse "[ id" without dropping [ *)
let test_bracket_ident =
  let open Procq.Lookahead in
  to_entry "test_bracket_ident" begin
    lk_kw "[" >> lk_ident
  end



let _ =
  let range_selector = Procq.Entry.make "range_selector"
  and range_selector_or_nth = Procq.Entry.make "range_selector_or_nth"
  in
  let () = assert (Procq.Entry.is_empty range_selector) in
  let () =
  Procq.grammar_extend range_selector
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm natural)))
                            (fun n loc -> 
# 1006 "vernac/g_vernac.mlg"
                         (n, n) 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm natural)))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("-")))))
                            ((Procq.Symbol.nterm natural)))
                           (fun m _ n loc -> 
# 1005 "vernac/g_vernac.mlg"
                                             (n, m) 
                                             )])]))
  in let () = assert (Procq.Entry.is_empty range_selector_or_nth) in
  let () =
  Procq.grammar_extend range_selector_or_nth
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.nterm natural)))
                             ((Procq.Symbol.opt (Procq.Symbol.rules [Procq.Rules.make 
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD (",")))))
                                                                    ((Procq.Symbol.list1sep ((Procq.Symbol.nterm range_selector)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                                                                    (fun l _
                                                                    loc -> 
                                                                    
# 1015 "vernac/g_vernac.mlg"
                                                            l 
                                                                    )]))))
                            (fun l n loc -> 
# 1016 "vernac/g_vernac.mlg"
          let open Goal_select in
          Option.cata (fun l -> SelectList ((n, n) :: l)) (SelectNth n) l 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.nterm natural)))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("-")))))
                             ((Procq.Symbol.nterm natural)))
                            ((Procq.Symbol.opt (Procq.Symbol.rules [Procq.Rules.make 
                                                                   (Procq.Rule.next_norec
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD (",")))))
                                                                    ((Procq.Symbol.list1sep ((Procq.Symbol.nterm range_selector)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                                                                   (fun l _
                                                                   loc -> 
                                                                   
# 1012 "vernac/g_vernac.mlg"
                                                            l 
                                                                   )]))))
                           (fun l m _ n loc -> 
# 1013 "vernac/g_vernac.mlg"
          Goal_select.SelectList ((n, m) :: Option.default [] l) 
                                               )])]))
  in let () = assert (Procq.Entry.is_empty goal_selector) in
  let () =
  Procq.grammar_extend goal_selector
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.nterm test_bracket_ident)))
                               ((Procq.Symbol.token (Tok.PKEYWORD ("[")))))
                              ((Procq.Symbol.nterm ident)))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("]")))))
                            (fun _ id _ _ loc -> 
# 1021 "vernac/g_vernac.mlg"
                                                    Goal_select.SelectId id 
                                                 );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm range_selector_or_nth)))
                           (fun l loc -> 
# 1020 "vernac/g_vernac.mlg"
                                     l 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty toplevel_selector) in
  let () =
  Procq.grammar_extend toplevel_selector
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("all"))))))
                             ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                            (fun _ _ loc -> 
# 1026 "vernac/g_vernac.mlg"
                              Goal_select.SelectAll 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("!")))))
                            ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                           (fun _ _ loc -> 
# 1025 "vernac/g_vernac.mlg"
                      Goal_select.SelectAlreadyFocused 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm goal_selector)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                           (fun _ sel loc -> 
# 1024 "vernac/g_vernac.mlg"
                                      sel 
                                             )])]))
  in ()

let _ =
  let printable = Procq.Entry.make "printable"
  and debug_univ_name = Procq.Entry.make "debug_univ_name"
  and printunivs_subgraph = Procq.Entry.make "printunivs_subgraph"
  and locatable = Procq.Entry.make "locatable"
  and option_setting = Procq.Entry.make "option_setting"
  and table_value = Procq.Entry.make "table_value"
  and setting_name = Procq.Entry.make "setting_name"
  and ne_in_or_out_modules = Procq.Entry.make "ne_in_or_out_modules"
  and in_or_out_modules = Procq.Entry.make "in_or_out_modules"
  and comment = Procq.Entry.make "comment"
  and positive_search_mark = Procq.Entry.make "positive_search_mark"
  and search_item = Procq.Entry.make "search_item"
  and logical_kind = Procq.Entry.make "logical_kind"
  and extended_def_token = Procq.Entry.make "extended_def_token"
  and search_where = Procq.Entry.make "search_where"
  and univ_name_list = Procq.Entry.make "univ_name_list"
  in
  let () =
  Procq.grammar_extend gallina_ext
  (Procq.Reuse (None, [Procq.Production.make
                       (Procq.Rule.next
                        (Procq.Rule.next
                         (Procq.Rule.next (Procq.Rule.stop)
                          ((Procq.Symbol.token (Tok.PIDENT (Some
                          ("Export"))))))
                         ((Procq.Symbol.token (Tok.PIDENT (Some ("Unset"))))))
                        ((Procq.Symbol.nterm setting_name)))
                       (fun table _ _ loc -> 
# 1037 "vernac/g_vernac.mlg"
            VernacSynterp (VernacSetOption (true, table, OptionUnset)) 
                                             );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next
                         (Procq.Rule.next (Procq.Rule.stop)
                          ((Procq.Symbol.token (Tok.PIDENT (Some
                          ("Export"))))))
                         ((Procq.Symbol.token (Tok.PKEYWORD ("Set")))))
                        ((Procq.Symbol.nterm setting_name)))
                       ((Procq.Symbol.nterm option_setting)))
                      (fun v table _ _ loc -> 
# 1035 "vernac/g_vernac.mlg"
          VernacSynterp (VernacSetOption (true, table, v)) 
                                              )]))
  in let () = assert (Procq.Entry.is_empty command) in
  let () =
  Procq.grammar_extend command
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Remove"))))))
                              ((Procq.Symbol.token (Tok.PIDENT (None)))))
                             ((Procq.Symbol.list1 ((Procq.Symbol.nterm table_value)))))
                            (fun v table _ loc -> 
# 1109 "vernac/g_vernac.mlg"
            VernacSynPure (VernacRemoveOption ([table], v)) 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Remove"))))))
                              ((Procq.Symbol.token (Tok.PIDENT (None)))))
                             ((Procq.Symbol.token (Tok.PIDENT (None)))))
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm table_value)))))
                           (fun v field table _ loc -> 
# 1107 "vernac/g_vernac.mlg"
             VernacSynPure (VernacRemoveOption ([table;field], v)) 
                                                       );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Test"))))))
                            ((Procq.Symbol.nterm setting_name)))
                           (fun table _ loc -> 
# 1104 "vernac/g_vernac.mlg"
            VernacSynPure (VernacPrintOption table) 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Test"))))))
                              ((Procq.Symbol.nterm setting_name)))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("for")))))
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm table_value)))))
                           (fun v _ table _ loc -> 
# 1102 "vernac/g_vernac.mlg"
             VernacSynPure (VernacMemOption (table, v)) 
                                                   );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Add"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (None)))))
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm table_value)))))
                           (fun v table _ loc -> 
# 1099 "vernac/g_vernac.mlg"
            VernacSynPure (VernacAddOption ([table], v)) 
                                                 );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Add"))))))
                              ((Procq.Symbol.token (Tok.PIDENT (None)))))
                             ((Procq.Symbol.token (Tok.PIDENT (None)))))
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm table_value)))))
                           (fun v field table _ loc -> 
# 1093 "vernac/g_vernac.mlg"
             VernacSynPure (VernacAddOption ([table;field], v)) 
                                                       );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Print"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Table"))))))
                            ((Procq.Symbol.nterm setting_name)))
                           (fun table _ _ loc -> 
# 1090 "vernac/g_vernac.mlg"
            VernacSynPure (VernacPrintOption table) 
                                                 );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Unset"))))))
                            ((Procq.Symbol.nterm setting_name)))
                           (fun table _ loc -> 
# 1087 "vernac/g_vernac.mlg"
            VernacSynterp (VernacSetOption (false, table, OptionUnset)) 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("Set")))))
                             ((Procq.Symbol.nterm setting_name)))
                            ((Procq.Symbol.nterm option_setting)))
                           (fun v table _ loc -> 
# 1085 "vernac/g_vernac.mlg"
            VernacSynterp (VernacSetOption (false, table, v)) 
                                                 );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Inspect"))))))
                            ((Procq.Symbol.nterm natural)))
                           (fun n _ loc -> 
# 1081 "vernac/g_vernac.mlg"
                                          VernacSynPure (VernacPrint (PrintInspect n)) 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Print"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Namespace"))))))
                            ((Procq.Symbol.nterm dirpath)))
                           (fun ns _ _ loc -> 
# 1080 "vernac/g_vernac.mlg"
            VernacSynPure (VernacPrint (PrintNamespace ns)) 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Print"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Module"))))))
                            ((Procq.Symbol.nterm global)))
                           (fun qid _ _ loc -> 
# 1078 "vernac/g_vernac.mlg"
            VernacSynPure (VernacPrint (PrintModule qid)) 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Print"))))))
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Module"))))))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("Type")))))
                            ((Procq.Symbol.nterm global)))
                           (fun qid _ _ _ loc -> 
# 1076 "vernac/g_vernac.mlg"
            VernacSynPure (VernacPrint (PrintModuleType qid)) 
                                                 );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Print"))))))
                             ((Procq.Symbol.nterm smart_global)))
                            ((Procq.Symbol.opt (Procq.Symbol.nterm univ_name_list))))
                           (fun l qid _ loc -> 
# 1074 "vernac/g_vernac.mlg"
                                                                       VernacSynPure (VernacPrint (PrintName (qid,l))) 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Print"))))))
                            ((Procq.Symbol.nterm printable)))
                           (fun p _ loc -> 
# 1073 "vernac/g_vernac.mlg"
                                          VernacSynPure (VernacPrint p) 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("Type")))))
                            ((Procq.Symbol.nterm lconstr)))
                           (fun c _ loc -> 
# 1070 "vernac/g_vernac.mlg"
                                 VernacSynPure (VernacGlobalCheck c) 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Locate"))))))
                            ((Procq.Symbol.nterm locatable)))
                           (fun l _ loc -> 
# 1067 "vernac/g_vernac.mlg"
                                           VernacSynPure (VernacLocate l) 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Declare"))))))
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("ML"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Module"))))))
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm ne_string)))))
                           (fun l _ _ _ loc -> 
# 1065 "vernac/g_vernac.mlg"
            VernacSynterp (VernacDeclareMLModule l) 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Load"))))))
                             ((Procq.Symbol.rules [Procq.Rules.make (Procq.Rule.stop)
                                                                    (fun
                                                                    loc -> 
                                                                    
# 1061 "vernac/g_vernac.mlg"
                                                                       false 
                                                                    );
                                                  Procq.Rules.make (Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PIDENT (Some
                                                                    ("Verbose"))))))
                                                                   (fun _
                                                                   loc -> 
                                                                   
# 1061 "vernac/g_vernac.mlg"
                                                         true 
                                                                   )])))
                            ((Procq.Symbol.rules [Procq.Rules.make (Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PIDENT (None)))))
                                                                   (fun s
                                                                   loc -> 
                                                                   
# 1062 "vernac/g_vernac.mlg"
                                                      s 
                                                                   );
                                                 Procq.Rules.make (Procq.Rule.next_norec
                                                                   (Procq.Rule.stop)
                                                                   ((Procq.Symbol.nterm ne_string)))
                                                                  (fun s
                                                                  loc -> 
                                                                  
# 1062 "vernac/g_vernac.mlg"
                                 s 
                                                                  )])))
                           (fun s verbosely _ loc -> 
# 1063 "vernac/g_vernac.mlg"
            VernacSynterp (VernacLoad (verbosely, s)) 
                                                     );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Cd"))))))
                            ((Procq.Symbol.nterm ne_string)))
                           (fun dir _ loc -> 
# 1059 "vernac/g_vernac.mlg"
                                         VernacSynterp (VernacChdir (Some dir)) 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some ("Cd"))))))
                           (fun _ loc -> 
# 1058 "vernac/g_vernac.mlg"
                        warn_chdir_pwd (); VernacSynterp (VernacChdir None) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Pwd"))))))
                           (fun _ loc -> 
# 1057 "vernac/g_vernac.mlg"
                         VernacSynterp (VernacChdir None) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Declare"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Scope"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (None)))))
                           (fun sc _ _ loc -> 
# 1054 "vernac/g_vernac.mlg"
            VernacSynPure (VernacDeclareScope sc) 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next (Procq.Rule.stop)
                                  ((Procq.Symbol.token (Tok.PIDENT (Some
                                  ("Declare"))))))
                                 ((Procq.Symbol.token (Tok.PIDENT (Some
                                 ("Instance"))))))
                                ((Procq.Symbol.nterm ident_decl)))
                               ((Procq.Symbol.nterm binders)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                             ((Procq.Symbol.nterml term ("200"))))
                            ((Procq.Symbol.nterm hint_info)))
                           (fun info t _ bl id _ _ loc -> 
# 1050 "vernac/g_vernac.mlg"
             VernacSynPure (VernacDeclareInstance (id, bl, t, info)) 
                                                          );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Attributes"))))))
                            ((Procq.Symbol.nterm attribute_list)))
                           (fun attr _ loc -> 
# 1044 "vernac/g_vernac.mlg"
          VernacSynPure (VernacAttributes attr) 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Comments"))))))
                            ((Procq.Symbol.list0 (Procq.Symbol.nterm comment))))
                           (fun l _ loc -> 
# 1041 "vernac/g_vernac.mlg"
                                                 VernacSynPure (VernacComments l) 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty query_command) in
  let () =
  Procq.grammar_extend query_command
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                ("Search"))))))
                               ((Procq.Symbol.nterm search_query)))
                              ((Procq.Symbol.nterm search_queries)))
                             ((Procq.Symbol.token (Tok.PKEYWORD (".")))))
                            (fun _ l s _ loc -> 
# 1126 "vernac/g_vernac.mlg"
            let (sl,m) = l in fun g -> VernacSearch (Search (s::sl),g, m) 
                                                );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("SearchRewrite"))))))
                              ((Procq.Symbol.nterm constr_pattern)))
                             ((Procq.Symbol.nterm in_or_out_modules)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (".")))))
                           (fun _ l c _ loc -> 
# 1124 "vernac/g_vernac.mlg"
            fun g -> VernacSearch (SearchRewrite c,g, l) 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("SearchPattern"))))))
                              ((Procq.Symbol.nterm constr_pattern)))
                             ((Procq.Symbol.nterm in_or_out_modules)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (".")))))
                           (fun _ l c _ loc -> 
# 1122 "vernac/g_vernac.mlg"
            fun g -> VernacSearch (SearchPattern c,g, l) 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("About"))))))
                              ((Procq.Symbol.nterm smart_global)))
                             ((Procq.Symbol.opt (Procq.Symbol.nterm univ_name_list))))
                            ((Procq.Symbol.token (Tok.PKEYWORD (".")))))
                           (fun _ l qid _ loc -> 
# 1120 "vernac/g_vernac.mlg"
           fun g -> VernacPrint (PrintAbout (qid,l,g)) 
                                                 );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Check"))))))
                             ((Procq.Symbol.nterm lconstr)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (".")))))
                           (fun _ c _ loc -> 
# 1117 "vernac/g_vernac.mlg"
           fun g -> VernacCheckMayEval (None, g, c) 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Compute"))))))
                             ((Procq.Symbol.nterm lconstr)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (".")))))
                           (fun _ c _ loc -> 
# 1115 "vernac/g_vernac.mlg"
            fun g -> VernacCheckMayEval (Some (Genredexpr.CbvVm None), g, c) 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                ("Eval"))))))
                               ((Procq.Symbol.nterm red_expr)))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("in")))))
                             ((Procq.Symbol.nterm lconstr)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (".")))))
                           (fun _ c _ r _ loc -> 
# 1113 "vernac/g_vernac.mlg"
            fun g -> VernacCheckMayEval (Some r, g, c) 
                                                 )])]))
  in let () = assert (Procq.Entry.is_empty printable) in
  let () =
  Procq.grammar_extend printable
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Registered"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Schemes"))))))
                            (fun _ _ loc -> 
# 1186 "vernac/g_vernac.mlg"
                                                 PrintRegisteredSchemes 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Registered"))))))
                           (fun _ loc -> 
# 1185 "vernac/g_vernac.mlg"
                                PrintRegistered 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Strategies"))))))
                           (fun _ loc -> 
# 1184 "vernac/g_vernac.mlg"
                                PrintStrategy None 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Strategy"))))))
                            ((Procq.Symbol.nterm smart_global)))
                           (fun qid _ loc -> 
# 1183 "vernac/g_vernac.mlg"
                                                  PrintStrategy (Some qid) 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("All"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Dependencies"))))))
                            ((Procq.Symbol.nterm smart_global)))
                           (fun qid _ _ loc -> 
# 1182 "vernac/g_vernac.mlg"
                                                                   PrintAssumptions (true, true, qid) 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Transparent"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Dependencies"))))))
                            ((Procq.Symbol.nterm smart_global)))
                           (fun qid _ _ loc -> 
# 1181 "vernac/g_vernac.mlg"
                                                                           PrintAssumptions (false, true, qid) 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Opaque"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Dependencies"))))))
                            ((Procq.Symbol.nterm smart_global)))
                           (fun qid _ _ loc -> 
# 1180 "vernac/g_vernac.mlg"
                                                                      PrintAssumptions (true, false, qid) 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Assumptions"))))))
                            ((Procq.Symbol.nterm smart_global)))
                           (fun qid _ loc -> 
# 1179 "vernac/g_vernac.mlg"
                                                     PrintAssumptions (false, false, qid) 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.rules [Procq.Rules.make 
                                                     (Procq.Rule.stop) (fun
                                                     loc -> 
# 1172 "vernac/g_vernac.mlg"
                                                false 
                                                            );
                                                     Procq.Rules.make 
                                                     (Procq.Rule.next_norec
                                                      (Procq.Rule.stop)
                                                      ((Procq.Symbol.token (Tok.PIDENT (Some
                                                      ("Sorted"))))))
                                                     (fun _ loc -> 
# 1172 "vernac/g_vernac.mlg"
                                  true 
                                                                   )])))
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Universes"))))))
                              ((Procq.Symbol.opt (Procq.Symbol.nterm printunivs_subgraph))))
                             ((Procq.Symbol.opt (Procq.Symbol.rules [Procq.Rules.make 
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.rules 
                                                                    [Procq.Rules.make 
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PIDENT (Some
                                                                    ("Without"))))))
                                                                    (fun _
                                                                    loc -> 
                                                                    
# 1175 "vernac/g_vernac.mlg"
                                                                false 
                                                                    );
                                                                    Procq.Rules.make 
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PIDENT (Some
                                                                    ("With"))))))
                                                                    (fun _
                                                                    loc -> 
                                                                    
# 1175 "vernac/g_vernac.mlg"
                                  true 
                                                                    )])))
                                                                    ((Procq.Symbol.token (Tok.PIDENT (Some
                                                                    ("Constraint"))))))
                                                                    ((Procq.Symbol.token (Tok.PIDENT (Some
                                                                    ("Sources"))))))
                                                                    (fun _ _
                                                                    b loc ->
                                                                    
# 1176 "vernac/g_vernac.mlg"
                                                   b 
                                                                    )]))))
                            ((Procq.Symbol.opt (Procq.Symbol.nterm ne_string))))
                           (fun fopt with_sources g _ b loc -> 
# 1178 "vernac/g_vernac.mlg"
          PrintUniverses {sort=b; subgraph=g; with_sources; file=fopt;} 
                                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Implicit"))))))
                            ((Procq.Symbol.nterm smart_global)))
                           (fun qid _ loc -> 
# 1171 "vernac/g_vernac.mlg"
                                                  PrintImplicit qid 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Visibility"))))))
                            ((Procq.Symbol.opt (Procq.Symbol.token (Tok.PIDENT (None))))))
                           (fun s _ loc -> 
# 1170 "vernac/g_vernac.mlg"
                                               PrintVisibility s 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Scope"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (None)))))
                           (fun s _ loc -> 
# 1169 "vernac/g_vernac.mlg"
                                      PrintScope s 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Scopes"))))))
                           (fun _ loc -> 
# 1168 "vernac/g_vernac.mlg"
                            PrintScopes 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("HintDb"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (None)))))
                           (fun s _ loc -> 
# 1167 "vernac/g_vernac.mlg"
                                       PrintHintDbName s 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Hint"))))))
                            ((Procq.Symbol.token (Tok.PKEYWORD ("*")))))
                           (fun _ _ loc -> 
# 1166 "vernac/g_vernac.mlg"
                               PrintHintDb 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Hint"))))))
                            ((Procq.Symbol.nterm smart_global)))
                           (fun qid _ loc -> 
# 1165 "vernac/g_vernac.mlg"
                                              PrintHint qid 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Hint"))))))
                           (fun _ loc -> 
# 1164 "vernac/g_vernac.mlg"
                          PrintHintGoal 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Options"))))))
                           (fun _ loc -> 
# 1163 "vernac/g_vernac.mlg"
                             PrintTables (* A Synonymous to Tables *) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Tables"))))))
                           (fun _ loc -> 
# 1162 "vernac/g_vernac.mlg"
                            PrintTables 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Typing"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Flags"))))))
                           (fun _ _ loc -> 
# 1161 "vernac/g_vernac.mlg"
                                           PrintTypingFlags 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Canonical"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Projections"))))))
                            ((Procq.Symbol.list0 (Procq.Symbol.nterm smart_global))))
                           (fun qids _ _ loc -> 
# 1160 "vernac/g_vernac.mlg"
            PrintCanonicalConversions qids 
                                                );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Coercion"))))))
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Paths"))))))
                             ((Procq.Symbol.nterm coercion_class)))
                            ((Procq.Symbol.nterm coercion_class)))
                           (fun t s _ _ loc -> 
# 1158 "vernac/g_vernac.mlg"
            PrintCoercionPaths (s,t) 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Coercions"))))))
                           (fun _ loc -> 
# 1156 "vernac/g_vernac.mlg"
                               PrintCoercions 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Instances"))))))
                            ((Procq.Symbol.nterm smart_global)))
                           (fun qid _ loc -> 
# 1155 "vernac/g_vernac.mlg"
                                                   PrintInstances qid 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Typeclasses"))))))
                           (fun _ loc -> 
# 1154 "vernac/g_vernac.mlg"
                                 PrintTypeclasses 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Classes"))))))
                           (fun _ loc -> 
# 1153 "vernac/g_vernac.mlg"
                              PrintClasses 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Graph"))))))
                           (fun _ loc -> 
# 1152 "vernac/g_vernac.mlg"
                           PrintGraph 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Debug"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some ("GC"))))))
                           (fun _ _ loc -> 
# 1151 "vernac/g_vernac.mlg"
                                       PrintDebugGC 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("ML"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Modules"))))))
                           (fun _ _ loc -> 
# 1150 "vernac/g_vernac.mlg"
                                         PrintMLModules 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("ML"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Path"))))))
                           (fun _ _ loc -> 
# 1149 "vernac/g_vernac.mlg"
                                      PrintMLLoadPath 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                ("Notation"))))))
                               ((Procq.Symbol.nterm string)))
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("in"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("custom"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (None)))))
                           (fun ent _ _ ntn _ loc -> 
# 1147 "vernac/g_vernac.mlg"
          PrintNotation (Constrexpr.InCustomEntry ent, ntn) 
                                                     );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Notation"))))))
                            ((Procq.Symbol.nterm string)))
                           (fun ntn _ loc -> 
# 1145 "vernac/g_vernac.mlg"
          PrintNotation (Constrexpr.InConstrEntry, ntn) 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Libraries"))))))
                           (fun _ loc -> 
# 1142 "vernac/g_vernac.mlg"
                               PrintLibraries 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("LoadPath"))))))
                            ((Procq.Symbol.opt (Procq.Symbol.nterm dirpath))))
                           (fun dir _ loc -> 
# 1141 "vernac/g_vernac.mlg"
                                                 PrintLoadPath dir 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Keywords"))))))
                           (fun _ loc -> 
# 1140 "vernac/g_vernac.mlg"
            PrintKeywords 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Custom"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Grammar"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (None)))))
                           (fun ent _ _ loc -> 
# 1138 "vernac/g_vernac.mlg"
            PrintCustomGrammar ent 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Grammar"))))))
                            ((Procq.Symbol.list0 (Procq.Symbol.token (Tok.PIDENT (None))))))
                           (fun ents _ loc -> 
# 1135 "vernac/g_vernac.mlg"
            PrintGrammar ents 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Section"))))))
                            ((Procq.Symbol.nterm global)))
                           (fun s _ loc -> 
# 1132 "vernac/g_vernac.mlg"
                                         PrintSectionContext s 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("All"))))))
                           (fun _ loc -> 
# 1131 "vernac/g_vernac.mlg"
                         PrintFullContext 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Term"))))))
                             ((Procq.Symbol.nterm smart_global)))
                            ((Procq.Symbol.opt (Procq.Symbol.nterm univ_name_list))))
                           (fun l qid _ loc -> 
# 1130 "vernac/g_vernac.mlg"
                                                                      PrintName (qid,l) 
                                               )])]))
  in let () = assert (Procq.Entry.is_empty debug_univ_name) in
  let () =
  Procq.grammar_extend debug_univ_name
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PSTRING (None)))))
                            (fun x loc -> 
# 1191 "vernac/g_vernac.mlg"
                        RawUniv (CAst.make ~loc x) 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm reference)))
                           (fun x loc -> 
# 1190 "vernac/g_vernac.mlg"
                           NamedUniv x 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty printunivs_subgraph) in
  let () =
  Procq.grammar_extend printunivs_subgraph
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                ("Subgraph"))))))
                               ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                              ((Procq.Symbol.list0 (Procq.Symbol.nterm debug_univ_name))))
                             ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                            (fun _ l _ _ loc -> 
# 1195 "vernac/g_vernac.mlg"
                                                                   l 
                                                )])]))
  in let () = assert (Procq.Entry.is_empty coercion_class) in
  let () =
  Procq.grammar_extend coercion_class
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm smart_global)))
                            (fun qid loc -> 
# 1200 "vernac/g_vernac.mlg"
                                RefClass qid 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Sortclass"))))))
                           (fun _ loc -> 
# 1199 "vernac/g_vernac.mlg"
                               SortClass 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Funclass"))))))
                           (fun _ loc -> 
# 1198 "vernac/g_vernac.mlg"
                              FunClass 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty locatable) in
  let () =
  Procq.grammar_extend locatable
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Module"))))))
                             ((Procq.Symbol.nterm global)))
                            (fun qid _ loc -> 
# 1207 "vernac/g_vernac.mlg"
                                          LocateModule qid 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Library"))))))
                            ((Procq.Symbol.nterm global)))
                           (fun qid _ loc -> 
# 1206 "vernac/g_vernac.mlg"
                                           LocateLibrary qid 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("File"))))))
                            ((Procq.Symbol.nterm ne_string)))
                           (fun f _ loc -> 
# 1205 "vernac/g_vernac.mlg"
                                         LocateFile f 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Term"))))))
                            ((Procq.Symbol.nterm smart_global)))
                           (fun qid _ loc -> 
# 1204 "vernac/g_vernac.mlg"
                                              LocateTerm qid 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm smart_global)))
                           (fun qid loc -> 
# 1203 "vernac/g_vernac.mlg"
                                LocateAny qid 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty option_setting) in
  let () =
  Procq.grammar_extend option_setting
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PSTRING (None)))))
                            (fun s loc -> 
# 1212 "vernac/g_vernac.mlg"
                            OptionSetString s 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm integer)))
                           (fun n loc -> 
# 1211 "vernac/g_vernac.mlg"
                            OptionSetInt n 
                                         );
                           Procq.Production.make (Procq.Rule.stop)
                           (fun loc -> 
# 1210 "vernac/g_vernac.mlg"
             OptionSetTrue 
                                       )])]))
  in let () = assert (Procq.Entry.is_empty table_value) in
  let () =
  Procq.grammar_extend table_value
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PSTRING (None)))))
                            (fun s loc -> 
# 1216 "vernac/g_vernac.mlg"
                           Goptions.StringRefValue s 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm global)))
                           (fun id loc -> 
# 1215 "vernac/g_vernac.mlg"
                           Goptions.QualidRefValue id 
                                          )])]))
  in let () = assert (Procq.Entry.is_empty setting_name) in
  let () =
  Procq.grammar_extend setting_name
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.list1 ((Procq.Symbol.rules 
                             [Procq.Rules.make (Procq.Rule.next_norec
                                                (Procq.Rule.stop)
                                                ((Procq.Symbol.token (Tok.PIDENT (None)))))
                                               (fun x loc -> 
# 1219 "vernac/g_vernac.mlg"
                                    x 
                                                             )])))))
                            (fun fl loc -> 
# 1219 "vernac/g_vernac.mlg"
                                               fl 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty ne_in_or_out_modules) in
  let () =
  Procq.grammar_extend ne_in_or_out_modules
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("outside"))))))
                             ((Procq.Symbol.list1 ((Procq.Symbol.nterm global)))))
                            (fun l _ loc -> 
# 1224 "vernac/g_vernac.mlg"
                                               SearchOutside l 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("in")))))
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm global)))))
                           (fun l _ loc -> 
# 1223 "vernac/g_vernac.mlg"
                                    SearchInside l 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("inside"))))))
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm global)))))
                           (fun l _ loc -> 
# 1222 "vernac/g_vernac.mlg"
                                              SearchInside l 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty in_or_out_modules) in
  let () =
  Procq.grammar_extend in_or_out_modules
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 1228 "vernac/g_vernac.mlg"
             SearchOutside [] 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm ne_in_or_out_modules)))
                           (fun m loc -> 
# 1227 "vernac/g_vernac.mlg"
                                      m 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty comment) in
  let () =
  Procq.grammar_extend comment
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm natural)))
                            (fun n loc -> 
# 1233 "vernac/g_vernac.mlg"
                         CommentInt n 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PSTRING (None)))))
                           (fun s loc -> 
# 1232 "vernac/g_vernac.mlg"
                        CommentString s 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm constr)))
                           (fun c loc -> 
# 1231 "vernac/g_vernac.mlg"
                        CommentConstr c 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty positive_search_mark) in
  let () =
  Procq.grammar_extend positive_search_mark
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 1236 "vernac/g_vernac.mlg"
                                true 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PKEYWORD ("-")))))
                           (fun _ loc -> 
# 1236 "vernac/g_vernac.mlg"
                 false 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty search_query) in
  let () =
  Procq.grammar_extend search_query
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.nterm positive_search_mark)))
                               ((Procq.Symbol.token (Tok.PKEYWORD ("[")))))
                              ((Procq.Symbol.list1sep ((Procq.Symbol.list1 ((Procq.Symbol.nterm search_query)))) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD ("|"))])) false)))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("]")))))
                            (fun _ l _ b loc -> 
# 1240 "vernac/g_vernac.mlg"
                                                                                        (b, SearchDisjConj l) 
                                                );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm positive_search_mark)))
                            ((Procq.Symbol.nterm search_item)))
                           (fun s b loc -> 
# 1239 "vernac/g_vernac.mlg"
                                                       (b, SearchLiteral s) 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty search_item) in
  let () =
  Procq.grammar_extend search_item
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm constr_pattern)))
                            (fun p loc -> 
# 1253 "vernac/g_vernac.mlg"
              SearchSubPattern ((Anywhere,false),p) 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.nterm test_id_colon)))
                              ((Procq.Symbol.nterm search_where)))
                             ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                            ((Procq.Symbol.nterm constr_pattern)))
                           (fun p _ where _ loc -> 
# 1251 "vernac/g_vernac.mlg"
              SearchSubPattern (where,p) 
                                                   );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm ne_string)))
                            ((Procq.Symbol.opt (Procq.Symbol.nterm scope_delimiter))))
                           (fun sc s loc -> 
# 1249 "vernac/g_vernac.mlg"
              SearchString ((Anywhere,false),s,sc) 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("is"))))))
                             ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                            ((Procq.Symbol.nterm logical_kind)))
                           (fun kl _ _ loc -> 
# 1247 "vernac/g_vernac.mlg"
              SearchKind kl 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.nterm test_id_colon)))
                               ((Procq.Symbol.nterm search_where)))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                             ((Procq.Symbol.nterm ne_string)))
                            ((Procq.Symbol.opt (Procq.Symbol.nterm scope_delimiter))))
                           (fun sc s _ where _ loc -> 
# 1245 "vernac/g_vernac.mlg"
              SearchString (where,s,sc) 
                                                      )])]))
  in let () = assert (Procq.Entry.is_empty logical_kind) in
  let () =
  Procq.grammar_extend logical_kind
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Symbol"))))))
                            (fun _ loc -> 
# 1262 "vernac/g_vernac.mlg"
                            IsSymbol 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Primitive"))))))
                           (fun _ loc -> 
# 1261 "vernac/g_vernac.mlg"
                               IsPrimitive 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm extended_def_token)))
                           (fun k loc -> 
# 1260 "vernac/g_vernac.mlg"
                                    IsDefinition k 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Context"))))))
                           (fun k loc -> 
# 1259 "vernac/g_vernac.mlg"
                                 IsAssumption Context 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm assumption_token)))
                           (fun k loc -> 
# 1258 "vernac/g_vernac.mlg"
                                  IsAssumption (snd k) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm thm_token)))
                           (fun k loc -> 
# 1257 "vernac/g_vernac.mlg"
                           IsProof k 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty extended_def_token) in
  let () =
  Procq.grammar_extend extended_def_token
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Method"))))))
                            (fun _ loc -> 
# 1273 "vernac/g_vernac.mlg"
                            Method 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Field"))))))
                           (fun _ loc -> 
# 1272 "vernac/g_vernac.mlg"
                           StructureComponent 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Canonical"))))))
                           (fun _ loc -> 
# 1271 "vernac/g_vernac.mlg"
                               CanonicalStructure 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Scheme"))))))
                           (fun _ loc -> 
# 1270 "vernac/g_vernac.mlg"
                            Scheme 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Instance"))))))
                           (fun _ loc -> 
# 1269 "vernac/g_vernac.mlg"
                              Instance 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("CoFixpoint"))))))
                           (fun _ loc -> 
# 1268 "vernac/g_vernac.mlg"
                                CoFixpoint 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Fixpoint"))))))
                           (fun _ loc -> 
# 1267 "vernac/g_vernac.mlg"
                              Fixpoint 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Coercion"))))))
                           (fun _ loc -> 
# 1266 "vernac/g_vernac.mlg"
                              Coercion 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm def_token)))
                           (fun k loc -> 
# 1265 "vernac/g_vernac.mlg"
                           snd k 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty search_where) in
  let () =
  Procq.grammar_extend search_where
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("headconcl"))))))
                            (fun _ loc -> 
# 1280 "vernac/g_vernac.mlg"
                               InConcl, true 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("headhyp"))))))
                           (fun _ loc -> 
# 1279 "vernac/g_vernac.mlg"
                             InHyp, true 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("concl"))))))
                           (fun _ loc -> 
# 1278 "vernac/g_vernac.mlg"
                           InConcl, false 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("hyp"))))))
                           (fun _ loc -> 
# 1277 "vernac/g_vernac.mlg"
                         InHyp, false 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("head"))))))
                           (fun _ loc -> 
# 1276 "vernac/g_vernac.mlg"
                          Anywhere, true 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty search_queries) in
  let () =
  Procq.grammar_extend search_queries
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 1286 "vernac/g_vernac.mlg"
             ([],SearchOutside []) 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm search_query)))
                            ((Procq.Symbol.nterm search_queries)))
                           (fun l s loc -> 
# 1285 "vernac/g_vernac.mlg"
          let (sl,m) = l in (s::sl,m) 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm ne_in_or_out_modules)))
                           (fun m loc -> 
# 1283 "vernac/g_vernac.mlg"
                                      ([],m) 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty univ_name_list) in
  let () =
  Procq.grammar_extend univ_name_list
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("@{")))))
                              ((Procq.Symbol.list0 (Procq.Symbol.nterm name))))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("}")))))
                            (fun _ l _ loc -> 
# 1290 "vernac/g_vernac.mlg"
                                         [],l 
                                              )])]))
  in ()

let _ = let () =
  Procq.grammar_extend command
  (Procq.Reuse (None, [Procq.Production.make
                       (Procq.Rule.next
                        (Procq.Rule.next
                         (Procq.Rule.next
                          (Procq.Rule.next (Procq.Rule.stop)
                           ((Procq.Symbol.token (Tok.PIDENT (Some
                           ("Declare"))))))
                          ((Procq.Symbol.token (Tok.PIDENT (Some
                          ("Custom"))))))
                         ((Procq.Symbol.token (Tok.PIDENT (Some ("Entry"))))))
                        ((Procq.Symbol.token (Tok.PIDENT (None)))))
                       (fun s _ _ _ loc -> 
# 1321 "vernac/g_vernac.mlg"
             VernacSynterp (VernacDeclareCustomEntry s) 
                                           );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next
                        (Procq.Rule.next
                         (Procq.Rule.next
                          (Procq.Rule.next (Procq.Rule.stop)
                           ((Procq.Symbol.token (Tok.PIDENT (Some
                           ("Declare"))))))
                          ((Procq.Symbol.token (Tok.PIDENT (Some
                          ("Reduction"))))))
                         ((Procq.Symbol.token (Tok.PIDENT (None)))))
                        ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                       ((Procq.Symbol.nterm red_expr)))
                      (fun r _ s _ _ loc -> 
# 1316 "vernac/g_vernac.mlg"
             VernacSynPure (VernacDeclareReduction (s,r)) 
                                            );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Debug"))))))
                       ((Procq.Symbol.token (Tok.PIDENT (Some ("Off"))))))
                      (fun _ _ loc -> 
# 1310 "vernac/g_vernac.mlg"
            VernacSynterp (VernacSetOption (false, ["Ltac";"Debug"], OptionUnset)) 
                                      );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Debug"))))))
                       ((Procq.Symbol.token (Tok.PIDENT (Some ("On"))))))
                      (fun _ _ loc -> 
# 1307 "vernac/g_vernac.mlg"
            VernacSynterp (VernacSetOption (false, ["Ltac";"Debug"], OptionSetTrue)) 
                                      );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Back"))))))
                       ((Procq.Symbol.nterm natural)))
                      (fun n _ loc -> 
# 1303 "vernac/g_vernac.mlg"
                                       VernacSynPure (VernacBack n) 
                                      );
                      Procq.Production.make
                      (Procq.Rule.next (Procq.Rule.stop)
                       ((Procq.Symbol.token (Tok.PIDENT (Some ("Back"))))))
                      (fun _ loc -> 
# 1302 "vernac/g_vernac.mlg"
                          VernacSynPure (VernacBack 1) 
                                    );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Reset"))))))
                       ((Procq.Symbol.nterm identref)))
                      (fun id _ loc -> 
# 1301 "vernac/g_vernac.mlg"
                                          VernacSynPure (VernacResetName id) 
                                       );
                      Procq.Production.make
                      (Procq.Rule.next
                       (Procq.Rule.next (Procq.Rule.stop)
                        ((Procq.Symbol.token (Tok.PIDENT (Some ("Reset"))))))
                       ((Procq.Symbol.token (Tok.PIDENT (Some ("Initial"))))))
                      (fun _ _ loc -> 
# 1300 "vernac/g_vernac.mlg"
                                            VernacSynPure (VernacResetInitial) 
                                      )]))
  in ()

let _ =
  let enable_enable_disable = Procq.Entry.make "enable_enable_disable"
  and enable_notation_rule = Procq.Entry.make "enable_notation_rule"
  and enable_notation_interpretation =
    Procq.Entry.make "enable_notation_interpretation"
  and enable_notation_flags = Procq.Entry.make "enable_notation_flags"
  and enable_notation_flag = Procq.Entry.make "enable_notation_flag"
  and opt_scope = Procq.Entry.make "opt_scope"
  and level = Procq.Entry.make "level"
  and syntax_modifier = Procq.Entry.make "syntax_modifier"
  and explicit_subentry = Procq.Entry.make "explicit_subentry"
  and at_level_opt = Procq.Entry.make "at_level_opt"
  and binder_interp = Procq.Entry.make "binder_interp"
  in
  let () = assert (Procq.Entry.is_empty syntax) in
  let () =
  Procq.grammar_extend syntax
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next
                                 (Procq.Rule.next (Procq.Rule.stop)
                                  ((Procq.Symbol.nterm enable_enable_disable)))
                                 ((Procq.Symbol.token (Tok.PIDENT (Some
                                 ("Notation"))))))
                                ((Procq.Symbol.nterm enable_notation_rule)))
                               ((Procq.Symbol.nterm enable_notation_interpretation)))
                              ((Procq.Symbol.nterm enable_notation_flags)))
                             ((Procq.Symbol.nterm opt_scope)))
                            (fun scope flags interp rule _ on loc -> 
# 1362 "vernac/g_vernac.mlg"
             VernacSynPure (VernacEnableNotation (on, rule, interp, flags, scope)) 
                                                                    );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Reserved"))))))
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Notation"))))))
                             ((Procq.Symbol.nterm ne_lstring)))
                            ((Procq.Symbol.nterm syntax_modifiers)))
                           (fun l s _ _ loc -> 
# 1357 "vernac/g_vernac.mlg"
             VernacSynterp (VernacReservedNotation (false,(s,l))) 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Reserved"))))))
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Infix"))))))
                             ((Procq.Symbol.nterm ne_lstring)))
                            ((Procq.Symbol.nterm syntax_modifiers)))
                           (fun l s _ _ loc -> 
# 1355 "vernac/g_vernac.mlg"
             VernacSynterp (VernacReservedNotation (true,(s,l))) 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Notation"))))))
                            ((Procq.Symbol.nterm notation_declaration)))
                           (fun ntn_decl _ loc -> 
# 1352 "vernac/g_vernac.mlg"
             VernacSynterp (VernacNotation (false,ntn_decl)) 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.token (Tok.PIDENT (Some
                                 ("Notation"))))))
                                ((Procq.Symbol.nterm identref)))
                               ((Procq.Symbol.list0 (Procq.Symbol.nterm ident))))
                              ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                             ((Procq.Symbol.nterm constr)))
                            ((Procq.Symbol.nterm syntax_modifiers)))
                           (fun modl c _ idl id _ loc -> 
# 1350 "vernac/g_vernac.mlg"
             VernacSynPure (VernacSyntacticDefinition (id,(idl,c), modl)) 
                                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Infix"))))))
                            ((Procq.Symbol.nterm notation_declaration)))
                           (fun ntn_decl _ loc -> 
# 1347 "vernac/g_vernac.mlg"
           VernacSynterp (VernacNotation (true,ntn_decl)) 
                                                  );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                ("Bind"))))))
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Scope"))))))
                              ((Procq.Symbol.token (Tok.PIDENT (None)))))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("with")))))
                            ((Procq.Symbol.list1 ((Procq.Symbol.nterm coercion_class)))))
                           (fun refl _ sc _ _ loc -> 
# 1344 "vernac/g_vernac.mlg"
                                        VernacSynPure (VernacBindScope (sc,refl)) 
                                                     );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Undelimit"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Scope"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (None)))))
                           (fun sc _ _ loc -> 
# 1341 "vernac/g_vernac.mlg"
           VernacSynPure(VernacDelimiters (sc, None)) 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                ("Delimit"))))))
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("Scope"))))))
                              ((Procq.Symbol.token (Tok.PIDENT (None)))))
                             ((Procq.Symbol.token (Tok.PKEYWORD ("with")))))
                            ((Procq.Symbol.token (Tok.PIDENT (None)))))
                           (fun key _ sc _ _ loc -> 
# 1339 "vernac/g_vernac.mlg"
           VernacSynPure(VernacDelimiters (sc, Some key)) 
                                                    );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Close"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Scope"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (None)))))
                           (fun sc _ _ loc -> 
# 1336 "vernac/g_vernac.mlg"
           VernacSynPure (VernacOpenCloseScope (false,sc)) 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("Open"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Scope"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (None)))))
                           (fun sc _ _ loc -> 
# 1333 "vernac/g_vernac.mlg"
           VernacSynPure (VernacOpenCloseScope (true,sc)) 
                                              )])]))
  in let () = assert (Procq.Entry.is_empty enable_enable_disable) in
  let () =
  Procq.grammar_extend enable_enable_disable
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("Disable"))))))
                            (fun _ loc -> 
# 1370 "vernac/g_vernac.mlg"
                             false 
                                          );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("Enable"))))))
                           (fun _ loc -> 
# 1369 "vernac/g_vernac.mlg"
                            true 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty enable_notation_rule) in
  let () =
  Procq.grammar_extend enable_notation_rule
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 1375 "vernac/g_vernac.mlg"
             None 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.nterm global)))
                            ((Procq.Symbol.list0 (Procq.Symbol.nterm ident))))
                           (fun idl qid loc -> 
# 1374 "vernac/g_vernac.mlg"
                                             Some (Inr (idl,qid)) 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.nterm ne_string)))
                           (fun s loc -> 
# 1373 "vernac/g_vernac.mlg"
                           Some (Inl s) 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty enable_notation_interpretation)
  in
  let () =
  Procq.grammar_extend enable_notation_interpretation
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 1379 "vernac/g_vernac.mlg"
             None 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD (":=")))))
                            ((Procq.Symbol.nterm constr)))
                           (fun c _ loc -> 
# 1378 "vernac/g_vernac.mlg"
                              Some c 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty enable_notation_flags) in
  let () =
  Procq.grammar_extend enable_notation_flags
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 1383 "vernac/g_vernac.mlg"
             [] 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                             ((Procq.Symbol.list1sep ((Procq.Symbol.nterm enable_notation_flag)) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                           (fun _ l _ loc -> 
# 1382 "vernac/g_vernac.mlg"
                                                              l 
                                             )])]))
  in let () = assert (Procq.Entry.is_empty enable_notation_flag) in
  let () =
  Procq.grammar_extend enable_notation_flag
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("in")))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("constr"))))))
                            (fun _ _ loc -> 
# 1390 "vernac/g_vernac.mlg"
                                  EnableNotationEntry (CAst.make ~loc InConstrEntry) 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("in")))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("custom"))))))
                            ((Procq.Symbol.nterm identref)))
                           (fun x _ _ loc -> 
# 1389 "vernac/g_vernac.mlg"
                                                EnableNotationEntry CAst.(make ?loc:x.loc (InCustomEntry (Id.to_string x.v))) 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("only"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("printing"))))))
                           (fun _ _ loc -> 
# 1388 "vernac/g_vernac.mlg"
                                            EnableNotationOnly Notationextern.OnlyPrinting 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("only"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("parsing"))))))
                           (fun _ _ loc -> 
# 1387 "vernac/g_vernac.mlg"
                                           EnableNotationOnly Notationextern.OnlyParsing 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("all"))))))
                           (fun _ loc -> 
# 1386 "vernac/g_vernac.mlg"
                         EnableNotationAll 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty opt_scope) in
  let () =
  Procq.grammar_extend opt_scope
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 1395 "vernac/g_vernac.mlg"
             None 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("no"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("scope"))))))
                           (fun _ _ _ loc -> 
# 1394 "vernac/g_vernac.mlg"
                                            Some LastLonelyNotation 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD (":")))))
                            ((Procq.Symbol.token (Tok.PIDENT (None)))))
                           (fun sc _ loc -> 
# 1393 "vernac/g_vernac.mlg"
                             Some (NotationInScope sc) 
                                            )])]))
  in let () = assert (Procq.Entry.is_empty level) in
  let () =
  Procq.grammar_extend level
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("next"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("level"))))))
                            (fun _ _ loc -> 
# 1399 "vernac/g_vernac.mlg"
                                         NextLevel 
                                            );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("level"))))))
                            ((Procq.Symbol.nterm natural)))
                           (fun n _ loc -> 
# 1398 "vernac/g_vernac.mlg"
                                        NumLevel n 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty syntax_modifier) in
  let () =
  Procq.grammar_extend syntax_modifier
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (None)))))
                             ((Procq.Symbol.nterm explicit_subentry)))
                            (fun typ x loc -> 
# 1419 "vernac/g_vernac.mlg"
                                                SetEntryType (x,typ) 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (None)))))
                            ((Procq.Symbol.nterm binder_interp)))
                           (fun b x loc -> 
# 1418 "vernac/g_vernac.mlg"
                                          SetItemLevel ([x],Some b,DefaultLevel) 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (None)))))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("in")))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("scope"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (None)))))
                           (fun k _ _ x loc -> 
# 1417 "vernac/g_vernac.mlg"
                                                       SetItemScope([x],k) 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (None)))))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("at")))))
                             ((Procq.Symbol.nterm level)))
                            ((Procq.Symbol.opt (Procq.Symbol.nterm binder_interp))))
                           (fun b lev _ x loc -> 
# 1416 "vernac/g_vernac.mlg"
          SetItemLevel ([x],b,lev) 
                                                 );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (None)))))
                              ((Procq.Symbol.token (Tok.PKEYWORD (",")))))
                             ((Procq.Symbol.list1sep ((Procq.Symbol.token (Tok.PIDENT (None)))) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                            ((Procq.Symbol.rules [Procq.Rules.make (Procq.Rule.next_norec
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (
                                                                    Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("in")))))
                                                                    ((Procq.Symbol.token (Tok.PIDENT (Some
                                                                    ("scope"))))))
                                                                    ((Procq.Symbol.token (Tok.PIDENT (None)))))
                                                                   (fun k _ _
                                                                   loc -> 
                                                                   
# 1414 "vernac/g_vernac.mlg"
                                                fun x l -> SetItemScope(x::l,k) 
                                                                   );
                                                 Procq.Rules.make (Procq.Rule.next_norec
                                                                   (Procq.Rule.next_norec
                                                                    (Procq.Rule.stop)
                                                                    ((Procq.Symbol.token (Tok.PKEYWORD ("at")))))
                                                                   ((Procq.Symbol.nterm level)))
                                                                  (fun lev _
                                                                  loc -> 
                                                                  
# 1413 "vernac/g_vernac.mlg"
                                   fun x l -> SetItemLevel (x::l,None,lev) 
                                                                  )])))
                           (fun v l _ x loc -> 
# 1414 "vernac/g_vernac.mlg"
                                                                                         v x l 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("format"))))))
                            ((Procq.Symbol.nterm lstring)))
                           (fun s _ loc -> 
# 1411 "vernac/g_vernac.mlg"
                                         SetFormat (TextFormat s) 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("only"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("parsing"))))))
                           (fun _ _ loc -> 
# 1410 "vernac/g_vernac.mlg"
                                           SetOnlyParsing 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("only"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("printing"))))))
                           (fun _ _ loc -> 
# 1409 "vernac/g_vernac.mlg"
                                            SetOnlyPrinting 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("no"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("associativity"))))))
                           (fun _ _ loc -> 
# 1408 "vernac/g_vernac.mlg"
                                               SetAssoc Gramlib.Gramext.NonA 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("right"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("associativity"))))))
                           (fun _ _ loc -> 
# 1407 "vernac/g_vernac.mlg"
                                                  SetAssoc Gramlib.Gramext.RightA 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("left"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("associativity"))))))
                           (fun _ _ loc -> 
# 1406 "vernac/g_vernac.mlg"
                                                 SetAssoc Gramlib.Gramext.LeftA 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next
                                (Procq.Rule.next (Procq.Rule.stop)
                                 ((Procq.Symbol.token (Tok.PKEYWORD ("in")))))
                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                ("custom"))))))
                               ((Procq.Symbol.token (Tok.PIDENT (None)))))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("at")))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("level"))))))
                            ((Procq.Symbol.nterm natural)))
                           (fun n _ _ x _ _ loc -> 
# 1405 "vernac/g_vernac.mlg"
           SetCustomEntry (x,Some n) 
                                                   );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("in")))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("custom"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (None)))))
                           (fun x _ _ loc -> 
# 1403 "vernac/g_vernac.mlg"
                                             SetCustomEntry (x,None) 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("at")))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("level"))))))
                            ((Procq.Symbol.nterm natural)))
                           (fun n _ _ loc -> 
# 1402 "vernac/g_vernac.mlg"
                                              SetLevel n 
                                             )])]))
  in let () = assert (Procq.Entry.is_empty syntax_modifiers) in
  let () =
  Procq.grammar_extend syntax_modifiers
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 1424 "vernac/g_vernac.mlg"
             [] 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PKEYWORD ("(")))))
                             ((Procq.Symbol.list1sep ((Procq.Symbol.rules 
                             [Procq.Rules.make (Procq.Rule.next_norec
                                                (Procq.Rule.stop)
                                                ((Procq.Symbol.nterm syntax_modifier)))
                                               (fun s loc -> 
# 1423 "vernac/g_vernac.mlg"
                                                  CAst.make ~loc s 
                                                             )])) ((Procq.Symbol.tokens [Procq.TPattern (Tok.PKEYWORD (","))])) false)))
                            ((Procq.Symbol.token (Tok.PKEYWORD (")")))))
                           (fun _ l _ loc -> 
# 1423 "vernac/g_vernac.mlg"
                                                                                         l 
                                             )])]))
  in let () = assert (Procq.Entry.is_empty explicit_subentry) in
  let () =
  Procq.grammar_extend explicit_subentry
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                ("custom"))))))
                               ((Procq.Symbol.token (Tok.PIDENT (None)))))
                              ((Procq.Symbol.nterm at_level_opt)))
                             ((Procq.Symbol.opt (Procq.Symbol.nterm binder_interp))))
                            (fun b n x _ loc -> 
# 1441 "vernac/g_vernac.mlg"
             ETConstr (InCustomEntry x,b,n) 
                                                );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("closed"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("binder"))))))
                           (fun _ _ loc -> 
# 1439 "vernac/g_vernac.mlg"
                                            ETBinder false 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next
                               (Procq.Rule.next (Procq.Rule.stop)
                                ((Procq.Symbol.token (Tok.PIDENT (Some
                                ("strict"))))))
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("pattern"))))))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("at")))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("level"))))))
                            ((Procq.Symbol.nterm natural)))
                           (fun n _ _ _ _ loc -> 
# 1438 "vernac/g_vernac.mlg"
                                                                               ETPattern (true,Some n) 
                                                 );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("strict"))))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("pattern"))))))
                           (fun _ _ loc -> 
# 1437 "vernac/g_vernac.mlg"
                                             ETPattern (true,None) 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PIDENT (Some
                               ("pattern"))))))
                              ((Procq.Symbol.token (Tok.PKEYWORD ("at")))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("level"))))))
                            ((Procq.Symbol.nterm natural)))
                           (fun n _ _ _ loc -> 
# 1436 "vernac/g_vernac.mlg"
                                                               ETPattern (false,Some n) 
                                               );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("pattern"))))))
                           (fun _ loc -> 
# 1435 "vernac/g_vernac.mlg"
                             ETPattern (false,None) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next
                             (Procq.Rule.next (Procq.Rule.stop)
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("constr"))))))
                             ((Procq.Symbol.nterm at_level_opt)))
                            ((Procq.Symbol.opt (Procq.Symbol.nterm binder_interp))))
                           (fun b n _ loc -> 
# 1434 "vernac/g_vernac.mlg"
                                                                     ETConstr (InConstrEntry,b,n) 
                                             );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("constr"))))))
                           (fun _ loc -> 
# 1433 "vernac/g_vernac.mlg"
                            ETConstr (InConstrEntry,None,DefaultLevel) 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("binder"))))))
                           (fun _ loc -> 
# 1432 "vernac/g_vernac.mlg"
                            ETBinder true 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("bigint"))))))
                           (fun _ loc -> 
# 1431 "vernac/g_vernac.mlg"
                            ETBigint 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("global"))))))
                           (fun _ loc -> 
# 1430 "vernac/g_vernac.mlg"
                            ETGlobal 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("name"))))))
                           (fun _ loc -> 
# 1429 "vernac/g_vernac.mlg"
                          ETName 
                                         );
                           Procq.Production.make
                           (Procq.Rule.next (Procq.Rule.stop)
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("ident"))))))
                           (fun _ loc -> 
# 1428 "vernac/g_vernac.mlg"
                           ETIdent 
                                         )])]))
  in let () = assert (Procq.Entry.is_empty at_level_opt) in
  let () =
  Procq.grammar_extend at_level_opt
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make (Procq.Rule.stop)
                            (fun loc -> 
# 1446 "vernac/g_vernac.mlg"
             DefaultLevel 
                                        );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("at")))))
                            ((Procq.Symbol.nterm level)))
                           (fun n _ loc -> 
# 1445 "vernac/g_vernac.mlg"
                             n 
                                           )])]))
  in let () = assert (Procq.Entry.is_empty binder_interp) in
  let () =
  Procq.grammar_extend binder_interp
  (Procq.Fresh
  (Gramlib.Gramext.First, [(None, None,
                           [Procq.Production.make
                            (Procq.Rule.next
                             (Procq.Rule.next
                              (Procq.Rule.next (Procq.Rule.stop)
                               ((Procq.Symbol.token (Tok.PKEYWORD ("as")))))
                              ((Procq.Symbol.token (Tok.PIDENT (Some
                              ("strict"))))))
                             ((Procq.Symbol.token (Tok.PIDENT (Some
                             ("pattern"))))))
                            (fun _ _ _ loc -> 
# 1452 "vernac/g_vernac.mlg"
                                                   Notation_term.AsStrictPattern 
                                              );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("as")))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("pattern"))))))
                           (fun _ _ loc -> 
# 1451 "vernac/g_vernac.mlg"
                                   Notation_term.AsAnyPattern 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("as")))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("name"))))))
                           (fun _ _ loc -> 
# 1450 "vernac/g_vernac.mlg"
                                Notation_term.AsName 
                                           );
                           Procq.Production.make
                           (Procq.Rule.next
                            (Procq.Rule.next (Procq.Rule.stop)
                             ((Procq.Symbol.token (Tok.PKEYWORD ("as")))))
                            ((Procq.Symbol.token (Tok.PIDENT (Some
                            ("ident"))))))
                           (fun _ _ loc -> 
# 1449 "vernac/g_vernac.mlg"
                                 warn_deprecated_as_ident_kind (); Notation_term.AsIdent 
                                           )])]))
  in ()

